@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -85,7 +85,7 @@ anything. */
 
 /* Regular expression include */
 
-#include <pcre.h>
+#include "pcre/pcre.h"
 
 /* Includes from the main source of Exim. We need to have MAXPACKET defined for
 the benefit of structs.h. One of these days I should tidy up this interface so
@@ -93,14 +93,13 @@ that this kind of kludge isn't needed. */
 
 #define MAXPACKET 1024
 
-#include "config.h"
 #include "mytypes.h"
 #include "macros.h"
+#include "config.h"
 
 #include "local_scan.h"
 #include "structs.h"
 #include "globals.h"
-#include "dbstuff.h"
 #include "functions.h"
 #include "osfunctions.h"
 #include "store.h"
