@@ -2,10 +2,10 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2014 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
-/* This module contains code for scanning the main log,
+/* This module contains code for scanning the smaill log,
 extracting information from it, and displaying a "tail". */
 
 #include "em_hdr.h"
@@ -56,8 +56,6 @@ static int scrolled = FALSE;
 static int size = 0;
 static int top = 0;
 
-static void show_log(char *s, ...) PRINTF_FUNCTION(1,2);
-
 static void show_log(char *s, ...)
 {
 int length, newtop;
@@ -250,20 +248,14 @@ if (LOG != NULL)
       }
 
     /* Munge the log entry and display shortened form on one line.
-    We omit the date and show only the time. Remove any time zone offset.
-    Take note of the presence of [pid]. */
+    We omit the date and show only the time. Remove any time zone offset. */
 
     if (pcre_exec(yyyymmdd_regex,NULL,CS buffer,length,0,PCRE_EOPT,NULL,0) >= 0)
       {
-      int pidlength = 0;
       if ((buffer[20] == '+' || buffer[20] == '-') &&
           isdigit(buffer[21]) && buffer[25] == ' ')
         memmove(buffer + 20, buffer + 26, Ustrlen(buffer + 26) + 1);
-      if (buffer[20] == '[')
-        {
-        while (Ustrchr("[]0123456789", buffer[20+pidlength++]) != NULL);
-        }
-      id = string_copyn(buffer + 20 + pidlength, MESSAGE_ID_LENGTH);
+      id = string_copyn(buffer + 20, MESSAGE_ID_LENGTH);
       show_log("%s", buffer+11);
       }
     else
@@ -364,8 +356,6 @@ link count of zero on the currently open file. */
 if (log_datestamping)
   {
   uschar log_file_wanted[256];
-  /* Do *not* use "%s" here, we need the %D datestamp in the log_file to
-   *   be expanded! */
   string_format(log_file_wanted, sizeof(log_file_wanted), CS log_file);
   if (Ustrcmp(log_file_wanted, log_file_open) != 0)
     {
