@@ -2,7 +2,7 @@
 *                  Exim Monitor                  *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -166,7 +166,7 @@ Returns:    nothing
 */
 
 void
-log_write(unsigned int selector, int flags, const char *format, ...)
+log_write(unsigned int selector, int flags, char *format, ...)
 {
 va_list ap;
 va_start(ap, format);
@@ -198,7 +198,7 @@ Returns:     0 if there is no port, else the port number.
 */
 
 int
-host_address_extract_port(uschar *address)
+host_extract_port(uschar *address)
 {
 int skip = -3;                     /* Skip 3 dots in IPv4 addresses */
 address--;
@@ -613,6 +613,7 @@ message_subdir[1] = 0;
 constructing file names and things. This call will initialize
 the store_get() function. */
 
+big_buffer_size = 1024;
 big_buffer = store_get(big_buffer_size);
 
 /* Set up the version string and date and output them */
@@ -642,8 +643,8 @@ stripchart_init();
 only, and we can't tail the log. If not, open the log file and position to the
 end of it. Before doing so, we have to detect whether the log files are
 datestamped, and if so, sort out the name. The string in log_file already has
-%s replaced by "main"; if datestamping is occurring, %D or %M will be present.
-In fact, we don't need to test explicitly - just process the string with
+%s replaced by "main"; if datestamping is occurring, %D will be present. In
+fact, we don't need to test explicitly - just process the string with
 string_format.
 
 Once opened, save the file's inode so that we can detect when the file is
@@ -653,8 +654,6 @@ today.) */
 
 if (log_file[0] != 0)
   {
-  /* Do *not* use "%s" here, we need the %D datestamp in the log_file to
-  be expanded! */
   (void)string_format(log_file_open, sizeof(log_file_open), CS log_file);
   log_datestamping = string_datestamp_offset >= 0;
 
@@ -669,14 +668,8 @@ if (log_file[0] != 0)
     {
     fseek(LOG, 0, SEEK_END);
     log_position = ftell(LOG);
-    if (fstat(fileno(LOG), &statdata))
-      {
-      perror("log file fstat");
-      fclose(LOG);
-      LOG=NULL;
-      }
-    else
-      log_inode = statdata.st_ino;
+    fstat(fileno(LOG), &statdata);
+    log_inode = statdata.st_ino;
     }
   }
 else
