@@ -2,7 +2,7 @@
 *                 Exim Monitor                   *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -122,29 +122,6 @@ store_free(p);
 }
 
 
-/*************************************************
-*         Set up an ACL variable                 *
-*************************************************/
-
-/* The spool_read_header() function calls acl_var_create() when it reads in an
-ACL variable. We know that in this case, the variable will be new, not re-used,
-so this is a cut-down version, to save including the whole acl.c module (which
-would need conditional compilation to cut most of it out). */
-
-tree_node *
-acl_var_create(uschar *name)
-{
-tree_node *node, **root;
-root = (name[0] == 'c')? &acl_var_c : &acl_var_m;
-node = store_get(sizeof(tree_node) + Ustrlen(name));
-Ustrcpy(node->name, name);
-node->data.ptr = NULL;
-(void)tree_insertnode(root, node);
-return node;
-}
-
-
-
 /*************************************************
 *             Set up new queue item              *
 *************************************************/
@@ -518,7 +495,7 @@ for (i = 0; i < subdir_max; i++)
         name[SPOOL_NAME_LENGTH - 2] == '-' &&
         name[SPOOL_NAME_LENGTH - 1] == 'H')
       {
-      uschar basename[SPOOL_NAME_LENGTH + 1];
+      uschar basename[SPOOL_NAME_LENGTH];
       stripchart_total[0]++;
       if (!eximon_initialized) { printf("."); fflush(stdout); }
       Ustrcpy(basename, name);
