@@ -48,18 +48,18 @@
  *
 int main (int argc, char ** argv)
 {
-       SPAAuthRequest   request;
+       SPAAuthRequest	request;
        SPAAuthChallenge challenge;
-       SPAAuthResponse  response;
+       SPAAuthResponse	response;
        char msgbuf[2048];
        char buffer[512];
        char *username, *password, *domain, *challenge_str;
 
        if (argc < 3)
        {
-               printf ("Usage: %s <username> <password> [SPA Challenge]\n",
-                       argv [0]);
-               exit (1);
+	       printf ("Usage: %s <username> <password> [SPA Challenge]\n",
+		       argv [0]);
+	       exit (1);
        }
 
        username = argv [1];
@@ -69,34 +69,27 @@ int main (int argc, char ** argv)
        spa_build_auth_request (&request, username, domain);
 
        spa_bits_to_base64 (msgbuf, (unsigned char*)&request,
-               spa_request_length(&request));
+	       spa_request_length(&request));
 
        printf ("SPA Login request for username=%s:\n   %s\n",
-               argv [1], msgbuf);
+	       argv [1], msgbuf);
 
        if (argc < 4)
        {
-               printf ("Run: %s <username> <password> [NTLM Challenge] " \
-                       "to complete authenitcation\n", argv [0]);
-               exit (0);
+	       printf ("Run: %s <username> <password> [NTLM Challenge] " \
+		       "to complete authenitcation\n", argv [0]);
+	       exit (0);
        }
 
        challenge_str = argv [3];
 
-       if (spa_base64_to_bits ((char *)&challenge, sizeof(challenge),
-                (const char *)(challenge_str))<0)
-       {
-                printf("bad base64 data in challenge: %s\n", challenge_str);
-                exit (1);
-       }
-
        spa_build_auth_response (&challenge, &response, username, password);
        spa_bits_to_base64 (msgbuf, (unsigned char*)&response,
-               spa_request_length(&response));
+	       spa_request_length(&response));
 
        printf ("SPA Response to challenge:\n   %s\n for " \
-               "username=%s, password=%s:\n   %s\n",
-               argv[3], argv [1], argv [2], msgbuf);
+	       "username=%s, password=%s:\n   %s\n",
+	       argv[3], argv [1], argv [2], msgbuf);
        return 0;
 }
  *
@@ -181,27 +174,27 @@ typedef int BOOL;
 
 #define RW_PCVAL(read,inbuf,outbuf,len) \
        { if (read) { PCVAL (inbuf,0,outbuf,len); } \
-       else      { PSCVAL(inbuf,0,outbuf,len); } }
+       else	 { PSCVAL(inbuf,0,outbuf,len); } }
 
 #define RW_PIVAL(read,big_endian,inbuf,outbuf,len) \
        { if (read) { if (big_endian) { RPIVAL(inbuf,0,outbuf,len); } else { PIVAL(inbuf,0,outbuf,len); } } \
-       else      { if (big_endian) { RPSIVAL(inbuf,0,outbuf,len); } else { PSIVAL(inbuf,0,outbuf,len); } } }
+       else	 { if (big_endian) { RPSIVAL(inbuf,0,outbuf,len); } else { PSIVAL(inbuf,0,outbuf,len); } } }
 
 #define RW_PSVAL(read,big_endian,inbuf,outbuf,len) \
        { if (read) { if (big_endian) { RPSVAL(inbuf,0,outbuf,len); } else { PSVAL(inbuf,0,outbuf,len); } } \
-       else      { if (big_endian) { RPSSVAL(inbuf,0,outbuf,len); } else { PSSVAL(inbuf,0,outbuf,len); } } }
+       else	 { if (big_endian) { RPSSVAL(inbuf,0,outbuf,len); } else { PSSVAL(inbuf,0,outbuf,len); } } }
 
 #define RW_CVAL(read, inbuf, outbuf, offset) \
        { if (read) { (outbuf) = CVAL (inbuf,offset); } \
-       else      { SCVAL(inbuf,offset,outbuf); } }
+       else	 { SCVAL(inbuf,offset,outbuf); } }
 
 #define RW_IVAL(read, big_endian, inbuf, outbuf, offset) \
        { if (read) { (outbuf) = ((big_endian) ? RIVAL(inbuf,offset) : IVAL (inbuf,offset)); } \
-       else      { if (big_endian) { RSIVAL(inbuf,offset,outbuf); } else { SIVAL(inbuf,offset,outbuf); } } }
+       else	 { if (big_endian) { RSIVAL(inbuf,offset,outbuf); } else { SIVAL(inbuf,offset,outbuf); } } }
 
 #define RW_SVAL(read, big_endian, inbuf, outbuf, offset) \
        { if (read) { (outbuf) = ((big_endian) ? RSVAL(inbuf,offset) : SVAL (inbuf,offset)); } \
-       else      { if (big_endian) { RSSVAL(inbuf,offset,outbuf); } else { SSVAL(inbuf,offset,outbuf); } } }
+       else	 { if (big_endian) { RSSVAL(inbuf,offset,outbuf); } else { SSVAL(inbuf,offset,outbuf); } } }
 
 #undef CAREFUL_ALIGNMENT
 
@@ -368,7 +361,7 @@ void mdfour (unsigned char *out, unsigned char *in, int n);
 static const char base64digits[] =
   "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
-#define BAD    (char) -1
+#define BAD    -1
 static const char base64val[] = {
   BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD, BAD,
     BAD,
@@ -411,11 +404,8 @@ spa_bits_to_base64 (unsigned char *out, const unsigned char *in, int inlen)
   *out = '\0';
 }
 
-
-/* The outlength parameter was added by PH, December 2004 */
-
 int
-spa_base64_to_bits (char *out, int outlength, const char *in)
+spa_base64_to_bits (char *out, const char *in)
 /* base 64 to raw bytes in quasi-big-endian order, returning count of bytes */
 {
   int len = 0;
@@ -428,8 +418,6 @@ spa_base64_to_bits (char *out, int outlength, const char *in)
 
   do
     {
-      if (len >= outlength)                   /* Added by PH */
-        return (-1);                          /* Added by PH */
       digit1 = in[0];
       if (DECODE64 (digit1) == BAD)
        return (-1);
@@ -447,18 +435,14 @@ spa_base64_to_bits (char *out, int outlength, const char *in)
       ++len;
       if (digit3 != '=')
        {
-         if (len >= outlength)                   /* Added by PH */
-           return (-1);                          /* Added by PH */
-         *out++ =
-           ((DECODE64 (digit2) << 4) & 0xf0) | (DECODE64 (digit3) >> 2);
-         ++len;
-         if (digit4 != '=')
-           {
-             if (len >= outlength)                   /* Added by PH */
-               return (-1);                          /* Added by PH */
-             *out++ = ((DECODE64 (digit3) << 6) & 0xc0) | DECODE64 (digit4);
-             ++len;
-           }
+	 *out++ =
+	   ((DECODE64 (digit2) << 4) & 0xf0) | (DECODE64 (digit3) >> 2);
+	 ++len;
+	 if (digit4 != '=')
+	   {
+	     *out++ = ((DECODE64 (digit3) << 6) & 0xc0) | DECODE64 (digit4);
+	     ++len;
+	   }
        }
     }
   while (*in && *in != '\r' && digit4 != '=');
@@ -663,22 +647,22 @@ dohash (char *out, char *in, char *key, int forw)
 
       for (j = 0; j < 8; j++)
        for (k = 0; k < 6; k++)
-         b[j][k] = erk[j * 6 + k];
+	 b[j][k] = erk[j * 6 + k];
 
       for (j = 0; j < 8; j++)
        {
-         int m, n;
-         m = (b[j][0] << 1) | b[j][5];
+	 int m, n;
+	 m = (b[j][0] << 1) | b[j][5];
 
-         n = (b[j][1] << 3) | (b[j][2] << 2) | (b[j][3] << 1) | b[j][4];
+	 n = (b[j][1] << 3) | (b[j][2] << 2) | (b[j][3] << 1) | b[j][4];
 
-         for (k = 0; k < 4; k++)
-           b[j][k] = (sbox[j][m][n] & (1 << (3 - k))) ? 1 : 0;
+	 for (k = 0; k < 4; k++)
+	   b[j][k] = (sbox[j][m][n] & (1 << (3 - k))) ? 1 : 0;
        }
 
       for (j = 0; j < 8; j++)
        for (k = 0; k < 4; k++)
-         cb[j * 4 + k] = b[j][k];
+	 cb[j * 4 + k] = b[j][k];
       permute (pcb, cb, perm5, 32);
 
       xor (r2, l, pcb, 32);
@@ -829,7 +813,7 @@ safe_strcpy (char *dest, const char *src, size_t maxlength)
   if (len > maxlength)
     {
       DEBUG (0, ("ERROR: string overflow by %d in safe_strcpy [%.50s]\n",
-                (int) (len - maxlength), src));
+		(int) (len - maxlength), src));
       len = maxlength;
     }
 
@@ -847,13 +831,13 @@ strupper (char *s)
       {
        size_t skip = skip_multibyte_char (*s);
        if (skip != 0)
-         s += skip;
+	 s += skip;
        else
-         {
-           if (islower ((unsigned char)(*s)))
-             *s = toupper (*s);
-           s++;
-         }
+	 {
+	   if (islower ((unsigned char)(*s)))
+	     *s = toupper (*s);
+	   s++;
+	 }
       }
     }
 }
@@ -938,7 +922,7 @@ E_md4hash (uchar * passwd, uchar * p16)
     len = 128;
   /* Password must be converted to NT unicode */
   _my_mbstowcs (wpwd, passwd, len);
-  wpwd[len] = 0;               /* Ensure string is null terminated */
+  wpwd[len] = 0;	       /* Ensure string is null terminated */
   /* Calculate length in bytes */
   len = _my_wcslen (wpwd) * sizeof (int16x);
 
@@ -1236,7 +1220,7 @@ char versionString[] = "libntlm version 0.21";
 
 #define spa_bytes_add(ptr, header, buf, count) \
 { \
-if (buf != NULL  &&  count) \
+if (buf && count) \
   { \
   SSVAL(&ptr->header.len,0,count); \
   SSVAL(&ptr->header.maxlen,0,count); \
@@ -1451,8 +1435,8 @@ involves the "d" and "domain" variables. */
 #ifdef NEVER
 void
 spa_build_auth_response (SPAAuthChallenge * challenge,
-                        SPAAuthResponse * response, char *user,
-                        char *password)
+			SPAAuthResponse * response, char *user,
+			char *password)
 {
   uint8x lmRespData[24];
   uint8x ntRespData[24];
@@ -1493,8 +1477,8 @@ spa_build_auth_response (SPAAuthChallenge * challenge,
 
 void
 spa_build_auth_response (SPAAuthChallenge * challenge,
-                        SPAAuthResponse * response, char *user,
-                        char *password)
+			SPAAuthResponse * response, char *user,
+			char *password)
 {
   uint8x lmRespData[24];
   uint8x ntRespData[24];
