@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -153,8 +153,8 @@ auth_cram_md5_server(auth_instance *ablock, uschar *data)
 {
 auth_cram_md5_options_block *ob =
   (auth_cram_md5_options_block *)(ablock->options_block);
-uschar *challenge = string_sprintf("<%d.%ld@%s>", getpid(),
-    (long int) time(NULL), primary_hostname);
+uschar *challenge = string_sprintf("<%d.%d@%s>", getpid(), time(NULL),
+  primary_hostname);
 uschar *clear, *secret;
 uschar digest[16];
 int i, rc, len;
@@ -171,15 +171,14 @@ if (*data != 0) return UNEXPECTED;
 
 /* Send the challenge, read the return */
 
-if ((rc = auth_get_data(&data, challenge, Ustrlen(challenge))) != OK) return rc;
-if ((len = b64decode(data, &clear)) < 0) return BAD64;
+if ((rc = auth_get_data(&data, challenge)) != OK) return rc;
+if ((len = auth_b64decode(data, &clear)) < 0) return BAD64;
 
 /* The return consists of a user name, space-separated from the CRAM-MD5
-digest, expressed in hex. Extract the user name and put it in $auth1 and $1.
-The former is now the preferred variable; the latter is the original one. Then
-check that the remaining length is 32. */
+digest, expressed in hex. Extract the user name and put it in $1. Then check
+that the remaining length is 32. */
 
-auth_vars[0] = expand_nstring[1] = clear;
+expand_nstring[1] = clear;
 while (*clear != 0 && !isspace(*clear)) clear++;
 if (!isspace(*clear)) return FAIL;
 *clear++ = 0;
@@ -211,7 +210,7 @@ compute_cram_md5(secret, challenge, digest);
 HDEBUG(D_auth)
   {
   uschar buff[64];
-  debug_printf("CRAM-MD5: user name = %s\n", auth_vars[0]);
+  debug_printf("CRAM-MD5: user name = %s\n", expand_nstring[1]);
   debug_printf("          challenge = %s\n", challenge);
   debug_printf("          received  = %s\n", clear);
   Ustrcpy(buff,"          digest    = ");
@@ -231,8 +230,7 @@ for (i = 0; i < 16; i++)
         ((b >= 'a')? b - 'a' + 10 : b - '0')) != digest[i]) return FAIL;
   }
 
-/* Expand server_condition as an authorization check */
-return auth_check_serv_cond(ablock);
+return OK;
 }
 
 
@@ -249,7 +247,7 @@ auth_cram_md5_client(
   smtp_inblock *inblock,                 /* input connection */
   smtp_outblock *outblock,               /* output connection */
   int timeout,                           /* command timeout */
-  uschar *buffer,                        /* for reading response */
+  uschar *buffer,                          /* for reading response */
   int buffsize)                          /* size of buffer */
 {
 auth_cram_md5_options_block *ob =
@@ -263,16 +261,12 @@ uschar digest[16];
 /* If expansion of either the secret or the user name failed, return CANCELLED
 or ERROR, as approriate. */
 
-if (!secret || !name)
+if (secret == NULL || name == NULL)
   {
-  if (expand_string_forcedfail)
-    {
-    *buffer = 0;           /* No message */
-    return CANCELLED;
-    }
+  if (expand_string_forcedfail) return CANCELLED;
   string_format(buffer, buffsize, "expansion of \"%s\" failed in "
     "%s authenticator: %s",
-    !secret ? ob->client_secret : ob->client_name,
+    (secret == NULL)? ob->client_secret : ob->client_name,
     ablock->name, expand_string_message);
   return ERROR;
   }
@@ -280,12 +274,12 @@ if (!secret || !name)
 /* Initiate the authentication exchange and read the challenge, which arrives
 in base 64. */
 
-if (smtp_write_command(outblock, FALSE, "AUTH %s\r\n", ablock->public_name) < 0)
+if (!smtp_write_command(outblock, FALSE, "AUTH %s\r\n", ablock->public_name))
   return FAIL_SEND;
-if (!smtp_read_response(inblock, buffer, buffsize, '3', timeout))
+if (!smtp_read_response(inblock, (uschar *)buffer, buffsize, '3', timeout))
   return FAIL;
 
-if (b64decode(buffer + 4, &challenge) < 0)
+if (auth_b64decode(buffer + 4, &challenge) < 0)
   {
   string_format(buffer, buffsize, "bad base 64 string in challenge: %s",
     big_buffer + 4);
@@ -299,7 +293,8 @@ compute_cram_md5(secret, challenge, digest);
 /* Create the response from the user name plus the CRAM-MD5 digest */
 
 string_format(big_buffer, big_buffer_size - 36, "%s", name);
-for (p = big_buffer; *p; ) p++;
+p = big_buffer;
+while (*p != 0) p++;
 *p++ = ' ';
 
 for (i = 0; i < 16; i++)
@@ -309,15 +304,15 @@ for (i = 0; i < 16; i++)
   }
 
 /* Send the response, in base 64, and check the result. The response is
-in big_buffer, but b64encode() returns its result in working store,
+in big_buffer, but auth_b64encode() returns its result in working store,
 so calling smtp_write_command(), which uses big_buffer, is OK. */
 
 buffer[0] = 0;
-if (smtp_write_command(outblock, FALSE, "%s\r\n", b64encode(big_buffer,
-  p - big_buffer)) < 0) return FAIL_SEND;
+if (!smtp_write_command(outblock, FALSE, "%s\r\n", auth_b64encode(big_buffer,
+  p - big_buffer))) return FAIL_SEND;
 
-return smtp_read_response(inblock, (uschar *)buffer, buffsize, '2', timeout)
-  ? OK : FAIL;
+return smtp_read_response(inblock, (uschar *)buffer, buffsize, '2', timeout)?
+  OK : FAIL;
 }
 #endif  /* STAND_ALONE */
 
