@@ -2,15 +2,15 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
 
 typedef struct {
+  uschar *server_condition;
   uschar *server_prompts;
   uschar *client_send;
-  BOOL    client_ignore_invalid_base64;
 } auth_plaintext_options_block;
 
 /* Data for reading the private options. */
