@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -38,19 +38,12 @@ arrange to hold the locks for the bare minimum of time. */
 
 /* For Berkeley DB >= 2, we can define a function to be called in case of DB
 errors. This should help with debugging strange DB problems, e.g. getting "File
-exists" when you try to open a db file. The API for this function was changed
-at DB release 4.3. */
+exists" when you try to open a db file. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
 void
-#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-dbfn_bdb_error_callback(const DB_ENV *dbenv, const char *pfx, const char *msg)
-{
-dbenv = dbenv;
-#else
 dbfn_bdb_error_callback(const char *pfx, char *msg)
 {
-#endif
 pfx = pfx;
 log_write(0, LOG_MAIN, "Berkeley DB error: %s", msg);
 }
@@ -78,10 +71,6 @@ Returns:   NULL if the open failed, or the locking failed. After locking
 
            On success, dbblock is returned. This contains the dbm pointer and
            the fd of the locked lock file.
-
-There are some calls that use O_RDWR|O_CREAT for the flags. Having discovered
-this in December 2005, I'm not sure if this is correct or not, but for the
-moment I haven't changed them.
 */
 
 open_db *
@@ -140,10 +129,10 @@ alarm(0);
 if (sigalrm_seen) errno = ETIMEDOUT;
 if (rc < 0)
   {
-  log_write(0, LOG_MAIN|LOG_PANIC, "Failed to get %s lock for %s: %s",
-    read_only? "read" : "write", buffer,
+  log_write(0, LOG_MAIN, "Failed to get %s lock for %s: %s",
+    ((flags & O_RDONLY) != 0)? "read" : "write", buffer,
     (errno == ETIMEDOUT)? "timed out" : strerror(errno));
-  (void)close(dbblock->lockfd);
+  close(dbblock->lockfd);
   errno = 0;       /* Indicates locking failure */
   return NULL;
   }
@@ -154,15 +143,10 @@ DEBUG(D_hints_lookup) debug_printf("locked %s\n", buffer);
 exclusive access to the database, so we can go ahead and open it. If we are
 expected to create it, don't do so at first, again so that we can detect
 whether we need to change its ownership (see comments about the lock file
-above.) There have been regular reports of crashes while opening hints
-databases - often this is caused by non-matching db.h and the library. To make
-it easy to pin this down, there are now debug statements on either side of the
-open call. */
+above.) */
 
 sprintf(CS buffer, "%s/db/%s", spool_directory, name);
-DEBUG(D_hints_lookup) debug_printf("EXIM_DBOPEN(%s)\n", buffer);
 EXIM_DBOPEN(buffer, flags, EXIMDB_MODE, &(dbblock->dbptr));
-DEBUG(D_hints_lookup) debug_printf("returned from EXIM_DBOPEN\n");
 
 if (dbblock->dbptr == NULL && errno == ENOENT && flags == O_RDWR)
   {
@@ -170,7 +154,6 @@ if (dbblock->dbptr == NULL && errno == ENOENT && flags == O_RDWR)
     debug_printf("%s appears not to exist: trying to create\n", buffer);
   created = TRUE;
   EXIM_DBOPEN(buffer, flags|O_CREAT, EXIMDB_MODE, &(dbblock->dbptr));
-  DEBUG(D_hints_lookup) debug_printf("returned from EXIM_DBOPEN\n");
   }
 
 save_errno = errno;
@@ -208,7 +191,7 @@ if (created && geteuid() == root_uid)
       if (Ustat(buffer, &statbuf) >= 0 && statbuf.st_uid != exim_uid)
         {
         DEBUG(D_hints_lookup) debug_printf("ensuring %s is owned by exim\n", buffer);
-        (void)Uchown(buffer, exim_uid, exim_gid);
+        Uchown(buffer, exim_uid, exim_gid);
         }
       }
     }
@@ -228,18 +211,15 @@ if (dbblock->dbptr == NULL)
         buffer));
     else
       DEBUG(D_hints_lookup)
-        debug_printf("%s", CS string_open_failed(save_errno, "DB file %s\n",
-          buffer));
+        debug_printf(CS string_open_failed(save_errno, "DB file %s\n", buffer));
     }
-  (void)close(dbblock->lockfd);
+  close(dbblock->lockfd);
   errno = save_errno;
   return NULL;
   }
 
 DEBUG(D_hints_lookup)
-  debug_printf("opened hints database %s: flags=%s\n", buffer,
-    (flags == O_RDONLY)? "O_RDONLY" : (flags == O_RDWR)? "O_RDWR" :
-    (flags == (O_RDWR|O_CREAT))? "O_RDWR|O_CREAT" : "??");
+  debug_printf("opened hints database %s: flags=%x\n", buffer, flags);
 
 /* Pass back the block containing the opened database handle and the open fd
 for the lock. */
@@ -265,7 +245,7 @@ void
 dbfn_close(open_db *dbblock)
 {
 EXIM_DBCLOSE(dbblock->dbptr);
-(void)close(dbblock->lockfd);
+close(dbblock->lockfd);
 }
 
 
