@@ -2,13 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 #include "exim.h"
 
-static uschar  debug_buffer[2048];
+static uschar  debug_buffer[1024];
 static uschar *debug_ptr = debug_buffer;
 static int     debug_prefix_length = 0;
 
@@ -146,7 +146,7 @@ get interleaved. Since some calls to debug_printf() don't end with newline,
 we save up the text until we do get the newline. */
 
 void
-debug_printf(const char *format, ...)
+debug_printf(char *format, ...)
 {
 va_list ap;
 va_start(ap, format);
@@ -155,23 +155,18 @@ va_end(ap);
 }
 
 void
-debug_vprintf(const char *format, va_list ap)
+debug_vprintf(char *format, va_list ap)
 {
 if (debug_file == NULL) return;
 
-/* Various things can be inserted at the start of a line. Don't use the
-tod_stamp() function for the timestamp, because that will overwrite the
-timestamp buffer, which may contain something useful. (This was a bug fix: the
-+memory debugging with +timestamp did cause a problem.) */
+/* Various things can be inserted at the start of a line. */
 
 if (debug_ptr == debug_buffer)
   {
   DEBUG(D_timestamp)
     {
-    time_t now = time(NULL);
-    struct tm *t = timestamps_utc? gmtime(&now) : localtime(&now);
-    (void) sprintf(CS debug_ptr, "%02d:%02d:%02d ", t->tm_hour, t->tm_min,
-      t->tm_sec);
+    uschar *ts = tod_stamp(tod_log_bare);
+    sprintf(CS debug_ptr, "%s ", ts + 11);
     while(*debug_ptr != 0) debug_ptr++;
     }
 
