@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -24,20 +24,13 @@ different.
 
 Input lines beginning with # are ignored, as are blank lines. Entries begin
 with a key terminated by a colon or end of line or whitespace and continue with
-indented lines. Keys may be quoted if they contain colons or whitespace or #
-characters. */
+indented lines. */
 
 
 #include "exim.h"
 
 
-#define max_insize   20000
-#define max_outsize 100000
-
-/* This is global because it's defined in the headers and compilers grumble
-if it is made static. */
-
-const uschar *hex_digits = CUS"0123456789abcdef";
+#define maxsize 10000
 
 
 #ifdef STRERROR_FROM_ERRLIST
@@ -56,78 +49,18 @@ return sys_errlist[n];
 
 /* For Berkeley DB >= 2, we can define a function to be called in case of DB
 errors. This should help with debugging strange DB problems, e.g. getting "File
-exists" when you try to open a db file. The API changed at release 4.3. */
+exists" when you try to open a db file. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
 void
-#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-dbfn_bdb_error_callback(const DB_ENV *dbenv, const char *pfx, const char *msg)
-{
-dbenv = dbenv;
-#else
 dbfn_bdb_error_callback(const char *pfx, char *msg)
 {
-#endif
 pfx = pfx;
 printf("Berkeley DB error: %s\n", msg);
 }
 #endif
 
 
-
-/*************************************************
-*          Interpret escape sequence             *
-*************************************************/
-
-/* This function is copied from the main Exim code.
-
-Arguments:
-  pp       points a pointer to the initiating "\" in the string;
-           the pointer gets updated to point to the final character
-Returns:   the value of the character escape
-*/
-
-int
-string_interpret_escape(const uschar **pp)
-{
-int ch;
-const uschar *p = *pp;
-ch = *(++p);
-if (isdigit(ch) && ch != '8' && ch != '9')
-  {
-  ch -= '0';
-  if (isdigit(p[1]) && p[1] != '8' && p[1] != '9')
-    {
-    ch = ch * 8 + *(++p) - '0';
-    if (isdigit(p[1]) && p[1] != '8' && p[1] != '9')
-      ch = ch * 8 + *(++p) - '0';
-    }
-  }
-else switch(ch)
-  {
-  case 'n':  ch = '\n'; break;
-  case 'r':  ch = '\r'; break;
-  case 't':  ch = '\t'; break;
-  case 'x':
-  ch = 0;
-  if (isxdigit(p[1]))
-    {
-    ch = ch * 16 +
-      Ustrchr(hex_digits, tolower(*(++p))) - hex_digits;
-    if (isxdigit(p[1])) ch = ch * 16 +
-      Ustrchr(hex_digits, tolower(*(++p))) - hex_digits;
-    }
-  break;
-  }
-*pp = p;
-return ch;
-}
-
-
-/*************************************************
-*               Main Program                     *
-*************************************************/
-
 int main(int argc, char **argv)
 {
 int started;
@@ -149,10 +82,10 @@ EXIM_DB *d;
 EXIM_DATUM key, content;
 uschar *bptr;
 uschar  keybuffer[256];
-uschar  temp_dbmname[512];
-uschar  real_dbmname[512];
-uschar *buffer = malloc(max_outsize);
-uschar *line = malloc(max_insize);
+uschar  temp_dbmname[256];
+uschar  real_dbmname[256];
+uschar *buffer = malloc(maxsize);
+uschar *line = malloc(maxsize);
 
 while (argc > 1)
   {
@@ -193,15 +126,6 @@ if (Ustrcmp(argv[arg], argv[arg+1]) == 0)
   }
 #endif
 
-/* Check length of filename; allow for adding .dbmbuild_temp and .db or
-.dir/.pag later. */
-
-if (strlen(argv[arg+1]) > sizeof(temp_dbmname) - 20)
-  {
-  printf("exim_dbmbuild: output filename is ridiculously long\n");
-  exit(1);
-  }
-
 Ustrcpy(temp_dbmname, argv[arg+1]);
 Ustrcat(temp_dbmname, ".dbmbuild_temp");
 
@@ -214,7 +138,7 @@ if (d == NULL)
   {
   printf("exim_dbmbuild: unable to create %s: %s\n", temp_dbmname,
     strerror(errno));
-  (void)fclose(f);
+  fclose(f);
   exit(1);
   }
 
@@ -231,21 +155,12 @@ is_db = Ustat(real_dbmname, &statbuf) == 0;
 bptr = buffer;
 started = 0;
 
-while (Ufgets(line, max_insize, f) != NULL)
+while (Ufgets(line, maxsize, f) != NULL)
   {
   uschar *p;
-  int len = Ustrlen(line);
-
-  p = line + len;
-
-  if (len >= max_insize - 1 && p[-1] != '\n')
-    {
-    printf("Overlong line read: max permitted length is %d\n", max_insize - 1);
-    yield = 2;
-    goto TIDYUP;
-    }
 
   if (line[0] == '#') continue;
+  p = line + Ustrlen(line);
   while (p > line && isspace(p[-1])) p--;
   *p = 0;
   if (line[0] == 0) continue;
@@ -263,15 +178,6 @@ while (Ufgets(line, max_insize, f) != NULL)
       }
     while (isspace(*s)) s++;
     *(--s) = ' ';
-
-    if (bptr - buffer + p - s >= max_outsize - 1)
-      {
-      printf("Continued set of lines is too long: max permitted length is %d\n",
-        max_outsize -1);
-      yield = 2;
-      goto TIDYUP;
-      }
-
     Ustrcpy(bptr, s);
     bptr += p - s;
     }
@@ -285,7 +191,6 @@ while (Ufgets(line, max_insize, f) != NULL)
     {
     int i, rc;
     uschar *s = line;
-    uschar *keystart;
 
     if (started)
       {
@@ -317,31 +222,10 @@ while (Ufgets(line, max_insize, f) != NULL)
       bptr = buffer;
       }
 
+    while (*s != 0 && *s != ':' && !isspace(*s)) s++;
     EXIM_DATUM_INIT(key);
     EXIM_DATUM_DATA(key) = CS keybuffer;
-
-    /* Deal with quoted keys. Escape sequences always make one character
-    out of several, so we can re-build in place. */
-
-    if (*s == '\"')
-      {
-      uschar *t = s++;
-      keystart = t;
-      while (*s != 0 && *s != '\"')
-        {
-        if (*s == '\\') *t++ = string_interpret_escape((const uschar **)&s);
-          else *t++ = *s;
-        s++;
-        }
-      if (*s != 0) s++;               /* Past terminating " */
-      EXIM_DATUM_SIZE(key) = t - keystart + add_zero;
-      }
-    else
-      {
-      keystart = s;
-      while (*s != 0 && *s != ':' && !isspace(*s)) s++;
-      EXIM_DATUM_SIZE(key) = s - keystart + add_zero;
-      }
+    EXIM_DATUM_SIZE(key) = s - line + add_zero;
 
     if (EXIM_DATUM_SIZE(key) > 256)
       {
@@ -354,12 +238,12 @@ while (Ufgets(line, max_insize, f) != NULL)
     if (lowercase)
       {
       for (i = 0; i < EXIM_DATUM_SIZE(key) - add_zero; i++)
-        keybuffer[i] = tolower(keystart[i]);
+        keybuffer[i] = tolower(line[i]);
       }
     else
       {
       for (i = 0; i < EXIM_DATUM_SIZE(key) - add_zero; i++)
-        keybuffer[i] = keystart[i];
+        keybuffer[i] = line[i];
       }
 
     keybuffer[i] = 0;
@@ -413,7 +297,7 @@ if (started)
 TIDYUP:
 
 EXIM_DBCLOSE(d);
-(void)fclose(f);
+fclose(f);
 
 /* If successful, output the number of entries and rename the temporary
 files. */
