@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -25,22 +25,48 @@ There are a number of common subroutines, followed by three main programs,
 whose inclusion is controlled by -D on the compilation command. */
 
 
-#include "exim.h"
+/* Standard C headers and Unix headers */
+
+#include <ctype.h>
+#include <signal.h>
+#include <stdarg.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+
+#include <errno.h>
+#include <fcntl.h>
+#include <unistd.h>
+#include <sys/stat.h>
+
+
+/* These are two values from macros.h which should perhaps be accessible in
+some better way than just repeating them here. */
+
+#define WAIT_NAME_MAX            50
+#define MESSAGE_ID_LENGTH        16
+
+
+/* This selection of Exim headers contains exactly what we need, and hopefully
+not too much extra baggage. */
+
+#include "config.h"              /* Needed to get the DB type */
+#include "mytypes.h"
+#include "macros.h"
+#include "dbstuff.h"
+#include "osfunctions.h"
+#include "store.h"
 
 
 /* Identifiers for the different database types. */
 
-#define type_retry     1
-#define type_wait      2
-#define type_misc      3
-#define type_callout   4
-#define type_ratelimit 5
+#define type_retry   1
+#define type_wait    2
+#define type_misc    3
+#define type_callout 4
 
 
-/* This is used by our cut-down dbfn_open(). */
-
-uschar *spool_directory;
-
 
 
 /*************************************************
@@ -49,18 +75,12 @@ uschar *spool_directory;
 
 /* For Berkeley DB >= 2, we can define a function to be called in case of DB
 errors. This should help with debugging strange DB problems, e.g. getting "File
-exists" when you try to open a db file. The API changed at release 4.3. */
+exists" when you try to open a db file. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
 void
-#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
-dbfn_bdb_error_callback(const DB_ENV *dbenv, const char *pfx, const char *msg)
-{
-dbenv = dbenv;
-#else
 dbfn_bdb_error_callback(const char *pfx, char *msg)
 {
-#endif
 pfx = pfx;
 printf("Berkeley DB error: %s\n", msg);
 }
@@ -72,7 +92,7 @@ printf("Berkeley DB error: %s\n", msg);
 *              SIGALRM handler                   *
 *************************************************/
 
-SIGNAL_BOOL sigalrm_seen;
+static int sigalrm_seen;
 
 void
 sigalrm_handler(int sig)
@@ -91,7 +111,7 @@ static void
 usage(uschar *name, uschar *options)
 {
 printf("Usage: exim_%s%s  <spool-directory> <database-name>\n", name, options);
-printf("  <database-name> = retry | misc | wait-<transport-name> | callout | ratelimit\n");
+printf("       <database-name> = retry | misc | wait-<transport-name> | callout\n");
 exit(1);
 }
 
@@ -113,7 +133,6 @@ if (argc == 3)
   if (Ustrcmp(argv[2], "misc") == 0) return type_misc;
   if (Ustrncmp(argv[2], "wait-", 5) == 0) return type_wait;
   if (Ustrcmp(argv[2], "callout") == 0) return type_callout;
-  if (Ustrcmp(argv[2], "ratelimit") == 0) return type_ratelimit;
   }
 usage(name, options);
 return -1;              /* Never obeyed */
@@ -140,7 +159,7 @@ Returns:    nothing
 */
 
 void
-log_write(unsigned int selector, int flags, const char *format, ...)
+log_write(unsigned int selector, int flags, char *format, ...)
 {
 va_list ap;
 va_start(ap, format);
@@ -237,18 +256,18 @@ uses. We assume the database exists, and therefore give up if we cannot open
 the lock file.
 
 Arguments:
+  spool    The spool directory
   name     The single-component name of one of Exim's database files.
   flags    O_RDONLY or O_RDWR
   dbblock  Points to an open_db block to be filled in.
-  lof      Unused.
 
 Returns:   NULL if the open failed, or the locking failed.
            On success, dbblock is returned. This contains the dbm pointer and
            the fd of the locked lock file.
 */
 
-open_db *
-dbfn_open(uschar *name, int flags, open_db *dbblock, BOOL lof)
+static open_db *
+dbfn_open(uschar *spool, uschar *name, int flags, open_db *dbblock)
 {
 int rc;
 struct flock lock_data;
@@ -259,7 +278,7 @@ uschar buffer[256];
 ensures that Exim has exclusive use of the database before it even tries to
 open it. If there is a database, there should be a lock file in existence. */
 
-sprintf(CS buffer, "%s/db/%s.lockfile", spool_directory, name);
+sprintf(CS buffer, "%s/db/%s.lockfile", spool, name);
 
 dbblock->lockfd = Uopen(buffer, flags, 0);
 if (dbblock->lockfd < 0)
@@ -287,14 +306,14 @@ if (rc < 0)
   printf("** Failed to get %s lock for %s: %s",
     ((flags & O_RDONLY) != 0)? "read" : "write", buffer,
     (errno == ETIMEDOUT)? "timed out" : strerror(errno));
-  (void)close(dbblock->lockfd);
+  close(dbblock->lockfd);
   return NULL;
   }
 
 /* At this point we have an opened and locked separate lock file, that is,
 exclusive access to the database, so we can go ahead and open it. */
 
-sprintf(CS buffer, "%s/db/%s", spool_directory, name);
+sprintf(CS buffer, "%s/db/%s", spool, name);
 EXIM_DBOPEN(buffer, flags, 0, &(dbblock->dbptr));
 
 if (dbblock->dbptr == NULL)
@@ -307,7 +326,7 @@ if (dbblock->dbptr == NULL)
     ""
     #endif
     );
-  (void)close(dbblock->lockfd);
+  close(dbblock->lockfd);
   return NULL;
   }
 
@@ -328,11 +347,11 @@ Argument: a pointer to an open database block
 Returns:  nothing
 */
 
-void
+static void
 dbfn_close(open_db *dbblock)
 {
 EXIM_DBCLOSE(dbblock->dbptr);
-(void)close(dbblock->lockfd);
+close(dbblock->lockfd);
 }
 
 
@@ -355,20 +374,16 @@ Returns: a pointer to the retrieved record, or
          NULL if the record is not found
 */
 
-void *
-dbfn_read_with_length(open_db *dbblock, const uschar *key, int *length)
+static void *
+dbfn_read_with_length(open_db *dbblock, uschar *key, int *length)
 {
 void *yield;
 EXIM_DATUM key_datum, result_datum;
-int klen = Ustrlen(key) + 1;
-uschar * key_copy = store_get(klen);
-
-memcpy(key_copy, key, klen);
 
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(result_datum);      /* to be cleared before use. */
-EXIM_DATUM_DATA(key_datum) = CS key_copy;
-EXIM_DATUM_SIZE(key_datum) = klen;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 
 if (!EXIM_DBGET(dbblock->dbptr, key_datum, result_datum)) return NULL;
 
@@ -399,21 +414,17 @@ Returns:    the yield of the underlying dbm or db "write" function. If this
             is dbm, the value is zero for OK.
 */
 
-int
-dbfn_write(open_db *dbblock, const uschar *key, void *ptr, int length)
+static int
+dbfn_write(open_db *dbblock, uschar *key, void *ptr, int length)
 {
 EXIM_DATUM key_datum, value_datum;
 dbdata_generic *gptr = (dbdata_generic *)ptr;
-int klen = Ustrlen(key) + 1;
-uschar * key_copy = store_get(klen);
-
-memcpy(key_copy, key, klen);
 gptr->time_stamp = time(NULL);
 
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require the datum */
 EXIM_DATUM_INIT(value_datum);       /* to be cleared before use. */
-EXIM_DATUM_DATA(key_datum) = CS key_copy;
-EXIM_DATUM_SIZE(key_datum) = klen;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 EXIM_DATUM_DATA(value_datum) = CS ptr;
 EXIM_DATUM_SIZE(value_datum) = length;
 return EXIM_DBPUT(dbblock->dbptr, key_datum, value_datum);
@@ -433,17 +444,13 @@ Arguments:
 Returns: the yield of the underlying dbm or db "delete" function.
 */
 
-int
-dbfn_delete(open_db *dbblock, const uschar *key)
+static int
+dbfn_delete(open_db *dbblock, uschar *key)
 {
-int klen = Ustrlen(key) + 1;
-uschar * key_copy = store_get(klen);
-
-memcpy(key_copy, key, klen);
 EXIM_DATUM key_datum;
 EXIM_DATUM_INIT(key_datum);         /* Some DBM libraries require clearing */
-EXIM_DATUM_DATA(key_datum) = CS key_copy;
-EXIM_DATUM_SIZE(key_datum) = klen;
+EXIM_DATUM_DATA(key_datum) = CS key;
+EXIM_DATUM_SIZE(key_datum) = Ustrlen(key) + 1;
 return EXIM_DBDEL(dbblock->dbptr, key_datum);
 }
 
@@ -468,7 +475,7 @@ Returns:   the next record from the file, or
            NULL if there are no more
 */
 
-uschar *
+static uschar *
 dbfn_scan(open_db *dbblock, BOOL start, EXIM_CURSOR **cursor)
 {
 EXIM_DATUM key_datum, value_datum;
@@ -514,8 +521,7 @@ uschar keybuffer[1024];
 /* Check the arguments, and open the database */
 
 dbdata_type = check_args(argc, argv, US"dumpdb", US"");
-spool_directory = argv[1];
-dbm = dbfn_open(argv[2], O_RDONLY, &dbblock, FALSE);
+dbm = dbfn_open(argv[1], argv[2], O_RDONLY, &dbblock);
 if (dbm == NULL) exit(1);
 
 /* Scan the file, formatting the information for each entry. Note
@@ -528,8 +534,6 @@ while (key != NULL)
   dbdata_retry *retry;
   dbdata_wait *wait;
   dbdata_callout_cache *callout;
-  dbdata_ratelimit *ratelimit;
-  dbdata_ratelimit_unique *rate_unique;
   int count_bad = 0;
   int i, length;
   uschar *t;
@@ -642,27 +646,19 @@ while (key != NULL)
         printf("\n");
         }
 
-      break;
+      /* Old-style domain record, without separate timestamps. This code can
+      eventually be thrown away, say in 5 years' time (it's now Feb 2003). */
 
-      case type_ratelimit:
-      if (Ustrstr(key, "/unique/") != NULL && length >= sizeof(*rate_unique))
-        {
-        ratelimit = (dbdata_ratelimit *)value;
-        rate_unique = (dbdata_ratelimit_unique *)value;
-        printf("%s.%06d rate: %10.3f epoch: %s size: %u key: %s\n",
-          print_time(ratelimit->time_stamp),
-          ratelimit->time_usec, ratelimit->rate,
-          print_time(rate_unique->bloom_epoch), rate_unique->bloom_size,
-          keybuffer);
-        }
       else
         {
-        ratelimit = (dbdata_ratelimit *)value;
-        printf("%s.%06d rate: %10.3f key: %s\n",
-          print_time(ratelimit->time_stamp),
-          ratelimit->time_usec, ratelimit->rate,
-          keybuffer);
+        printf("%s %s callout=%s postmaster=%s random=%s\n",
+          print_time(((dbdata_generic *)value)->time_stamp),
+          keybuffer,
+          print_cache(callout->result),
+          print_cache(callout->postmaster_result),
+          print_cache(callout->random_result));
         }
+
       break;
       }
     store_reset(value);
@@ -735,8 +731,6 @@ for(;;)
   dbdata_retry *retry;
   dbdata_wait *wait;
   dbdata_callout_cache *callout;
-  dbdata_ratelimit *ratelimit;
-  dbdata_ratelimit_unique *rate_unique;
   int i, oldlength;
   uschar *t;
   uschar field[256], value[256];
@@ -752,20 +746,20 @@ for(;;)
   /* If the buffer contains just one digit, or just consists of "d", use the
   previous name for an update. */
 
-  if ((isdigit((uschar)buffer[0]) && (buffer[1] == ' ' || buffer[1] == '\0'))
-       || Ustrcmp(buffer, "d") == 0)
+  if ((isdigit((uschar)buffer[0]) && !isdigit((uschar)buffer[1])) ||
+       Ustrcmp(buffer, "d") == 0)
     {
     if (name[0] == 0)
       {
       printf("No previous record name is set\n");
       continue;
       }
-    (void)sscanf(CS buffer, "%s %s", field, value);
+    sscanf(CS buffer, "%s %s", field, value);
     }
   else
     {
     name[0] = 0;
-    (void)sscanf(CS buffer, "%s %s %s", name, field, value);
+    sscanf(CS buffer, "%s %s %s", name, field, value);
     }
 
   /* Handle an update request */
@@ -773,8 +767,7 @@ for(;;)
   if (field[0] != 0)
     {
     int verify = 1;
-    spool_directory = argv[1];
-    dbm = dbfn_open(argv[2], O_RDWR, &dbblock, FALSE);
+    dbm = dbfn_open(argv[1], argv[2], O_RDWR, &dbblock);
     if (dbm == NULL) continue;
 
     if (Ustrcmp(field, "d") == 0)
@@ -855,7 +848,7 @@ for(;;)
             printf("Can't change contents of misc database record\n");
             break;
 
-            case type_callout:
+	    case type_callout:
             callout = (dbdata_callout_cache *)record;
             length = sizeof(dbdata_callout_cache);
             switch(fieldno)
@@ -877,79 +870,10 @@ for(;;)
               verify = 0;
               break;
               }
-            break;
-
-            case type_ratelimit:
-            ratelimit = (dbdata_ratelimit *)record;
-            switch(fieldno)
-              {
-              case 0:
-              if ((tt = read_time(value)) > 0) ratelimit->time_stamp = tt;
-                else printf("bad time value\n");
-              break;
-
-              case 1:
-              ratelimit->time_usec = Uatoi(value);
-              break;
-
-              case 2:
-              ratelimit->rate = Ustrtod(value, NULL);
-              break;
-
-              case 3:
-              if (Ustrstr(name, "/unique/") != NULL
-                && oldlength >= sizeof(dbdata_ratelimit_unique))
-                {
-                rate_unique = (dbdata_ratelimit_unique *)record;
-                if ((tt = read_time(value)) > 0) rate_unique->bloom_epoch = tt;
-                  else printf("bad time value\n");
-                break;
-                }
-              /* else fall through */
-
-              case 4:
-              case 5:
-              if (Ustrstr(name, "/unique/") != NULL
-                && oldlength >= sizeof(dbdata_ratelimit_unique))
-                {
-                /* see acl.c */
-                BOOL seen;
-                unsigned n, hash, hinc;
-                uschar md5sum[16];
-                md5 md5info;
-                md5_start(&md5info);
-                md5_end(&md5info, value, Ustrlen(value), md5sum);
-                hash = md5sum[0] <<  0 | md5sum[1] <<  8
-                     | md5sum[2] << 16 | md5sum[3] << 24;
-                hinc = md5sum[4] <<  0 | md5sum[5] <<  8
-                     | md5sum[6] << 16 | md5sum[7] << 24;
-                rate_unique = (dbdata_ratelimit_unique *)record;
-                seen = TRUE;
-                for (n = 0; n < 8; n++, hash += hinc)
-                  {
-                  int bit = 1 << (hash % 8);
-                  int byte = (hash / 8) % rate_unique->bloom_size;
-                  if ((rate_unique->bloom[byte] & bit) == 0)
-                    {
-                    seen = FALSE;
-                    if (fieldno == 5) rate_unique->bloom[byte] |= bit;
-                    }
-                  }
-                printf("%s %s\n",
-                  seen ? "seen" : fieldno == 5 ? "added" : "unseen", value);
-                break;
-                }
-              /* else fall through */
-
-              default:
-              printf("unknown field number\n");
-              verify = 0;
-              break;
-              }
-            break;
+	    break;
             }
 
-          dbfn_write(dbm, name, record, oldlength);
+          dbfn_write(dbm, name, record, length);
           }
         }
       }
@@ -970,8 +894,7 @@ for(;;)
 
   /* Handle a read request, or verify after an update. */
 
-  spool_directory = argv[1];
-  dbm = dbfn_open(argv[2], O_RDONLY, &dbblock, FALSE);
+  dbm = dbfn_open(argv[1], argv[2], O_RDONLY, &dbblock);
   if (dbm == NULL) continue;
 
   record = dbfn_read_with_length(dbm, name, &oldlength);
@@ -1045,21 +968,6 @@ for(;;)
             callout->random_result);
         }
       break;
-
-      case type_ratelimit:
-      ratelimit = (dbdata_ratelimit *)record;
-      printf("0 time stamp:  %s\n", print_time(ratelimit->time_stamp));
-      printf("1 fract. time: .%06d\n", ratelimit->time_usec);
-      printf("2 sender rate: % .3f\n", ratelimit->rate);
-      if (Ustrstr(name, "/unique/") != NULL
-       && oldlength >= sizeof(dbdata_ratelimit_unique))
-       {
-       rate_unique = (dbdata_ratelimit_unique *)record;
-       printf("3 filter epoch: %s\n", print_time(rate_unique->bloom_epoch));
-       printf("4 test filter membership\n");
-       printf("5 add element to filter\n");
-       }
-      break;
       }
     }
 
@@ -1157,8 +1065,7 @@ database */
 oldest = time(NULL) - maxkeep;
 printf("Tidying Exim hints database %s/db/%s\n", argv[1], argv[2]);
 
-spool_directory = argv[1];
-dbm = dbfn_open(argv[2], O_RDWR, &dbblock, FALSE);
+dbm = dbfn_open(argv[1], argv[2], O_RDWR, &dbblock);
 if (dbm == NULL) exit(1);
 
 /* Prepare for building file names */
@@ -1337,12 +1244,8 @@ while (keychain != NULL)
 
     if (Ustat(buffer, &statbuf) != 0)
       {
-      sprintf(CS(buffer + path_len), "%c/%s-D", id[5], id);
-      if (Ustat(buffer, &statbuf) != 0)
-        {
-        dbfn_delete(dbm, key);
-        printf("deleted %s (no message)\n", key);
-        }
+      dbfn_delete(dbm, key);
+      printf("deleted %s (no message)\n", key);
       }
     }
   }
