@@ -54,6 +54,7 @@
 
 #include "../exim.h"
 #include "lf_functions.h"
+#include "cdb.h"
 
 #ifdef HAVE_MMAP
 #  include <sys/mman.h>
@@ -110,8 +111,8 @@ cdb_hash(uschar *buf, unsigned int len)
 
 static int
 cdb_bread(int fd,
-         uschar *buf,
-         int len)
+	 uschar *buf,
+	 int len)
 {
   int r;
   while (len > 0) {
@@ -141,11 +142,9 @@ cdb_unpack(uschar *buf)
   return num;
 }
 
-static void cdb_close(void *handle);
-
-static void *
+void *
 cdb_open(uschar *filename,
-         uschar **errmsg)
+	 uschar **errmsg)
 {
   int fileno;
   struct cdb_state *cdbp;
@@ -166,23 +165,23 @@ cdb_open(uschar *filename,
     if (statbuf.st_size < CDB_HASH_TABLE) {
       int save_errno = errno;
       *errmsg = string_open_failed(errno,
-                                  "%s too short for cdb lookup",
-                                  filename);
+				  "%s to short for cdb lookup",
+				  filename);
       errno = save_errno;
       return NULL;
     }
   } else {
     int save_errno = errno;
     *errmsg = string_open_failed(errno,
-                                "fstat(%s) failed - cannot do cdb lookup",
-                                filename);
+				"fstat(%s) failed - cannot do cdb lookup",
+				filename);
     errno = save_errno;
     return NULL;
   }
 
   /* Having got a file open we need the structure to put things in */
-  cdbp = store_get(sizeof(struct cdb_state));
-  /* store_get() does not return if memory was not available... */
+  cdbp = store_malloc(sizeof(struct cdb_state));
+  /* store_malloc() does not return if memory was not available... */
   /* preload the structure.... */
   cdbp->fileno = fileno;
   cdbp->filelen = statbuf.st_size;
@@ -192,11 +191,11 @@ cdb_open(uschar *filename,
   /* if we are allowed to we use mmap here.... */
 #ifdef HAVE_MMAP
   mapbuf = mmap(NULL,
-               statbuf.st_size,
-               PROT_READ,
-               MAP_SHARED,
-               fileno,
-               0);
+	       statbuf.st_size,
+	       PROT_READ,
+	       MAP_SHARED,
+	       fileno,
+	       0);
   if (mapbuf != MAP_FAILED) {
     /* We have an mmap-ed section.  Now we can just use it */
     cdbp->cdb_map = mapbuf;
@@ -220,7 +219,7 @@ cdb_open(uschar *filename,
 
   /* get a buffer to stash the basic offsets in - this should speed
    * things up a lot - especially on multiple lookups */
-  cdbp->cdb_offsets = store_get(CDB_HASH_TABLE);
+  cdbp->cdb_offsets = store_malloc(CDB_HASH_TABLE);
 
   /* now fill the buffer up... */
   if (cdb_bread(fileno, cdbp->cdb_offsets, CDB_HASH_TABLE) == -1) {
@@ -228,8 +227,8 @@ cdb_open(uschar *filename,
      * time to give up I think....
      * call the close routine (deallocs the memory), and return NULL */
     *errmsg = string_open_failed(errno,
-                                "cannot read header from %s for cdb lookup",
-                                filename);
+				"cannot read header from %s for cdb lookup",
+				filename);
     cdb_close(cdbp);
     return NULL;
   }
@@ -244,22 +243,22 @@ cdb_open(uschar *filename,
 *             Check entry point                  *
 *************************************************/
 
-static BOOL
+BOOL
 cdb_check(void *handle,
-         uschar *filename,
-         int modemask,
-         uid_t *owners,
-         gid_t *owngroups,
-         uschar **errmsg)
+	 uschar *filename,
+	 int modemask,
+	 uid_t *owners,
+	 gid_t *owngroups,
+	 uschar **errmsg)
 {
   struct cdb_state * cdbp = handle;
   return lf_check_file(cdbp->fileno,
-                       filename,
+		       filename,
                        S_IFREG,
-                       modemask,
-                       owners,
-                       owngroups,
-                       "cdb",
+		       modemask,
+		       owners,
+		       owngroups,
+		       "cdb",
                        errmsg) == 0;
 }
 
@@ -269,14 +268,13 @@ cdb_check(void *handle,
 *              Find entry point                  *
 *************************************************/
 
-static int
+int
 cdb_find(void *handle,
-        uschar *filename,
-        uschar *keystring,
-        int  key_len,
-        uschar **result,
-        uschar **errmsg,
-        BOOL *do_cache)
+	uschar *filename,
+	uschar *keystring,
+	int  key_len,
+	uschar **result,
+	uschar **errmsg)
 {
   struct cdb_state * cdbp = handle;
   uint32 item_key_len,
@@ -292,9 +290,6 @@ cdb_find(void *handle,
     hash_slotnm;
   int loop;
 
-  /* Keep picky compilers happy */
-  do_cache = do_cache;
-
   key_hash = cdb_hash((uschar *)keystring, key_len);
 
   hash_offset_entry = CDB_HASH_ENTRY * (key_hash & CDB_HASH_MASK);
@@ -312,7 +307,7 @@ cdb_find(void *handle,
    * than the file, then we have problems.... */
   if ((hash_offset + (hash_offlen * CDB_HASH_ENTRY)) > cdbp->filelen) {
     *errmsg = string_sprintf("cdb: corrupt cdb file %s (too short)",
-                            filename);
+			    filename);
     DEBUG(D_lookup) debug_printf("%s\n", *errmsg);
     return DEFER;
   }
@@ -343,17 +338,17 @@ cdb_find(void *handle,
        item_ptr += 4;
        /* check key length matches */
        if (item_key_len == key_len) {
-         /* finally check if key matches */
-         if (Ustrncmp(keystring, item_ptr, key_len) == 0) {
-           /* we have a match....
-            * make item_ptr point to data */
-           item_ptr += item_key_len;
-           /* ... and the returned result */
-           *result = store_get(item_dat_len + 1);
-           memcpy(*result, item_ptr, item_dat_len);
-           (*result)[item_dat_len] = 0;
-           return OK;
-         }
+	 /* finally check if key matches */
+	 if (Ustrncmp(keystring, item_ptr, key_len) == 0) {
+	   /* we have a match....
+	    * make item_ptr point to data */
+	   item_ptr += item_key_len;
+	   /* ... and the returned result */
+	   *result = store_malloc(item_dat_len + 1);
+	   memcpy(*result, item_ptr, item_dat_len);
+	   (*result)[item_dat_len] = 0;
+	   return OK;
+	 }
        }
       }
       /* handle warp round of table */
@@ -382,22 +377,18 @@ cdb_find(void *handle,
       /* check key length matches */
       if (item_key_len == key_len) {
        /* finally check if key matches */
-       uschar * item_key = store_get(key_len);
+       uschar * item_key = store_malloc(key_len);
        if (cdb_bread(cdbp->fileno, item_key, key_len) == -1) return DEFER;
        if (Ustrncmp(keystring, item_key, key_len) == 0) {
-         /* Reclaim some store */
-         store_reset(item_key);
-         /* matches - get data length */
-         item_dat_len = cdb_unpack(packbuf + 4);
-         /* then we build a new result string */
-         *result = store_get(item_dat_len + 1);
-         if (cdb_bread(cdbp->fileno, *result, item_dat_len) == -1)
-           return DEFER;
-         (*result)[item_dat_len] = 0;
-         return OK;
+	 /* matches - get data length */
+	 item_dat_len = cdb_unpack(packbuf + 4);
+	 /* then we build a new result string */
+	 *result = store_malloc(item_dat_len + 1);
+	 if (cdb_bread(cdbp->fileno, *result, item_dat_len) == -1)
+	   return DEFER;
+	 (*result)[item_dat_len] = 0;
+	 return OK;
        }
-       /* Reclaim some store */
-       store_reset(item_key);
       }
     }
     cur_offset += 8;
@@ -417,7 +408,7 @@ cdb_find(void *handle,
 
 /* See local README for interface description */
 
-static void
+void
 cdb_close(void *handle)
 {
 struct cdb_state * cdbp = handle;
@@ -429,46 +420,11 @@ struct cdb_state * cdbp = handle;
      cdbp->cdb_offsets = NULL;
  }
 #endif /* HAVE_MMAP */
+ if (cdbp->cdb_offsets)
+   store_free(cdbp->cdb_offsets);
 
- (void)close(cdbp->fileno);
+ close(cdbp->fileno);
+ store_free(cdbp);
 }
 
-
-
-/*************************************************
-*         Version reporting entry point          *
-*************************************************/
-
-/* See local README for interface description. */
-
-#include "../version.h"
-
-void
-cdb_version_report(FILE *f)
-{
-#ifdef DYNLOOKUP
-fprintf(f, "Library version: CDB: Exim version %s\n", EXIM_VERSION_STR);
-#endif
-}
-
-
-lookup_info cdb_lookup_info = {
-  US"cdb",                       /* lookup name */
-  lookup_absfile,                /* uses absolute file name */
-  cdb_open,                      /* open function */
-  cdb_check,                     /* check function */
-  cdb_find,                      /* find function */
-  cdb_close,                     /* close function */
-  NULL,                          /* no tidy function */
-  NULL,                          /* no quoting function */
-  cdb_version_report             /* version reporting */
-};
-
-#ifdef DYNLOOKUP
-#define cdb_lookup_module_info _lookup_module_info
-#endif
-
-static lookup_info *_lookup_list[] = { &cdb_lookup_info };
-lookup_module_info cdb_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
-
 /* End of lookups/cdb.c */
