@@ -2,14 +2,21 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* The code in this module was contributed by Ard Biesheuvel. */
 
 #include "../exim.h"
 #include "lf_functions.h"
+#include "ibase.h"
 
+#ifndef LOOKUP_IBASE
+static void dummy(int x)
+{
+    dummy(x - 1);
+}
+#else
 #include <ibase.h>              /* The system header */
 
 /* Structure and anchor for caching connections. */
@@ -31,7 +38,7 @@ static ibase_connection *ibase_connections = NULL;
 
 /* See local README for interface description. */
 
-static void *ibase_open(uschar * filename, uschar ** errmsg)
+void *ibase_open(uschar * filename, uschar ** errmsg)
 {
     return (void *) (1);        /* Just return something non-null */
 }
@@ -44,7 +51,7 @@ static void *ibase_open(uschar * filename, uschar ** errmsg)
 
 /* See local README for interface description. */
 
-static void ibase_tidy(void)
+void ibase_tidy(void)
 {
     ibase_connection *cn;
     ISC_STATUS status[20];
@@ -55,6 +62,8 @@ static void ibase_tidy(void)
                                      cn->server);
         isc_commit_transaction(status, &cn->transh);
         isc_detach_database(status, &cn->dbh);
+        store_free(cn->server);
+        store_free(cn);
     }
 }
 
@@ -142,7 +151,7 @@ has the password removed. This copy is also used for debugging output. */
         *pp++ = 0;
         sdata[i] = pp;
         if (i == 2)
-            server_copy = string_copy(server);   /* sans password */
+            server_copy = string_copy_malloc(server);   /* sans password */
     }
     sdata[0] = server;          /* What's left at the start */
 
@@ -175,7 +184,7 @@ has the password removed. This copy is also used for debugging output. */
                              server_copy);
         }
     } else {
-        cn = store_get(sizeof(ibase_connection));
+        cn = store_malloc(sizeof(ibase_connection));
         cn->server = server_copy;
         cn->dbh = NULL;
         cn->transh = NULL;
@@ -217,6 +226,7 @@ has the password removed. This copy is also used for debugging output. */
             isc_interprete(buffer, &statusp);
             *errmsg =
                 string_sprintf("Interbase attach() failed: %s", buffer);
+            store_free(server_copy);
             *defer_break = FALSE;
             goto IBASE_EXIT;
         }
@@ -226,10 +236,11 @@ has the password removed. This copy is also used for debugging output. */
             (status, &cn->transh, 1, &cn->dbh, sizeof(trans_options),
              trans_options)) {
             isc_interprete(buffer, &statusp);
-            isc_detach_database(status, &cn->dbh);
             *errmsg =
                 string_sprintf("Interbase start_transaction() failed: %s",
                                buffer);
+            isc_detach_database(status, &cn->dbh);
+            store_free(server_copy);
             *defer_break = FALSE;
             goto IBASE_EXIT;
         }
@@ -245,37 +256,39 @@ has the password removed. This copy is also used for debugging output. */
         goto IBASE_EXIT;
     }
 
-    out_sqlda = store_get(XSQLDA_LENGTH(1));
+    out_sqlda = store_malloc(XSQLDA_LENGTH(1));
     out_sqlda->version = SQLDA_VERSION1;
     out_sqlda->sqln = 1;
 
     if (isc_dsql_prepare
         (status, &cn->transh, &stmth, 0, query, 1, out_sqlda)) {
         isc_interprete(buffer, &statusp);
-        store_reset(out_sqlda);
-        out_sqlda = NULL;
         *errmsg =
             string_sprintf("Interbase prepare_statement() failed: %s",
                            buffer);
+        store_free(out_sqlda);
+        out_sqlda = NULL;
         *defer_break = FALSE;
         goto IBASE_EXIT;
     }
 
 /* re-allocate the output structure if there's more than one field */
     if (out_sqlda->sqln < out_sqlda->sqld) {
-        XSQLDA *new_sqlda = store_get(XSQLDA_LENGTH(out_sqlda->sqld));
+        XSQLDA *new_sqlda = store_malloc(XSQLDA_LENGTH(out_sqlda->sqld));
         if (isc_dsql_describe
             (status, &stmth, out_sqlda->version, new_sqlda)) {
             isc_interprete(buffer, &statusp);
-            isc_dsql_free_statement(status, &stmth, DSQL_drop);
-            store_reset(out_sqlda);
-            out_sqlda = NULL;
             *errmsg =
                 string_sprintf("Interbase describe_statement() failed: %s",
                                buffer);
+            isc_dsql_free_statement(status, &stmth, DSQL_drop);
+            store_free(out_sqlda);
+            out_sqlda = NULL;
+            store_free(new_sqlda);
             *defer_break = FALSE;
             goto IBASE_EXIT;
         }
+        store_free(out_sqlda);
         out_sqlda = new_sqlda;
     }
 
@@ -284,47 +297,47 @@ has the password removed. This copy is also used for debugging output. */
         switch (var->sqltype & ~1) {
         case SQL_VARYING:
             var->sqldata =
-                (char *) store_get(sizeof(char) * var->sqllen + 2);
+                (char *) store_malloc(sizeof(char) * var->sqllen + 2);
             break;
         case SQL_TEXT:
             var->sqldata =
-                (char *) store_get(sizeof(char) * var->sqllen);
+                (char *) store_malloc(sizeof(char) * var->sqllen);
             break;
         case SQL_SHORT:
-            var->sqldata = (char *) store_get(sizeof(short));
+            var->sqldata = (char *) store_malloc(sizeof(short));
             break;
         case SQL_LONG:
-            var->sqldata = (char *) store_get(sizeof(ISC_LONG));
+            var->sqldata = (char *) store_malloc(sizeof(ISC_LONG));
             break;
 #ifdef SQL_INT64
         case SQL_INT64:
-            var->sqldata = (char *) store_get(sizeof(ISC_INT64));
+            var->sqldata = (char *) store_malloc(sizeof(ISC_INT64));
             break;
 #endif
         case SQL_FLOAT:
-            var->sqldata = (char *) store_get(sizeof(float));
+            var->sqldata = (char *) store_malloc(sizeof(float));
             break;
         case SQL_DOUBLE:
-            var->sqldata = (char *) store_get(sizeof(double));
+            var->sqldata = (char *) store_malloc(sizeof(double));
             break;
 #ifdef SQL_TIMESTAMP
         case SQL_DATE:
-            var->sqldata = (char *) store_get(sizeof(ISC_QUAD));
+            var->sqldata = (char *) store_malloc(sizeof(ISC_QUAD));
             break;
 #else
         case SQL_TIMESTAMP:
-            var->sqldata = (char *) store_get(sizeof(ISC_TIMESTAMP));
+            var->sqldata = (char *) store_malloc(sizeof(ISC_TIMESTAMP));
             break;
         case SQL_TYPE_DATE:
-            var->sqldata = (char *) store_get(sizeof(ISC_DATE));
+            var->sqldata = (char *) store_malloc(sizeof(ISC_DATE));
             break;
         case SQL_TYPE_TIME:
-            var->sqldata = (char *) store_get(sizeof(ISC_TIME));
+            var->sqldata = (char *) store_malloc(sizeof(ISC_TIME));
             break;
 #endif
         }
         if (var->sqltype & 1) {
-            var->sqlind = (short *) store_get(sizeof(short));
+            var->sqlind = (short *) store_malloc(sizeof(short));
         }
     }
 
@@ -336,6 +349,7 @@ has the password removed. This copy is also used for debugging output. */
             string_sprintf("Interbase describe_statement() failed: %s",
                            buffer);
         isc_dsql_free_statement(status, &stmth, DSQL_drop);
+        store_free(out_sqlda);
         *defer_break = FALSE;
         goto IBASE_EXIT;
     }
@@ -348,6 +362,7 @@ has the password removed. This copy is also used for debugging output. */
             *errmsg =
                 string_sprintf("Interbase fetch() failed: %s", buffer);
             isc_dsql_free_statement(status, &stmth, DSQL_drop);
+            store_free(out_sqlda);
             *defer_break = FALSE;
             goto IBASE_EXIT;
         }
@@ -414,16 +429,23 @@ always leaves enough room for a terminating zero. */
     if (result == NULL) {
         yield = FAIL;
         *errmsg = US "Interbase: no data found";
-    } else {
+    } else
         result[offset] = 0;
-        store_reset(result + offset + 1);
-    }
-
 
 /* Get here by goto from various error checks. */
 
   IBASE_EXIT:
 
+       if (out_sqlda != NULL) {
+	   /* de-allocate storage for every returned field */
+	   for (i = 0, var = out_sqlda->sqlvar; i < out_sqlda->sqld; i++, var++) {
+		       store_free(var->sqldata);
+
+	       if (var->sqltype & 1) {
+		   store_free(var->sqlind);
+	       }
+	   }
+       }
     if (stmth != NULL)
         isc_dsql_free_statement(status, &stmth, DSQL_drop);
 
@@ -449,18 +471,15 @@ always leaves enough room for a terminating zero. */
 arguments are not used. Loop through a list of servers while the query is
 deferred with a retryable error. */
 
-static int
+int
 ibase_find(void *handle, uschar * filename, uschar * query, int length,
-           uschar ** result, uschar ** errmsg, uint *do_cache)
+           uschar ** result, uschar ** errmsg)
 {
     int sep = 0;
     uschar *server;
     uschar *list = ibase_servers;
     uschar buffer[512];
 
-    /* Keep picky compilers happy */
-    do_cache = do_cache;
-
     DEBUG(D_lookup) debug_printf("Interbase query: %s\n", query);
 
     while ((server =
@@ -501,7 +520,7 @@ Arguments:
 Returns:     the processed string or NULL for a bad option
 */
 
-static uschar *ibase_quote(uschar * s, uschar * opt)
+uschar *ibase_quote(uschar * s, uschar * opt)
 {
     register int c;
     int count = 0;
@@ -544,41 +563,6 @@ static uschar *ibase_quote(uschar * s, uschar * opt)
     return quoted;
 }
 
-
-/*************************************************
-*         Version reporting entry point          *
-*************************************************/
-
-/* See local README for interface description. */
-
-#include "../version.h"
-
-void
-ibase_version_report(FILE *f)
-{
-#ifdef DYNLOOKUP
-fprintf(f, "Library version: ibase: Exim version %s\n", EXIM_VERSION_STR);
 #endif
-}
-
-
-static lookup_info _lookup_info = {
-  US"ibase",                     /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  ibase_open,                    /* open function */
-  NULL,                          /* no check function */
-  ibase_find,                    /* find function */
-  NULL,                          /* no close function */
-  ibase_tidy,                    /* tidy function */
-  ibase_quote,                   /* quoting function */
-  ibase_version_report           /* version reporting */
-};
-
-#ifdef DYNLOOKUP
-#define ibase_lookup_module_info _lookup_module_info
-#endif
-
-static lookup_info *_lookup_list[] = { &_lookup_info };
-lookup_module_info ibase_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
 
 /* End of lookups/ibase.c */
