@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Many thanks to Stuart Lynne for contributing the original code for this
@@ -13,14 +13,23 @@ researching how to handle the different kinds of error. */
 
 #include "../exim.h"
 #include "lf_functions.h"
+#include "ldap.h"
 
 
-/* Include LDAP headers. The code below uses some "old" LDAP interfaces that
-are deprecated in OpenLDAP. I don't know their status in other LDAP
-implementations. LDAP_DEPRECATED causes their prototypes to be defined in
-ldap.h. */
+/* We can't just compile this code and allow the library mechanism to omit the
+functions if they are not wanted, because we need to have the LDAP headers
+available for compiling. Therefore, compile these functions only if LOOKUP_LDAP
+is defined. However, some compilers don't like compiling empty modules, so keep
+them happy with a dummy when skipping the rest. Make it reference itself to
+stop picky compilers complaining that it is unused, and put in a dummy argument
+to stop even pickier compilers complaining about infinite loops. */
 
-#define LDAP_DEPRECATED 1
+#ifndef LOOKUP_LDAP
+static void dummy(int x) { dummy(x-1); }
+#else
+
+
+/* Include LDAP headers */
 
 #include <lber.h>
 #include <ldap.h>
@@ -61,6 +70,13 @@ LDAP_LIB_OPENLDAP1
 #endif
 
 
+/* For libraries without TCP connect timeouts */
+
+#ifndef LDAP_X_IO_TIMEOUT_NO_TIMEOUT
+#define LDAP_X_IO_TIMEOUT_NO_TIMEOUT (-1)
+#endif
+
+
 /* Four types of LDAP search are implemented */
 
 #define SEARCH_LDAP_MULTIPLE 0       /* Get attributes from multiple entries */
@@ -81,7 +97,6 @@ typedef struct ldap_connection {
   uschar *password;
   BOOL  bound;
   int   port;
-  BOOL  is_start_tls_called;
   LDAP *ld;
 } LDAP_CONNECTION;
 
@@ -119,10 +134,9 @@ Arguments:
   password      password for authentication, or NULL
   sizelimit     max number of entries returned, or 0 for no limit
   timelimit     max time to wait, or 0 for no limit
-  tcplimit      max time for network activity, e.g. connect, or 0 for OS default
+  tcplimit      max time to connect, or NULL for OS default
   deference     the dereference option, which is one of
                   LDAP_DEREF_{NEVER,SEARCHING,FINDING,ALWAYS}
-  referrals     the referral option, which is LDAP_OPT_ON or LDAP_OPT_OFF
 
 Returns:        OK or FAIL or DEFER
                 FAIL is given only if a lookup was performed successfully, but
@@ -130,10 +144,9 @@ Returns:        OK or FAIL or DEFER
 */
 
 static int
-perform_ldap_search(const uschar *ldap_url, uschar *server, int s_port,
-  int search_type, uschar **res, uschar **errmsg, BOOL *defer_break,
-  uschar *user, uschar *password, int sizelimit, int timelimit, int tcplimit,
-  int dereference, void *referrals)
+perform_ldap_search(uschar *ldap_url, uschar *server, int s_port, int search_type,
+  uschar **res, uschar **errmsg, BOOL *defer_break, uschar *user, uschar *password,
+  int sizelimit, int timelimit, int tcplimit, int dereference)
 {
 LDAPURLDesc     *ludp = NULL;
 LDAPMessage     *result = NULL;
@@ -146,7 +159,6 @@ struct timeval *timeoutptr = NULL;
 uschar *attr;
 uschar **attrp;
 uschar *data = NULL;
-uschar *dn = NULL;
 uschar *host;
 uschar **values;
 uschar **firstval;
@@ -156,19 +168,20 @@ uschar *error1 = NULL;   /* string representation of errcode (static) */
 uschar *error2 = NULL;   /* error message from the server */
 uschar *matched = NULL;  /* partially matched DN */
 
-int    attrs_requested = 0;
+int    attr_count = 0;
 int    error_yield = DEFER;
 int    msgid;
-int    rc, ldap_rc, ldap_parse_rc;
+int    rc;
 int    port;
 int    ptr = 0;
 int    rescount = 0;
 int    size = 0;
+int    old_pool = store_pool;
 BOOL   attribute_found = FALSE;
 BOOL   ldapi = FALSE;
 
 DEBUG(D_lookup)
-  debug_printf("perform_ldap_search: ldap%s URL = \"%s\" server=%s port=%d "
+  debug_printf("perform_ldap_search: ldap%s URL =\"%s\" server=%s port=%d "
     "sizelimit=%d timelimit=%d tcplimit=%d\n",
     (search_type == SEARCH_LDAP_MULTIPLE)? "m" :
     (search_type == SEARCH_LDAP_DN)? "dn" :
@@ -248,7 +261,7 @@ if (host != NULL)
 /* Count the attributes; we need this later to tell us how to format results */
 
 for (attrp = USS ludp->lud_attrs; attrp != NULL && *attrp != NULL; attrp++)
-  attrs_requested++;
+  attr_count++;
 
 /* See if we can find a cached connection to this host. The port is not
 relevant for ldapi. The host name pointer is set to NULL if no host was given
@@ -263,15 +276,6 @@ for (lcp = ldap_connections; lcp != NULL; lcp = lcp->next)
   if (ldapi || port == lcp->port) break;
   }
 
-/* Use this network timeout in any requests. */
-
-if (tcplimit > 0)
-  {
-  timeout.tv_sec = tcplimit;
-  timeout.tv_usec = 0;
-  timeoutptr = &timeout;
-  }
-
 /* If no cached connection found, we must open a connection to the server. If
 the server name is actually an absolute path, we set ldapi=TRUE above. This
 requests connection via a Unix socket. However, as far as I know, only OpenLDAP
@@ -281,13 +285,6 @@ if (lcp == NULL)
   {
   LDAP *ld;
 
-  #ifdef LDAP_OPT_X_TLS_NEWCTX
-  int  am_server = 0;
-  LDAP *ldsetctx;
-  #else
-  LDAP *ldsetctx = NULL;
-  #endif
-
 
   /* --------------------------- OpenLDAP ------------------------ */
 
@@ -373,31 +370,11 @@ if (lcp == NULL)
     goto RETURN_ERROR;
     }
 
-  #ifdef LDAP_OPT_X_TLS_NEWCTX
-  ldsetctx = ld;
-  #endif
-
   /* Set the TCP connect time limit if available. This is something that is
   in Netscape SDK v4.1; I don't know about other libraries. */
 
   #ifdef LDAP_X_OPT_CONNECT_TIMEOUT
-  if (tcplimit > 0)
-    {
-    int timeout1000 = tcplimit*1000;
-    ldap_set_option(ld, LDAP_X_OPT_CONNECT_TIMEOUT, (void *)&timeout1000);
-    }
-  else
-    {
-    int notimeout = LDAP_X_IO_TIMEOUT_NO_TIMEOUT;
-    ldap_set_option(ld, LDAP_X_OPT_CONNECT_TIMEOUT, (void *)&notimeout);
-    }
-  #endif
-
-  /* Set the TCP connect timeout. This works with OpenLDAP 2.2.14. */
-
-  #ifdef LDAP_OPT_NETWORK_TIMEOUT
-  if (tcplimit > 0)
-    ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT, (void *)timeoutptr);
+  ldap_set_option(ld, LDAP_X_OPT_CONNECT_TIMEOUT, (void *)&tcplimit);
   #endif
 
   /* I could not get TLS to work until I set the version to 3. That version
@@ -428,132 +405,33 @@ if (lcp == NULL)
   if (!ldapi)
     {
     int tls_option;
-    #ifdef LDAP_OPT_X_TLS_REQUIRE_CERT
-    if (eldap_require_cert != NULL)
-      {
-      tls_option = LDAP_OPT_X_TLS_NEVER;
-      if (Ustrcmp(eldap_require_cert, "hard") == 0)
-        {
-        tls_option = LDAP_OPT_X_TLS_HARD;
-        }
-      else if (Ustrcmp(eldap_require_cert, "demand") == 0)
-        {
-        tls_option = LDAP_OPT_X_TLS_DEMAND;
-        }
-      else if (Ustrcmp(eldap_require_cert, "allow") == 0)
-        {
-        tls_option = LDAP_OPT_X_TLS_ALLOW;
-        }
-      else if (Ustrcmp(eldap_require_cert, "try") == 0)
-        {
-        tls_option = LDAP_OPT_X_TLS_TRY;
-        }
-      DEBUG(D_lookup)
-        debug_printf("Require certificate overrides LDAP_OPT_X_TLS option (%d)\n",
-                     tls_option);
-      }
-    else
-    #endif  /* LDAP_OPT_X_TLS_REQUIRE_CERT */
     if (strncmp(ludp->lud_scheme, "ldaps", 5) == 0)
       {
       tls_option = LDAP_OPT_X_TLS_HARD;
-      DEBUG(D_lookup)
-        debug_printf("LDAP_OPT_X_TLS_HARD set due to ldaps:// URI\n");
+      DEBUG(D_lookup) debug_printf("LDAP_OPT_X_TLS_HARD set\n");
       }
     else
       {
       tls_option = LDAP_OPT_X_TLS_TRY;
-      DEBUG(D_lookup)
-        debug_printf("LDAP_OPT_X_TLS_TRY set due to ldap:// URI\n");
+      DEBUG(D_lookup) debug_printf("LDAP_OPT_X_TLS_TRY set\n");
       }
     ldap_set_option(ld, LDAP_OPT_X_TLS, (void *)&tls_option);
     }
   #endif  /* LDAP_OPT_X_TLS */
 
-  #ifdef LDAP_OPT_X_TLS_CACERTFILE
-  if (eldap_ca_cert_file != NULL)
-    {
-    ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_CACERTFILE, eldap_ca_cert_file);
-    }
-  #endif
-  #ifdef LDAP_OPT_X_TLS_CACERTDIR
-  if (eldap_ca_cert_dir != NULL)
-    {
-    ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_CACERTDIR, eldap_ca_cert_dir);
-    }
-  #endif
-  #ifdef LDAP_OPT_X_TLS_CERTFILE
-  if (eldap_cert_file != NULL)
-    {
-    ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_CERTFILE, eldap_cert_file);
-    }
-  #endif
-  #ifdef LDAP_OPT_X_TLS_KEYFILE
-  if (eldap_cert_key != NULL)
-    {
-    ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_KEYFILE, eldap_cert_key);
-    }
-  #endif
-  #ifdef LDAP_OPT_X_TLS_CIPHER_SUITE
-  if (eldap_cipher_suite != NULL)
-    {
-    ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_CIPHER_SUITE, eldap_cipher_suite);
-    }
-  #endif
-  #ifdef LDAP_OPT_X_TLS_REQUIRE_CERT
-  if (eldap_require_cert != NULL)
-    {
-    int cert_option = LDAP_OPT_X_TLS_NEVER;
-    if (Ustrcmp(eldap_require_cert, "hard") == 0)
-      {
-      cert_option = LDAP_OPT_X_TLS_HARD;
-      }
-    else if (Ustrcmp(eldap_require_cert, "demand") == 0)
-      {
-      cert_option = LDAP_OPT_X_TLS_DEMAND;
-      }
-    else if (Ustrcmp(eldap_require_cert, "allow") == 0)
-      {
-      cert_option = LDAP_OPT_X_TLS_ALLOW;
-      }
-    else if (Ustrcmp(eldap_require_cert, "try") == 0)
-      {
-      cert_option = LDAP_OPT_X_TLS_TRY;
-      }
-    /* This ldap handle is set at compile time based on client libs. Older
-     * versions want it to be global and newer versions can force a reload
-     * of the TLS context (to reload these settings we are changing from the
-     * default that loaded at instantiation). */
-    rc = ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_REQUIRE_CERT, &cert_option);
-    if (rc)
-      {
-      DEBUG(D_lookup)
-        debug_printf("Unable to set TLS require cert_option(%d) globally: %s\n",
-          cert_option, ldap_err2string(rc));
-      }
-    }
-  #endif
-  #ifdef LDAP_OPT_X_TLS_NEWCTX
-  rc = ldap_set_option(ldsetctx, LDAP_OPT_X_TLS_NEWCTX, &am_server);
-  if (rc)
-    {
-    DEBUG(D_lookup)
-      debug_printf("Unable to reload TLS context %d: %s\n",
-                   rc, ldap_err2string(rc));
-    }
-  #endif
-
   /* Now add this connection to the chain of cached connections */
 
+  store_pool = POOL_PERM;
   lcp = store_get(sizeof(LDAP_CONNECTION));
   lcp->host = (host == NULL)? NULL : string_copy(host);
+  store_pool = old_pool;
+
   lcp->bound = FALSE;
   lcp->user = NULL;
   lcp->password = NULL;
   lcp->port = port;
   lcp->ld = ld;
   lcp->next = ldap_connections;
-  lcp->is_start_tls_called = FALSE;
   ldap_connections = lcp;
   }
 
@@ -580,61 +458,23 @@ if (!lcp->bound ||
   {
   DEBUG(D_lookup) debug_printf("%sbinding with user=%s password=%s\n",
     (lcp->bound)? "re-" : "", user, password);
-  if (eldap_start_tls && !lcp->is_start_tls_called)
+  if ((rc = ldap_bind_s(lcp->ld, CS user, CS password, LDAP_AUTH_SIMPLE))
+       != LDAP_SUCCESS)
     {
-#if defined(LDAP_OPT_X_TLS) && !defined(LDAP_LIB_SOLARIS)
-    /* The Oracle LDAP libraries (LDAP_LIB_TYPE=SOLARIS) don't support this.
-     * Note: moreover, they appear to now define LDAP_OPT_X_TLS and still not
-     *       export an ldap_start_tls_s symbol.
-     */
-    if ( (rc = ldap_start_tls_s(lcp->ld, NULL, NULL)) != LDAP_SUCCESS)
+    /* Invalid credentials when just checking credentials returns FAIL. This
+    stops any further servers being tried. */
+
+    if (search_type == SEARCH_LDAP_AUTH && rc == LDAP_INVALID_CREDENTIALS)
       {
-      *errmsg = string_sprintf("failed to initiate TLS processing on an "
-          "LDAP session to server %s%s - ldap_start_tls_s() returned %d:"
-          " %s", host, porttext, rc, ldap_err2string(rc));
-      goto RETURN_ERROR;
+      DEBUG(D_lookup)
+        debug_printf("Invalid credentials: ldapauth returns FAIL\n");
+      error_yield = FAIL;
+      goto RETURN_ERROR_NOMSG;
       }
-    lcp->is_start_tls_called = TRUE;
-#else
-    DEBUG(D_lookup)
-      debug_printf("TLS initiation not supported with this Exim and your LDAP library.\n");
-#endif
-    }
-  if ((msgid = ldap_bind(lcp->ld, CS user, CS password, LDAP_AUTH_SIMPLE))
-       == -1)
-    {
-    *errmsg = string_sprintf("failed to bind the LDAP connection to server "
-      "%s%s - ldap_bind() returned -1", host, porttext);
-    goto RETURN_ERROR;
-    }
 
-  if ((rc = ldap_result( lcp->ld, msgid, 1, timeoutptr, &result )) <= 0)
-    {
-    *errmsg = string_sprintf("failed to bind the LDAP connection to server "
-      "%s%s - LDAP error: %s", host, porttext,
-      rc == -1 ? "result retrieval failed" : "timeout" );
-    result = NULL;
-    goto RETURN_ERROR;
-    }
+    /* Otherwise we have a problem that doesn't stop further servers from being
+    tried. */
 
-  rc = ldap_result2error( lcp->ld, result, 0 );
-
-  /* Invalid credentials when just checking credentials returns FAIL. This
-  stops any further servers being tried. */
-
-  if (search_type == SEARCH_LDAP_AUTH && rc == LDAP_INVALID_CREDENTIALS)
-    {
-    DEBUG(D_lookup)
-      debug_printf("Invalid credentials: ldapauth returns FAIL\n");
-    error_yield = FAIL;
-    goto RETURN_ERROR_NOMSG;
-    }
-
-  /* Otherwise we have a problem that doesn't stop further servers from being
-  tried. */
-
-  if (rc != LDAP_SUCCESS)
-    {
     *errmsg = string_sprintf("failed to bind the LDAP connection to server "
       "%s%s - LDAP error %d: %s", host, porttext, rc, ldap_err2string(rc));
     goto RETURN_ERROR;
@@ -642,12 +482,11 @@ if (!lcp->bound ||
 
   /* Successful bind */
 
+  store_pool = POOL_PERM;
   lcp->bound = TRUE;
   lcp->user = (user == NULL)? NULL : string_copy(user);
   lcp->password = (password == NULL)? NULL : string_copy(password);
-
-  ldap_msgfree(result);
-  result = NULL;
+  store_pool = old_pool;
   }
 
 /* If we are just checking credentials, return OK. */
@@ -676,14 +515,6 @@ an LDAP library without LDAP_OPT_DEREF. */
 ldap_set_option(lcp->ld, LDAP_OPT_DEREF, (void *)&dereference);
 #endif
 
-/* Similarly for the referral setting; should the library follow referrals that
-the LDAP server returns? The conditional is just in case someone uses a library
-without it. */
-
-#if defined(LDAP_OPT_REFERRALS)
-ldap_set_option(lcp->ld, LDAP_OPT_REFERRALS, referrals);
-#endif
-
 /* Start the search on the server. */
 
 DEBUG(D_lookup) debug_printf("Start search\n");
@@ -693,39 +524,32 @@ msgid = ldap_search(lcp->ld, ludp->lud_dn, ludp->lud_scope, ludp->lud_filter,
 
 if (msgid == -1)
   {
-  #if defined LDAP_LIB_SOLARIS || defined LDAP_LIB_OPENLDAP2
-  int err;
-  ldap_get_option(lcp->ld, LDAP_OPT_ERROR_NUMBER, &err);
-  *errmsg = string_sprintf("ldap_search failed: %d, %s", err,
-    ldap_err2string(err));
-
-  #else
-  *errmsg = string_sprintf("ldap_search failed");
-  #endif
-
+  *errmsg = string_sprintf("ldap search initiation failed");
   goto RETURN_ERROR;
   }
 
 /* Loop to pick up results as they come in, setting a timeout if one was
 given. */
 
+if (timelimit > 0)
+  {
+  timeout.tv_sec = timelimit;
+  timeout.tv_usec = 0;
+  timeoutptr = &timeout;
+  }
+
 while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
         LDAP_RES_SEARCH_ENTRY)
   {
   LDAPMessage  *e;
-  int valuecount;   /* We can see an attr spread across several
-                    entries. If B is derived from A and we request
-                    A and the directory contains both, A and B,
-                    then we get two entries, one for A and one for B.
-                    Here we just count the values per entry */
 
-  DEBUG(D_lookup) debug_printf("LDAP result loop\n");
+  DEBUG(D_lookup) debug_printf("ldap_result loop\n");
 
-  for(e = ldap_first_entry(lcp->ld, result), valuecount = 0;
+  for(e = ldap_first_entry(lcp->ld, result);
       e != NULL;
       e = ldap_next_entry(lcp->ld, e))
     {
-    uschar *new_dn;
+    uschar *dn;
     BOOL insert_space = FALSE;
 
     DEBUG(D_lookup) debug_printf("LDAP entry loop\n");
@@ -736,21 +560,17 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
 
     if (data != NULL) data = string_cat(data, &size, &ptr, US"\n", 1);
 
-    /* Get the DN from the last result. */
+    /* Get the DN from the last result and put it in $ldap_dn. */
 
-    new_dn = US ldap_get_dn(lcp->ld, e);
-    if (new_dn != NULL)
+    dn = US ldap_get_dn(lcp->ld, e);
+    if (dn != NULL)
       {
-      if (dn != NULL)
-        {
-        #if defined LDAP_LIB_NETSCAPE || defined LDAP_LIB_OPENLDAP2
-        ldap_memfree(dn);
-        #else   /* OPENLDAP 1, UMich, Solaris */
-        free(dn);
-        #endif
-        }
-      /* Save for later */
-      dn = new_dn;
+      eldap_dn = string_copy(dn);
+      #if defined LDAP_LIB_NETSCAPE || defined LDAP_LIB_OPENLDAP2
+      ldap_memfree(dn);
+      #else   /* OPENLDAP 1, UMich, Solaris */
+      free(dn);
+      #endif
       }
 
     /* If the data we want is actually the DN rather than any attribute values,
@@ -758,11 +578,11 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
     entries, the DNs will be concatenated, but we test for this case below, as
     for SEARCH_LDAP_SINGLE, and give an error. */
 
-    if (search_type == SEARCH_LDAP_DN)   /* Do not amalgamate these into one */
-      {                                  /* condition, because of the else */
-      if (new_dn != NULL)                /* below, that's for the first only */
+    if (search_type == SEARCH_LDAP_DN)
+      {
+      if (eldap_dn != NULL)
         {
-        data = string_cat(data, &size, &ptr, new_dn, Ustrlen(new_dn));
+        data = string_cat(data, &size, &ptr, eldap_dn, Ustrlen(eldap_dn));
         data[ptr] = 0;
         attribute_found = TRUE;
         }
@@ -770,21 +590,15 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
 
     /* Otherwise, loop through the entry, grabbing attribute values. If there's
     only one attribute being retrieved, no attribute name is given, and the
-    result is not quoted. Multiple values are separated by (comma).
+    result is not quoted. Multiple values are separated by (comma, space).
     If more than one attribute is being retrieved, the data is given as a
-    sequence of name=value pairs, separated by (space), with the value always in quotes.
-    If there are multiple values, they are given within the quotes, comma separated. */
+    sequence of name=value pairs, with the value always in quotes. If there are
+    multiple values, they are given within the quotes, comma separated. */
 
     else for (attr = US ldap_first_attribute(lcp->ld, e, &ber);
               attr != NULL;
               attr = US ldap_next_attribute(lcp->ld, e, ber))
       {
-      DEBUG(D_lookup) debug_printf("LDAP attr loop\n");
-
-      /* In case of attrs_requested == 1 we just count the values, in all other cases
-      (0, >1) we count the values per attribute */
-      if (attrs_requested != 1) valuecount = 0;
-
       if (attr[0] != 0)
         {
         /* Get array of values for this attribute. */
@@ -792,8 +606,7 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
         if ((firstval = values = USS ldap_get_values(lcp->ld, e, CS attr))
              != NULL)
           {
-
-          if (attrs_requested != 1)
+          if (attr_count != 1)
             {
             if (insert_space)
               data = string_cat(data, &size, &ptr, US" ", 1);
@@ -807,31 +620,22 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
             {
             uschar *value = *values;
             int len = Ustrlen(value);
-            ++valuecount;
 
-            DEBUG(D_lookup) debug_printf("LDAP value loop %s:%s\n", attr, value);
+            DEBUG(D_lookup) debug_printf("LDAP attr loop %s:%s\n", attr, value);
 
-            /* In case we requested one attribute only but got several times
-            into that attr loop, we need to append the additional values.
-            (This may happen if you derive attributeTypes B and C from A and
-            then query for A.) In all other cases we detect the different
-            attribute and append only every non first value. */
-
-            if (data && valuecount > 1)
-              data = string_cat(data, &size, &ptr, US",", 1);
+            if (values != firstval)
+              data = string_cat(data, &size, &ptr, US", ", 2);
 
             /* For multiple attributes, the data is in quotes. We must escape
-            internal quotes, backslashes, newlines, and must double commas. */
+            internal quotes, backslashes, newlines. */
 
-            if (attrs_requested != 1)
+            if (attr_count != 1)
               {
               int j;
               for (j = 0; j < len; j++)
                 {
                 if (value[j] == '\n')
                   data = string_cat(data, &size, &ptr, US"\\n", 2);
-                else if (value[j] == ',')
-                  data = string_cat(data, &size, &ptr, US",,", 2);
                 else
                   {
                   if (value[j] == '\"' || value[j] == '\\')
@@ -841,18 +645,9 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
                 }
               }
 
-            /* For single attributes, just double commas */
-
-	    else
-	      {
-	      int j;
-	      for (j = 0; j < len; j++)
-	        if (value[j] == ',')
-	          data = string_cat(data, &size, &ptr, US",,", 2);
-	        else
-	          data = string_cat(data, &size, &ptr, value+j, 1);
-	      }
+            /* For single attributes, copy the value verbatim */
 
+            else data = string_cat(data, &size, &ptr, value, len);
 
             /* Move on to the next value */
 
@@ -862,7 +657,7 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
 
           /* Closing quote at the end of the data for a named attribute. */
 
-          if (attrs_requested != 1)
+          if (attr_count != 1)
             data = string_cat(data, &size, &ptr, US"\"", 1);
 
           /* Free the values */
@@ -888,25 +683,9 @@ while ((rc = ldap_result(lcp->ld, msgid, 0, timeoutptr, &result)) ==
   result = NULL;
   }            /* End "while" loop for multiple results */
 
-/* Terminate the dynamic string that we have built and reclaim unused store */
+/* Terminate the dynamic string that we have built. */
 
-if (data != NULL)
-  {
-  data[ptr] = 0;
-  store_reset(data + ptr + 1);
-  }
-
-/* Copy the last dn into eldap_dn */
-
-if (dn != NULL)
-  {
-  eldap_dn = string_copy(dn);
-  #if defined LDAP_LIB_NETSCAPE || defined LDAP_LIB_OPENLDAP2
-  ldap_memfree(dn);
-  #else   /* OPENLDAP 1, UMich, Solaris */
-  free(dn);
-  #endif
-  }
+if (data != NULL) data[ptr] = 0;
 
 DEBUG(D_lookup) debug_printf("search ended by ldap_result yielding %d\n",rc);
 
@@ -951,16 +730,10 @@ if (rc == -1 || result == NULL)
   }
 
 /* A return code that isn't -1 doesn't necessarily mean there were no problems
-with the search. The message must be an LDAP_RES_SEARCH_RESULT or
-LDAP_RES_SEARCH_REFERENCE or else it's something we can't handle. Some versions
-of LDAP do not define LDAP_RES_SEARCH_REFERENCE (LDAP v1 is one, it seems). So
-we don't provide that functionality when we can't. :-) */
+with the search. The message must be an LDAP_RES_SEARCH_RESULT or else it's
+something we can't handle. */
 
-if (rc != LDAP_RES_SEARCH_RESULT
-#ifdef LDAP_RES_SEARCH_REFERENCE
-    && rc != LDAP_RES_SEARCH_REFERENCE
-#endif
-   )
+if (rc != LDAP_RES_SEARCH_RESULT)
   {
   *errmsg = string_sprintf("ldap_result returned unexpected code %d", rc);
   goto RETURN_ERROR;
@@ -969,19 +742,11 @@ if (rc != LDAP_RES_SEARCH_RESULT
 /* We have a result message from the server. This doesn't yet mean all is well.
 We need to parse the message to find out exactly what's happened. */
 
-#if defined LDAP_LIB_SOLARIS || defined LDAP_LIB_OPENLDAP2
-  ldap_rc = rc;
-  ldap_parse_rc = ldap_parse_result(lcp->ld, result, &rc, CSS &matched,
-    CSS &error2, NULL, NULL, 0);
-  DEBUG(D_lookup) debug_printf("ldap_parse_result: %d\n", ldap_parse_rc);
-  if (ldap_parse_rc < 0 &&
-      (ldap_parse_rc != LDAP_NO_RESULTS_RETURNED
-      #ifdef LDAP_RES_SEARCH_REFERENCE
-      || ldap_rc != LDAP_RES_SEARCH_REFERENCE
-      #endif
-     ))
+  #if defined LDAP_LIB_SOLARIS || defined LDAP_LIB_OPENLDAP2
+  if (ldap_parse_result(lcp->ld, result, &rc, CSS &matched, CSS &error2, NULL,
+      NULL, 0) < 0)
     {
-    *errmsg = string_sprintf("ldap_parse_result failed %d", ldap_parse_rc);
+    *errmsg = US"ldap_parse_result failed";
     goto RETURN_ERROR;
     }
   error1 = US ldap_err2string(rc);
@@ -1002,27 +767,18 @@ We need to parse the message to find out exactly what's happened. */
   (1) If we get LDAP_SIZELIMIT_EXCEEDED, just carry on, to return the
       truncated result list.
 
-  (2) If we get LDAP_RES_SEARCH_REFERENCE, also just carry on. This was a
-      submitted patch that is reported to "do the right thing" with Solaris
-      LDAP libraries. (The problem it addresses apparently does not occur with
-      Open LDAP.)
-
-  (3) The range of errors defined by LDAP_NAME_ERROR generally mean "that
+  (2) The range of errors defined by LDAP_NAME_ERROR generally mean "that
       object does not, or cannot, exist in the database". For those cases we
       fail the lookup.
 
-  (4) All other non-successes here are treated as some kind of problem with
+  (3) All other non-successes here are treated as some kind of problem with
       the lookup, so return DEFER (which is the default in error_yield).
 */
 
 DEBUG(D_lookup) debug_printf("ldap_parse_result yielded %d: %s\n",
   rc, ldap_err2string(rc));
 
-if (rc != LDAP_SUCCESS && rc != LDAP_SIZELIMIT_EXCEEDED
-    #ifdef LDAP_RES_SEARCH_REFERENCE
-    && rc != LDAP_RES_SEARCH_REFERENCE
-    #endif
-    )
+if (rc != LDAP_SUCCESS && rc != LDAP_SIZELIMIT_EXCEEDED)
   {
   *errmsg = string_sprintf("LDAP search failed - error %d: %s%s%s%s%s",
     rc,
@@ -1137,23 +893,20 @@ Returns:        OK or FAIL or DEFER
 */
 
 static int
-control_ldap_search(const uschar *ldap_url, int search_type, uschar **res,
+control_ldap_search(uschar *ldap_url, int search_type, uschar **res,
   uschar **errmsg)
 {
 BOOL defer_break = FALSE;
 int timelimit = LDAP_NO_LIMIT;
 int sizelimit = LDAP_NO_LIMIT;
-int tcplimit = 0;
-int sep = 0;
+int tcplimit = LDAP_X_IO_TIMEOUT_NO_TIMEOUT;
 int dereference = LDAP_DEREF_NEVER;
-void* referrals = LDAP_OPT_ON;
-const uschar *url = ldap_url;
-const uschar *p;
+int sep = 0;
+uschar *url = ldap_url;
+uschar *p;
 uschar *user = NULL;
 uschar *password = NULL;
-uschar *local_servers = NULL;
-uschar *server;
-const uschar *list;
+uschar *server, *list;
 uschar buffer[512];
 
 while (isspace(*url)) url++;
@@ -1165,7 +918,7 @@ NAME has the value "ldap". */
 
 while (strncmpic(url, US"ldap", 4) != 0)
   {
-  const uschar *name = url;
+  uschar *name = url;
   while (*url != 0 && *url != '=') url++;
   if (*url == '=')
     {
@@ -1179,9 +932,7 @@ while (strncmpic(url, US"ldap", 4) != 0)
       else if (strncmpic(name, US"PASS=", namelen) == 0) password = value;
       else if (strncmpic(name, US"SIZE=", namelen) == 0) sizelimit = Uatoi(value);
       else if (strncmpic(name, US"TIME=", namelen) == 0) timelimit = Uatoi(value);
-      else if (strncmpic(name, US"CONNECT=", namelen) == 0) tcplimit = Uatoi(value);
-      else if (strncmpic(name, US"NETTIME=", namelen) == 0) tcplimit = Uatoi(value);
-      else if (strncmpic(name, US"SERVERS=", namelen) == 0) local_servers = value;
+      else if (strncmpic(name, US"CONNECT=", namelen) == 0) tcplimit = Uatoi(value) * 1000;
 
       /* Don't know if all LDAP libraries have LDAP_OPT_DEREF */
 
@@ -1203,29 +954,7 @@ while (strncmpic(url, US"ldap", 4) != 0)
         DEBUG(D_lookup) debug_printf("%s\n", *errmsg);
         return DEFER;
         }
-      #endif
 
-      #ifdef LDAP_OPT_REFERRALS
-      else if (strncmpic(name, US"REFERRALS=", namelen) == 0)
-        {
-        if (strcmpic(value, US"follow") == 0) referrals = LDAP_OPT_ON;
-        else if (strcmpic(value, US"nofollow") == 0) referrals = LDAP_OPT_OFF;
-        else
-          {
-          *errmsg = string_sprintf("LDAP option REFERRALS is not \"follow\" "
-            "or \"nofollow\"");
-          DEBUG(D_lookup) debug_printf("%s\n", *errmsg);
-          return DEFER;
-          }
-        }
-      #else
-      else if (strncmpic(name, US"REFERRALS=", namelen) == 0)
-        {
-        *errmsg = string_sprintf("LDAP_OP_REFERRALS not defined in this LDAP "
-          "library - cannot use \"referrals\"");
-        DEBUG(D_lookup) debug_printf("%s\n", *errmsg);
-        return DEFER;
-        }
       #endif
 
       else
@@ -1274,8 +1003,8 @@ if (user != NULL)
 
 DEBUG(D_lookup)
   debug_printf("LDAP parameters: user=%s pass=%s size=%d time=%d connect=%d "
-    "dereference=%d referrals=%s\n", user, password, sizelimit, timelimit,
-    tcplimit, dereference, (referrals == LDAP_OPT_ON)? "on" : "off");
+    "dereference=%d\n", user, password, sizelimit, timelimit, tcplimit,
+    dereference);
 
 /* If the request is just to check authentication, some credentials must
 be given. The password must not be empty because LDAP binds with an empty
@@ -1309,16 +1038,15 @@ if (Ustrncmp(p, "://", 3) != 0)
 
 /* No default servers, or URL contains a server name: just one attempt */
 
-if ((eldap_default_servers == NULL && local_servers == NULL) || p[3] != '/')
+if (eldap_default_servers == NULL || p[3] != '/')
   {
   return perform_ldap_search(url, NULL, 0, search_type, res, errmsg,
-    &defer_break, user, password, sizelimit, timelimit, tcplimit, dereference,
-    referrals);
+    &defer_break, user, password, sizelimit, timelimit, tcplimit, dereference);
   }
 
-/* Loop through the default servers until OK or FAIL. Use local_servers list
- * if defined in the lookup, otherwise use the global default list */
-list = (local_servers == NULL) ? eldap_default_servers : local_servers;
+/* Loop through the default servers until OK or FAIL */
+
+list = eldap_default_servers;
 while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
   {
   int rc;
@@ -1330,8 +1058,7 @@ while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL
     port = Uatoi(colon+1);
     }
   rc = perform_ldap_search(url, server, port, search_type, res, errmsg,
-    &defer_break, user, password, sizelimit, timelimit, tcplimit, dereference,
-    referrals);
+    &defer_break, user, password, sizelimit, timelimit, tcplimit, dereference);
   if (rc != DEFER || defer_break) return rc;
   }
 
@@ -1348,39 +1075,31 @@ return DEFER;
 are handled by a common function, with a flag to differentiate between them.
 The handle and filename arguments are not used. */
 
-static int
-eldap_find(void *handle, uschar *filename, const uschar *ldap_url, int length,
-  uschar **result, uschar **errmsg, uint *do_cache)
+int
+eldap_find(void *handle, uschar *filename, uschar *ldap_url, int length,
+  uschar **result, uschar **errmsg)
 {
-/* Keep picky compilers happy */
-do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_SINGLE, result, errmsg));
 }
 
-static int
-eldapm_find(void *handle, uschar *filename, const uschar *ldap_url, int length,
-  uschar **result, uschar **errmsg, uint *do_cache)
+int
+eldapm_find(void *handle, uschar *filename, uschar *ldap_url, int length,
+  uschar **result, uschar **errmsg)
 {
-/* Keep picky compilers happy */
-do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_MULTIPLE, result, errmsg));
 }
 
-static int
-eldapdn_find(void *handle, uschar *filename, const uschar *ldap_url, int length,
-  uschar **result, uschar **errmsg, uint *do_cache)
+int
+eldapdn_find(void *handle, uschar *filename, uschar *ldap_url, int length,
+  uschar **result, uschar **errmsg)
 {
-/* Keep picky compilers happy */
-do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_DN, result, errmsg));
 }
 
 int
-eldapauth_find(void *handle, uschar *filename, const uschar *ldap_url, int length,
-  uschar **result, uschar **errmsg, uint *do_cache)
+eldapauth_find(void *handle, uschar *filename, uschar *ldap_url, int length,
+  uschar **result, uschar **errmsg)
 {
-/* Keep picky compilers happy */
-do_cache = do_cache;
 return(control_ldap_search(ldap_url, SEARCH_LDAP_AUTH, result, errmsg));
 }
 
@@ -1392,7 +1111,7 @@ return(control_ldap_search(ldap_url, SEARCH_LDAP_AUTH, result, errmsg));
 
 /* See local README for interface description. */
 
-static void *
+void *
 eldap_open(uschar *filename, uschar **errmsg)
 {
 return (void *)(1);    /* Just return something non-null */
@@ -1404,21 +1123,17 @@ return (void *)(1);    /* Just return something non-null */
 *               Tidy entry point                 *
 *************************************************/
 
-/* See local README for interface description.
-Make sure that eldap_dn does not refer to reclaimed or worse, freed store */
+/* See local README for interface description. */
 
-static void
+void
 eldap_tidy(void)
 {
 LDAP_CONNECTION *lcp = NULL;
-eldap_dn = NULL;
-
 while ((lcp = ldap_connections) != NULL)
   {
   DEBUG(D_lookup) debug_printf("unbind LDAP connection to %s:%d\n", lcp->host,
     lcp->port);
-  if(lcp->bound == TRUE)
-    ldap_unbind(lcp->ld);
+  ldap_unbind(lcp->ld);
   ldap_connections = lcp->next;
   }
 }
@@ -1504,12 +1219,12 @@ quote_ldap_dn, respectively. */
 
 
 
-static uschar *
+uschar *
 eldap_quote(uschar *s, uschar *opt)
 {
 register int c;
 int count = 0;
-int len = 0;
+int len;
 BOOL dn = FALSE;
 uschar *t = s;
 uschar *quoted;
@@ -1533,13 +1248,13 @@ the string. */
 
 while ((c = *t++) != 0)
   {
-  len++;
   if (!isalnum(c) && Ustrchr(ALWAYS_LITERAL, c) == NULL) count += 5;
   }
 if (count == 0) return s;
 
 /* Get sufficient store to hold the quoted string */
 
+len = Ustrlen(s);
 t = quoted = store_get(len + count + 1);
 
 /* Handle plain quote_ldap */
@@ -1623,66 +1338,6 @@ else
 return quoted;
 }
 
-
-
-/*************************************************
-*         Version reporting entry point          *
-*************************************************/
-
-/* See local README for interface description. */
-
-#include "../version.h"
-
-void
-ldap_version_report(FILE *f)
-{
-#ifdef DYNLOOKUP
-fprintf(f, "Library version: LDAP: Exim version %s\n", EXIM_VERSION_STR);
-#endif
-}
-
-
-static lookup_info ldap_lookup_info = {
-  US"ldap",                      /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  eldap_open,                    /* open function */
-  NULL,                          /* check function */
-  eldap_find,                    /* find function */
-  NULL,                          /* no close function */
-  eldap_tidy,                    /* tidy function */
-  eldap_quote,                   /* quoting function */
-  ldap_version_report            /* version reporting */
-};
-
-static lookup_info ldapdn_lookup_info = {
-  US"ldapdn",                     /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  eldap_open,       /* sic */    /* open function */
-  NULL,                          /* check function */
-  eldapdn_find,                  /* find function */
-  NULL,                          /* no close function */
-  eldap_tidy,       /* sic */    /* tidy function */
-  eldap_quote,      /* sic */    /* quoting function */
-  NULL                           /* no version reporting (redundant) */
-};
-
-static lookup_info ldapm_lookup_info = {
-  US"ldapm",                     /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  eldap_open,       /* sic */    /* open function */
-  NULL,                          /* check function */
-  eldapm_find,                   /* find function */
-  NULL,                          /* no close function */
-  eldap_tidy,       /* sic */    /* tidy function */
-  eldap_quote,      /* sic */    /* quoting function */
-  NULL                           /* no version reporting (redundant) */
-};
-
-#ifdef DYNLOOKUP
-#define ldap_lookup_module_info _lookup_module_info
-#endif
-
-static lookup_info *_lookup_list[] = { &ldap_lookup_info, &ldapdn_lookup_info, &ldapm_lookup_info };
-lookup_module_info ldap_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 3 };
+#endif  /* LOOKUP_LDAP */
 
 /* End of lookups/ldap.c */
