@@ -2,14 +2,14 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the mysql lookup functions */
 
-extern void *mysql_open(char *, char **);
-extern int   mysql_find(void *, char *, char *, int, char **, char **);
+extern void *mysql_open(uschar *, uschar **);
+extern int   mysql_find(void *, uschar *, uschar *, int, uschar **, uschar **);
 extern void  mysql_tidy(void);
-extern char *mysql_quote(char *);
+extern uschar *mysql_quote(uschar *, uschar *);
 
 /* End of lookups/mysql.h */
