@@ -2,13 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2002 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Header for the nisplus lookup */
 
-extern void *nisplus_open(char *, char **);
-extern int   nisplus_find(void *, char *, char *, int, char **, char **);
-extern char *nisplus_quote(char *);
+extern void *nisplus_open(uschar *, uschar **);
+extern int   nisplus_find(void *, uschar *, uschar *, int, uschar **, uschar **);
+extern uschar *nisplus_quote(uschar *, uschar *);
 
 /* End of lookups/nisplus.h */
