@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Interface to an Oracle database. This code was originally supplied by
@@ -13,26 +13,36 @@ some comments from my position of Oracle ignorance. */
 #include "../exim.h"
 
 
+/* We can't just compile this code and allow the library mechanism to omit the
+functions if they are not wanted, because we need to have the ORACLE headers
+available for compiling. Therefore, compile these functions only if
+LOOKUP_ORACLE is defined. However, some compilers don't like compiling empty
+modules, so keep them happy with a dummy when skipping the rest. Make it
+reference itself to stop picky compilers complaining that it is unused, and put
+in a dummy argument to stop even pickier compilers complaining about infinite
+loops. */
+
+#ifndef LOOKUP_ORACLE
+static void dummy(int x) { dummy(x-1); }
+#else
+
 /* The Oracle system headers */
 
 #include <oratypes.h>
 #include <ocidfn.h>
 #include <ocikpr.h>
 
+#include "oracle.h"                    /* The local header */
+
 #define PARSE_NO_DEFER           0     /* parse straight away */
 #define PARSE_V7_LNG             2
 #define MAX_ITEM_BUFFER_SIZE  1024     /* largest size of a cell of data */
 #define MAX_SELECT_LIST_SIZE    32     /* maximum number of columns (not rows!) */
 
 /* Paul's comment on this was "change this to 512 for 64bit cpu", but I don't
-understand why. The Oracle manual just asks for 256 bytes.
+understand why. The Oracle manual just asks for 256 bytes. */
 
-That was years ago. Jin Choi suggested (March 2007) that this change should
-be made in the source, as at worst it wastes 256 bytes, and it saves people
-having to discover about this for themselves as more and more systems are
-64-bit. So I have changed 256 to 512. */
-
-#define HDA_SIZE 512
+#define HDA_SIZE 256
 
 /* Internal/external datatype codes */
 
@@ -195,7 +205,7 @@ return col;
 
 /* See local README for interface description. */
 
-static void *
+void *
 oracle_open(uschar *filename, uschar **errmsg)
 {
 return (void *)(1);    /* Just return something non-null */
@@ -209,7 +219,7 @@ return (void *)(1);    /* Just return something non-null */
 
 /* See local README for interface description. */
 
-static void
+void
 oracle_tidy(void)
 {
 oracle_connection *cn;
@@ -218,6 +228,10 @@ while ((cn = oracle_connections) != NULL)
   oracle_connections = cn->next;
   DEBUG(D_lookup) debug_printf("close ORACLE connection: %s\n", cn->server);
   ologof(cn->handle);
+  store_free(cn->server);
+  store_free(cn->hda_mem);
+  store_free(cn->handle);
+  store_free(cn);
   }
 }
 
@@ -281,7 +295,7 @@ for (i = 3; i > 0; i--)
     }
   *pp++ = 0;
   sdata[i] = pp;
-  if (i == 3) server_copy = string_copy(server);  /* sans password */
+  if (i == 3) server_copy = string_copy_malloc(server);  /* sans password */
   }
 sdata[0] = server;   /* What's left at the start */
 
@@ -311,8 +325,8 @@ if (cn == NULL)
 
   /* Get store for a new connection, initialize it, and connect to the server */
 
-   oracle_handle = store_get(sizeof(struct cda_def));
-   hda = store_get(HDA_SIZE);
+   oracle_handle = store_malloc(sizeof(struct cda_def));
+   hda = store_malloc(HDA_SIZE);
    memset(hda,'\0',HDA_SIZE);
 
   /*
@@ -329,13 +343,16 @@ if (cn == NULL)
     {
     *errmsg = oracle_error(oracle_handle, oracle_handle->rc,
       US"connection failed");
+    store_free(oracle_handle);
+    store_free(hda);
+    store_free(server_copy);
     *defer_break = FALSE;
     goto ORACLE_EXIT_NO_VALS;
     }
 
   /* Add the connection to the cache */
 
-  cn = store_get(sizeof(oracle_connection));
+  cn = store_malloc(sizeof(oracle_connection));
   cn->server = server_copy;
   cn->handle = oracle_handle;
   cn->next = oracle_connections;
@@ -354,12 +371,13 @@ else
 
 /* We have a connection. Open a cursor and run the query */
 
-cda = store_get(sizeof(Cda_Def));
+cda = store_malloc(sizeof(Cda_Def));
 
 if (oopen(cda, oracle_handle, (text *)0, -1, -1, (text *)0, -1) != 0)
   {
   *errmsg = oracle_error(oracle_handle, cda->rc, "failed to open cursor");
   *defer_break = FALSE;
+  store_free(cda);
   goto ORACLE_EXIT_NO_VALS;
   }
 
@@ -369,14 +387,15 @@ if (oparse(cda, (text *)query, (sb4) -1,
   *errmsg = oracle_error(oracle_handle, cda->rc, "query failed");
   *defer_break = FALSE;
   oclose(cda);
+  store_free(cda);
   goto ORACLE_EXIT_NO_VALS;
   }
 
 /* Find the number of fields returned and sort out their types. If the number
 is one, we don't add field names to the data. Otherwise we do. */
 
-def = store_get(sizeof(Ora_Define)*MAX_SELECT_LIST_SIZE);
-desc = store_get(sizeof(Ora_Describe)*MAX_SELECT_LIST_SIZE);
+def = store_malloc(sizeof(Ora_Define)*MAX_SELECT_LIST_SIZE);
+desc = store_malloc(sizeof(Ora_Describe)*MAX_SELECT_LIST_SIZE);
 
 if ((num_fields = describe_define(cda,def,desc)) == -1)
   {
@@ -430,9 +449,9 @@ while (cda->rc != NO_DATA_FOUND)  /* Loop for each row */
       result = string_cat(result, &ssize, &offset, "\"", 1);
       for (j = 0; j < def[i].col_retlen; j++)
         {
-        if (def[i].buf[j] == '\"' || def[i].buf[j] == '\\')
-          result = string_cat(result, &ssize, &offset, "\\", 1);
-        result = string_cat(result, &ssize, &offset, def[i].buf+j, 1);
+	if (def[i].buf[j] == '\"' || def[i].buf[j] == '\\')
+	  result = string_cat(result, &ssize, &offset, "\\", 1);
+	result = string_cat(result, &ssize, &offset, def[i].buf+j, 1);
         }
       result = string_cat(result, &ssize, &offset, "\"", 1);
       }
@@ -474,25 +493,25 @@ if (result == NULL)
   yield = FAIL;
   *errmsg = "ORACLE: no data found";
   }
-else
-  {
-  result[offset] = 0;
-  store_reset(result + offset + 1);
-  }
+else result[offset] = 0;
 
 /* Get here by goto from various error checks. */
 
 ORACLE_EXIT:
 
-/* Close the cursor; don't close the connection, as it is cached. */
+/* Close the cursor, and free memory for any result that was got; don't close
+the connection, as it is cached. */
 
 oclose(cda);
+store_free(cda);
+store_free(def);
+store_free(desc);
 
 ORACLE_EXIT_NO_VALS:
 
 /* Non-NULL result indicates a sucessful result */
 
-if (result != NULL)
+if(result != NULL)
   {
   *resultptr = result;
   return OK;
@@ -515,17 +534,15 @@ else
 arguments are not used. Loop through a list of servers while the query is
 deferred with a retryable error. */
 
-static int
+int
 oracle_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg, BOOL *do_cache)
+  uschar **result, uschar **errmsg)
 {
 int sep = 0;
 uschar *server;
 uschar *list = oracle_servers;
 uschar buffer[512];
 
-do_cache = do_cache;   /* Placate picky compilers */
-
 DEBUG(D_lookup) debug_printf("ORACLE query: %s\n", query);
 
 while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
@@ -560,7 +577,7 @@ Arguments:
 Returns:     the processed string or NULL for a bad option
 */
 
-static uschar *
+uschar *
 oracle_quote(uschar *s, uschar *opt)
 {
 register int c;
@@ -602,41 +619,6 @@ while ((c = *s++) != 0)
 return quoted;
 }
 
-
-/*************************************************
-*         Version reporting entry point          *
-*************************************************/
-
-/* See local README for interface description. */
-
-#include "../version.h"
-
-void
-oracle_version_report(FILE *f)
-{
-#ifdef DYNLOOKUP
-fprintf(f, "Library version: Oracle: Exim version %s\n", EXIM_VERSION_STR);
-#endif
-}
-
-
-static lookup_info _lookup_info = {
-  US"oracle",                    /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  oracle_open,                   /* open function */
-  NULL,                          /* check function */
-  oracle_find,                   /* find function */
-  NULL,                          /* no close function */
-  oracle_tidy,                   /* tidy function */
-  oracle_quote,                  /* quoting function */
-  oracle_version_report          /* version reporting */
-};
-
-#ifdef DYNLOOKUP
-#define oracle_lookup_module_info _lookup_module_info
-#endif
-
-static lookup_info *_lookup_list[] = { &_lookup_info };
-lookup_module_info oracle_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
+#endif  /* LOOKUP_ORACLE */
 
 /* End of lookups/oracle.c */
