@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This code originally came from Robert Wal. */
@@ -10,7 +10,22 @@
 #include "../exim.h"
 
 
+/* We can't just compile this code and allow the library mechanism to omit the
+functions if they are not wanted, because we need to have the WHOSON headers
+available for compiling. Therefore, compile these functions only if
+LOOKUP_WHOSON is defined. However, some compilers don't like compiling empty
+modules, so keep them happy with a dummy when skipping the rest. Make it
+reference itself to stop picky compilers complaining that it is unused, and put
+in a dummy argument to stop even pickier compilers complaining about infinite
+loops. */
+
+#ifndef LOOKUP_WHOSON
+static void dummy(int x) { dummy(x-1); }
+#else
+
+
 #include <whoson.h>        /* Public header */
+#include "whoson.h"        /* Local header */
 
 
 /*************************************************
@@ -19,7 +34,7 @@
 
 /* See local README for interface description. */
 
-static void *
+void *
 whoson_open(uschar *filename, uschar **errmsg)
 {
 filename = filename;   /* Keep picky compilers happy */
@@ -34,18 +49,17 @@ return (void *)(1);    /* Just return something non-null */
 
 /* See local README for interface description. */
 
-static int
+int
 whoson_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg, BOOL *do_cache)
+  uschar **result, uschar **errmsg)
 {
 uschar buffer[80];
 handle = handle;          /* Keep picky compilers happy */
 filename = filename;
 length = length;
 errmsg = errmsg;
-do_cache = do_cache;
 
-switch (wso_query(CS query, CS buffer, sizeof(buffer)))
+switch (wso_query(query, CS buffer, sizeof(buffer)))
   {
   case 0:
   *result = string_copy(buffer);    /* IP in database; return name of user */
@@ -60,42 +74,6 @@ switch (wso_query(CS query, CS buffer, sizeof(buffer)))
   }
 }
 
-
-
-/*************************************************
-*         Version reporting entry point          *
-*************************************************/
-
-/* See local README for interface description. */
-
-#include "../version.h"
-
-void
-whoson_version_report(FILE *f)
-{
-fprintf(f, "Library version: Whoson: Runtime: %s\n", wso_version());
-#ifdef DYNLOOKUP
-fprintf(f, "                         Exim version %s\n", EXIM_VERSION_STR);
-#endif
-}
-
-static lookup_info _lookup_info = {
-  US"whoson",                    /* lookup name */
-  lookup_querystyle,             /* query-style lookup */
-  whoson_open,                   /* open function */
-  NULL,                          /* check function */
-  whoson_find,                   /* find function */
-  NULL,                          /* no close function */
-  NULL,                          /* no tidy function */
-  NULL,                          /* no quoting function */
-  whoson_version_report          /* version reporting */
-};
-
-#ifdef DYNLOOKUP
-#define whoson_lookup_module_info _lookup_module_info
-#endif
-
-static lookup_info *_lookup_list[] = { &_lookup_info };
-lookup_module_info whoson_lookup_module_info = { LOOKUP_MODULE_INFO_MAGIC, _lookup_list, 1 };
+#endif  /* LOOKUP_WHOSON */
 
 /* End of lookups/whoson.c */
