@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for matching strings */
@@ -15,8 +15,8 @@
 strings, domains, and local parts. */
 
 typedef struct check_string_block {
-  const uschar *origsubject;           /* caseful; keep these two first, in */
-  const uschar *subject;               /* step with the block below */
+  uschar *origsubject;               /* caseful; keep these two first, in */
+  uschar *subject;                   /* step with the block below */
   int    expand_setup;
   BOOL   use_partial;
   BOOL   caseless;
@@ -28,7 +28,7 @@ typedef struct check_string_block {
 addresses. */
 
 typedef struct check_address_block {
-  const uschar *origaddress;         /* caseful; keep these two first, in */
+  uschar *origaddress;               /* caseful; keep these two first, in */
   uschar *address;                   /* step with the block above */
   int    expand_setup;
   BOOL   caseless;
@@ -60,9 +60,6 @@ The possible types of pattern are:
       @mx_secondary  matches any domain with a secondary MX to the local host
   . literal - anything else
 
-Any of the @mx_xxx options can be followed by "/ignore=<list>" where <list> is
-a list of IP addresses that are to be ignored (typically 127.0.0.1).
-
 Arguments:
   arg            check_string_block pointer - see below
   pattern        the pattern to be matched
@@ -72,8 +69,7 @@ Arguments:
                    returns ERROR)
 
 Contents of the argument block:
-  origsubject    the subject in its original casing
-  subject        the subject string to be checked, lowercased if caseless
+  subject        the subject string to be checked
   expand_setup   if < 0, don't set up any numeric expansion variables;
                  if = 0, set $0 to whole subject, and either
                    $1 to what matches * or
@@ -92,13 +88,13 @@ Returns:       OK    if matched
 */
 
 static int
-check_string(void *arg, const uschar *pattern, const uschar **valueptr, uschar **error)
+check_string(void *arg, uschar *pattern, uschar **valueptr, uschar **error)
 {
-const check_string_block *cb = arg;
+check_string_block *cb = (check_string_block *)arg;
 int search_type, partial, affixlen, starflags;
 int expand_setup = cb->expand_setup;
-const uschar *affix;
-uschar *s;
+uschar *affix;
+uschar *s = cb->subject;
 uschar *filename = NULL;
 uschar *keyquery, *result, *semicolon;
 void *handle;
@@ -107,12 +103,6 @@ error = error;  /* Keep clever compilers from complaining */
 
 if (valueptr != NULL) *valueptr = NULL;  /* For non-lookup matches */
 
-/* For regular expressions, use cb->origsubject rather than cb->subject so that
-it works if the pattern uses (?-i) to turn off case-independence, overriding
-"caseless". */
-
-s = string_copy(pattern[0] == '^' ? cb->origsubject : cb->subject);
-
 /* If required to set up $0, initialize the data but don't turn on by setting
 expand_nmax until the match is assured. */
 
@@ -131,7 +121,7 @@ if (pattern[0] == '^')
   {
   const pcre *re = regex_must_compile(pattern, cb->caseless, FALSE);
   return ((expand_setup < 0)?
-           pcre_exec(re, NULL, CCS s, Ustrlen(s), 0, PCRE_EOPT, NULL, 0) >= 0
+           pcre_exec(re, NULL, CS s, Ustrlen(s), 0, PCRE_EOPT, NULL, 0) >= 0
            :
            regex_match_and_setup(re, s, 0, expand_setup)
          )?
@@ -167,63 +157,41 @@ just fall through - the match will fail. */
 
 if (cb->at_is_special && pattern[0] == '@')
   {
-  if (pattern[1] == 0)
-    {
-    pattern = primary_hostname;
-    goto NOT_AT_SPECIAL;               /* Handle as exact string match */
-    }
+  int prim = -1;
+  int secy = -1;
 
-  if (Ustrcmp(pattern, "@[]") == 0)
+  if (pattern[1] == 0) pattern = primary_hostname;  /* Fall through to exact */
+
+  else if (Ustrcmp(pattern, "@[]") == 0)
     {
     ip_address_item *ip;
     int slen = Ustrlen(s);
     if (s[0] != '[' && s[slen-1] != ']') return FAIL;
     for (ip = host_find_interfaces(); ip != NULL; ip = ip->next)
-      if (Ustrncmp(ip->address, s+1, slen - 2) == 0
-            && ip->address[slen - 2] == 0)
-        return OK;
+      if (Ustrncmp(ip->address, s+1, slen - 2) == 0) return OK;
     return FAIL;
     }
 
-  if (strncmpic(pattern, US"@mx_", 4) == 0)
+  else if (strcmpic(pattern, US"@mx_any") == 0 ||
+            (prim = strcmpic(pattern, US"@mx_primary")) == 0 ||
+            (secy = strcmpic(pattern, US"@mx_secondary")) == 0)
     {
-    int rc;
     host_item h;
-    BOOL prim = FALSE;
-    BOOL secy = FALSE;
     BOOL removed = FALSE;
-    const uschar *ss = pattern + 4;
-    const uschar *ignore_target_hosts = NULL;
-
-    if (strncmpic(ss, US"any", 3) == 0) ss += 3;
-    else if (strncmpic(ss, US"primary", 7) == 0)
-      {
-      ss += 7;
-      prim = TRUE;
-      }
-    else if (strncmpic(ss, US"secondary", 9) == 0)
-      {
-      ss += 9;
-      secy = TRUE;
-      }
-    else goto NOT_AT_SPECIAL;
-
-    if (strncmpic(ss, US"/ignore=", 8) == 0) ignore_target_hosts = ss + 8;
-      else if (*ss != 0) goto NOT_AT_SPECIAL;
+    int rc;
 
     h.next = NULL;
     h.name = s;
     h.address = NULL;
 
     rc = host_find_bydns(&h,
-      ignore_target_hosts,
-      HOST_FIND_BY_MX,     /* search only for MX, not SRV or A */
-      NULL,                /* service name not relevant */
-      NULL,                /* srv_fail_domains not relevant */
-      NULL,                /* mx_fail_domains not relevant */
-      NULL,                /* no dnssec request/require XXX ? */
-      NULL,                /* no feedback FQDN */
-      &removed);           /* feedback if local removed */
+      NULL,       /* ignore list */
+      TRUE,       /* MX required */
+      FALSE,      /* not A only */
+      FALSE,      /* no widening */
+      FALSE,      /* no widening */
+      NULL,       /* no feedback FQDN */
+      &removed);  /* feedback if local removed */
 
     if (rc == HOST_FIND_AGAIN)
       {
@@ -231,8 +199,8 @@ if (cb->at_is_special && pattern[0] == '@')
       return DEFER;
       }
 
-    if (rc == HOST_FOUND_LOCAL && !secy) return OK;
-    if (prim) return FAIL;
+    if (rc == HOST_FOUND_LOCAL && secy != 0 ) return OK;
+    if (prim == 0) return FAIL;
     return removed? OK : FAIL;
 
     /*** The above line used to be the following line, but this is incorrect,
@@ -245,10 +213,6 @@ if (cb->at_is_special && pattern[0] == '@')
     }
   }
 
-/* Escape point from code for specials that start with "@" */
-
-NOT_AT_SPECIAL:
-
 /* This is an exact string match if there is no semicolon in the pattern. */
 
 if ((semicolon = Ustrchr(pattern, ';')) == NULL)
@@ -274,20 +238,12 @@ up user@domain for sender rejection). There's a flag to disable it. */
 
 if (!cb->use_partial) partial = -1;
 
-/* Set the parameters for the three different kinds of lookup. */
+/* Set the parameters for the two different kinds of lookup. */
 
 keyquery = semicolon + 1;
 while (isspace(*keyquery)) keyquery++;
 
-if (mac_islookup(search_type, lookup_absfilequery))
-  {
-  filename = keyquery;
-  while (*keyquery != 0 && !isspace(*keyquery)) keyquery++;
-  filename = string_copyn(filename, keyquery - filename);
-  while (isspace(*keyquery)) keyquery++;
-  }
-
-else if (!mac_islookup(search_type, lookup_querystyle))
+if (!mac_islookup(search_type, lookup_querystyle))
   {
   filename = keyquery;
   keyquery = s;
@@ -336,8 +292,8 @@ Returns:       OK    if matched
 */
 
 int
-match_check_string(const uschar *s, const uschar *pattern, int expand_setup,
-  BOOL use_partial, BOOL caseless, BOOL at_is_special, const uschar **valueptr)
+match_check_string(uschar *s, uschar *pattern, int expand_setup,
+  BOOL use_partial, BOOL caseless, BOOL at_is_special, uschar **valueptr)
 {
 check_string_block cb;
 cb.origsubject = s;
@@ -351,41 +307,6 @@ return check_string(&cb, pattern, valueptr, NULL);
 
 
 
-/*************************************************
-*       Get key string from check block          *
-*************************************************/
-
-/* When caching the data from a lookup for a named list, we have to save the
-key that was found, because other lookups of different keys on the same list
-may occur. This function has knowledge of the different lookup types, and
-extracts the appropriate key.
-
-Arguments:
-  arg          the check block
-  type         MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, or MCL_LOCALPART
-*/
-
-static const uschar *
-get_check_key(void *arg, int type)
-{
-switch(type)
-  {
-  case MCL_STRING:
-  case MCL_DOMAIN:
-  case MCL_LOCALPART:
-  return ((check_string_block *)arg)->subject;
-
-  case MCL_HOST:
-  return ((check_host_block *)arg)->host_address;
-
-  case MCL_ADDRESS:
-  return ((check_address_block *)arg)->address;
-  }
-return US"";  /* In practice, should never happen */
-}
-
-
-
 /*************************************************
 *       Scan list and run matching function      *
 *************************************************/
@@ -411,7 +332,9 @@ Arguments:
   listptr      pointer to the pointer to the list
   sep          separator character for string_nextinlist();
                  normally zero for a standard list;
-                 sometimes UCHAR_MAX+1 for single items;
+                 sometimes UCHAR_MAX+n for single items;
+                 UCHAR_MAX+1 is used when the item is to be expanded
+                 UCHAR_MAX+2 is used when the item is not to be expanded
   anchorptr    -> tree of named items, or NULL if no named items
   cache_ptr    pointer to pointer to cache bits for named items, or
                  pointer to NULL if not caching; may get set NULL if an
@@ -421,8 +344,6 @@ Arguments:
                  in the structure it points to
   type         MCL_STRING, MCL_DOMAIN, MCL_HOST, MCL_ADDRESS, or MCL_LOCALPART
                  these are used for some special handling
-               MCL_NOEXPAND (whose value is greater than any of them) may
-                 be added to any value to suppress expansion of the list
   name         string to use in debugging info
   valueptr     where to pass back data from a lookup
 
@@ -431,21 +352,18 @@ Returns:       OK    if matched a non-negated item
                FAIL  if expansion force-failed
                FAIL  if matched a negated item
                FAIL  if hit end of list after a non-negated item
-               DEFER if a something deferred or expansion failed
+               DEFER if a lookup deferred or expansion failed
 */
 
 int
-match_check_list(const uschar **listptr, int sep, tree_node **anchorptr,
-  unsigned int **cache_ptr, int (*func)(void *,const uschar *,const uschar **,uschar **),
-  void *arg, int type, const uschar *name, const uschar **valueptr)
+match_check_list(uschar **listptr, int sep, tree_node **anchorptr,
+  unsigned int **cache_ptr, int (*func)(void *,uschar *,uschar **,uschar **),
+  void *arg, int type, uschar *name, uschar **valueptr)
 {
 int yield = OK;
 unsigned int *original_cache_bits = *cache_ptr;
 BOOL include_unknown = FALSE;
-BOOL ignore_unknown = FALSE;
-BOOL include_defer = FALSE;
-BOOL ignore_defer = FALSE;
-const uschar *list;
+uschar *list;
 uschar *sss;
 uschar *ot = NULL;
 uschar buffer[1024];
@@ -458,43 +376,27 @@ HDEBUG(D_any)
   if (listname[0] != 0) ot = string_sprintf("%s in %s?", name, listname);
   }
 
-/* If the list is empty, the answer is no. Skip the debugging output for
-an unnamed list. */
+/* If the list is empty, the answer is no. This case happens only when checking
+named options, so ot will be set. */
 
 if (*listptr == NULL)
   {
-  HDEBUG(D_lists)
-    {
-    if (ot != NULL) debug_printf("%s no (option unset)\n", ot);
-    }
+  HDEBUG(D_lists) debug_printf("%s no (option unset)\n", ot);
   return FAIL;
   }
 
 /* Expand the list before we scan it. A forced expansion gives the answer
-"not in list"; other expansion errors cause DEFER to be returned. However,
-if the type value is greater than or equal to than MCL_NOEXPAND, do not expand
-the list. */
+"not in list"; other expansion errors cause DEFER to be returned. If the
+separator is set to UCHAR_MAX+2 we have a single item list that is not to
+be expanded. */
 
-if (type >= MCL_NOEXPAND)
+if (sep == UCHAR_MAX+2)
   {
   list = *listptr;
-  type -= MCL_NOEXPAND;       /* Remove the "no expand" flag */
   }
 else
   {
-  /* If we are searching a domain list, and $domain is not set, set it to the
-  subject that is being sought for the duration of the expansion. */
-
-  if (type == MCL_DOMAIN && deliver_domain == NULL)
-    {
-    check_string_block *cb = (check_string_block *)arg;
-    deliver_domain = string_copy(cb->subject);
-    list = expand_cstring(*listptr);
-    deliver_domain = NULL;
-    }
-
-  else list = expand_cstring(*listptr);
-
+  list = expand_string(*listptr);
   if (list == NULL)
     {
     if (expand_string_forcedfail)
@@ -554,34 +456,14 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
       }
     }
 
-  /* If the host item is "+include_unknown" or "+ignore_unknown", remember it
-  in case there's a subsequent failed reverse lookup. There is similar
-  processing for "defer". */
+  /* If the host item is "+include_unknown", remember it in case there's a
+  subsequent failed reverse lookup. */
 
-  else if (type == MCL_HOST && *ss == '+')
+  else if (type == MCL_HOST)
     {
     if (Ustrcmp(ss, "+include_unknown") == 0)
       {
       include_unknown = TRUE;
-      ignore_unknown = FALSE;
-      continue;
-      }
-    if (Ustrcmp(ss, "+ignore_unknown") == 0)
-      {
-      ignore_unknown = TRUE;
-      include_unknown = FALSE;
-      continue;
-      }
-    if (Ustrcmp(ss, "+include_defer") == 0)
-      {
-      include_defer = TRUE;
-      ignore_defer = FALSE;
-      continue;
-      }
-    if (Ustrcmp(ss, "+ignore_defer") == 0)
-      {
-      ignore_defer = TRUE;
-      include_defer = FALSE;
       continue;
       }
     }
@@ -641,6 +523,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
 
       if (bits == 0)
         {
+        nb->cache_data = NULL;
         switch (match_check_list(&(nb->string), 0, anchorptr, &use_cache_bits,
                 func, arg, type, name, valueptr))
           {
@@ -650,7 +533,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
           }
 
         /* If this list was uncacheable, or a sublist turned out to be
-        uncacheable, the value of use_cache_bits will now be NULL, even if it
+        uncacheable, the value of use_cache_lists will now be NULL, even if it
         wasn't before. Ensure that this is passed up to the next level.
         Otherwise, remember the result of the search in the cache. */
 
@@ -661,25 +544,9 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         else
           {
           use_cache_bits[offset] |= bits << shift;
-
           if (valueptr != NULL)
             {
-            int old_pool = store_pool;
-            namedlist_cacheblock *p;
-
-            /* Cached data for hosts persists over more than one message,
-            so we use the permanent store pool */
-
-            store_pool = POOL_PERM;
-            p = store_get(sizeof(namedlist_cacheblock));
-            p->key = string_copy(get_check_key(arg, type));
-
-
-            p->data = (*valueptr == NULL)? NULL : string_copy(*valueptr);
-            store_pool = old_pool;
-
-            p->next = nb->cache_data;
-            nb->cache_data = p;
+            nb->cache_data = *valueptr;
             if (*valueptr != NULL)
               {
               DEBUG(D_lists) debug_printf("data from lookup saved for "
@@ -689,9 +556,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
           }
         }
 
-       /* Previously cached; to find a lookup value, search a chain of values
-       and compare keys. Typically, there is only one such, but it is possible
-       for different keys to have matched the same named list. */
+       /* Previously cached */
 
       else
         {
@@ -700,16 +565,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         cached = US" - cached";
         if (valueptr != NULL)
           {
-          const uschar *key = get_check_key(arg, type);
-          namedlist_cacheblock *p;
-          for (p = nb->cache_data; p != NULL; p = p->next)
-            {
-            if (Ustrcmp(key, p->key) == 0)
-              {
-              *valueptr = p->data;
-              break;
-              }
-            }
+          *valueptr = nb->cache_data;
           DEBUG(D_lists) debug_printf("cached lookup data = %s\n", *valueptr);
           }
         }
@@ -729,7 +585,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
 
     else
       {
-      uschar *error = NULL;
+      uschar *error;
       switch ((func)(arg, ss, valueptr, &error))
         {
         case OK:
@@ -738,46 +594,18 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
         return yield;
 
         case DEFER:
-        if (error == NULL)
-          error = string_sprintf("DNS lookup of \"%s\" deferred", ss);
-        if (ignore_defer)
-          {
-          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_defer\n",
-            error);
-          break;
-          }
-        if (include_defer)
-          {
-          log_write(0, LOG_MAIN, "%s: accepted by +include_defer", error);
-          return OK;
-          }
-        if (!search_error_message) search_error_message = error;
         goto DEFER_RETURN;
 
-        /* The ERROR return occurs when checking hosts, when either a forward
-        or reverse lookup has failed. It can also occur in a match_ip list if a
-        non-IP address item is encountered. The error string gives details of
+        /* The ERROR return occurs only when checking hosts, when either a
+        forward or reverse lookup has failed. The error string gives details of
         which it was. */
 
         case ERROR:
-        if (ignore_unknown)
-          {
-          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown\n",
-            error);
-          }
-        else
-          {
-          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
-            include_unknown? "yes":"no", error);
-          if (!include_unknown)
-            {
-            if ((log_extra_selector & LX_unknown_in_list) != 0)
-              log_write(0, LOG_MAIN, "list matching forced to fail: %s", error);
-            return FAIL;
-            }
-          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-          return OK;
-          }
+        HDEBUG(D_lists)
+          debug_printf("%s %s (%s)\n", ot, include_unknown? "yes":"no", error);
+        if (!include_unknown) return FAIL;
+        log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+        return OK;
         }
       }
     }
@@ -845,48 +673,22 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
       switch ((func)(arg, ss, valueptr, &error))
         {
         case OK:
-        (void)fclose(f);
+        fclose(f);
         HDEBUG(D_lists) debug_printf("%s %s (matched \"%s\" in %s)\n", ot,
           (yield == OK)? "yes" : "no", sss, filename);
         return file_yield;
 
         case DEFER:
-        if (error == NULL)
-          error = string_sprintf("DNS lookup of %s deferred", ss);
-        if (ignore_defer)
-          {
-          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_defer\n",
-            error);
-          break;
-          }
-        (void)fclose(f);
-        if (include_defer)
-          {
-          log_write(0, LOG_MAIN, "%s: accepted by +include_defer", error);
-          return OK;
-          }
+        fclose(f);
         goto DEFER_RETURN;
 
         case ERROR:          /* host name lookup failed - this can only */
-        if (ignore_unknown)  /* be for an incoming host (not outgoing) */
-          {
-          HDEBUG(D_lists) debug_printf("%s: item ignored by +ignore_unknown\n",
-            error);
-          }
-        else
-         {
-          HDEBUG(D_lists) debug_printf("%s %s (%s)\n", ot,
-            include_unknown? "yes":"no", error);
-          (void)fclose(f);
-          if (!include_unknown)
-            {
-            if ((log_extra_selector & LX_unknown_in_list) != 0)
-              log_write(0, LOG_MAIN, "list matching forced to fail: %s", error);
-            return FAIL;
-            }
-          log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
-          return OK;
-          }
+        HDEBUG(D_lists)      /* be for an incoming host (not outgoing) */
+          debug_printf("%s %s (%s)\n", ot, include_unknown? "yes":"no", error);
+        fclose(f);
+        if (!include_unknown) return FAIL;
+        log_write(0, LOG_MAIN, "%s: accepted by +include_unknown", error);
+        return OK;
         }
       }
 
@@ -894,7 +696,7 @@ while ((sss = string_nextinlist(&list, &sep, buffer, sizeof(buffer))) != NULL)
     for the file, in case this is the last item in the list. */
 
     yield = file_yield;
-    (void)fclose(f);
+    fclose(f);
     }
   }    /* Loop for the next item on the top-level list */
 
@@ -904,23 +706,25 @@ HDEBUG(D_lists)
   debug_printf("%s %s (end of list)\n", ot, (yield == OK)? "no":"yes");
 return (yield == OK)? FAIL : OK;
 
-/* Something deferred */
+/* Handle lookup defer */
 
 DEFER_RETURN:
-HDEBUG(D_lists) debug_printf("%s list match deferred for %s\n", ot, sss);
+HDEBUG(D_lists) debug_printf("%s lookup deferred for %s\n", ot, sss);
 return DEFER;
 }
 
 
+
+
 /*************************************************
-*          Match in colon-separated list         *
+*            Match in colon-separated list       *
 *************************************************/
 
-/* This function is used for domain lists and local part lists. It is not used
-for host lists or address lists, which have additional interpretation of the
-patterns. Some calls of it set sep > UCHAR_MAX in order to use its matching
-facilities on single items. When this is done, it arranges to set the numerical
-variables as a result of the match.
+/* This function is used for domain lists. It is not used for host lists or
+address lists, which have additional interpretation of the patterns. Some calls
+of it set sep > UCHAR_MAX in order to use its matching facilities on single
+items. When this is done, it arranges to set the numerical variables as a
+result of the match.
 
 This function is now just a short interface to match_check_list(), which does
 list scanning in a general way. A good compiler will optimize the tail
@@ -936,8 +740,6 @@ Arguments:
                  MCL_LOCALPART when matching a local part list (address lists
                    have their own function)
                  MCL_STRING for others (e.g. list of ciphers)
-                 MCL_NOEXPAND (whose value is greater than any of them) may
-                   be added to any value to suppress expansion of the list
   caseless       TRUE for (mostly) caseless matching - passed directly to
                    match_check_string()
   valueptr       pointer to where any lookup data is to be passed back,
@@ -952,9 +754,8 @@ Returns:         OK    if matched a non-negated item
 */
 
 int
-match_isinlist(const uschar *s, const uschar **listptr, int sep,
-   tree_node **anchorptr,
-  unsigned int *cache_bits, int type, BOOL caseless, const uschar **valueptr)
+match_isinlist(uschar *s, uschar **listptr, int sep, tree_node **anchorptr,
+  unsigned int *cache_bits, int type, BOOL caseless, uschar **valueptr)
 {
 unsigned int *local_cache_bits = cache_bits;
 check_string_block cb;
@@ -963,7 +764,7 @@ cb.subject = caseless? string_copylc(s) : string_copy(s);
 cb.expand_setup = (sep > UCHAR_MAX)? 0 : -1;
 cb.use_partial = TRUE;
 cb.caseless = caseless;
-cb.at_is_special = (type == MCL_DOMAIN || type == MCL_DOMAIN + MCL_NOEXPAND);
+cb.at_is_special = (type == MCL_DOMAIN);
 if (valueptr != NULL) *valueptr = NULL;
 return  match_check_list(listptr, sep, anchorptr, &local_cache_bits,
   check_string, &cb, type, s, valueptr);
@@ -982,7 +783,7 @@ its arguments are in an indirect block.
 Arguments:
   arg            the argument block (see below)
   pattern        the pattern to match
-  valueptr       where to return a value
+  valueptr       where to return a value (never used)
   error          for error messages (not used in this function; it never
                    returns ERROR)
 
@@ -1000,55 +801,49 @@ Returns:         OK     for a match
 */
 
 static int
-check_address(void *arg, const uschar *pattern, const uschar **valueptr, uschar **error)
+check_address(void *arg, uschar *pattern, uschar **valueptr, uschar **error)
 {
 check_address_block *cb = (check_address_block *)arg;
 check_string_block csb;
 int rc;
 int expand_inc = 0;
 unsigned int *null = NULL;
-const uschar *listptr;
+uschar *listptr;
 uschar *subject = cb->address;
-const uschar *s;
-uschar *pdomain, *sdomain;
+uschar *s, *pdomain, *sdomain;
 
 error = error;  /* Keep clever compilers from complaining */
+valueptr = valueptr;
 
-DEBUG(D_lists) debug_printf("address match test: subject=%s pattern=%s\n",
+DEBUG(D_lists) debug_printf("address match: subject=%s pattern=%s\n",
   subject, pattern);
 
-/* Find the subject's domain */
-
-sdomain = Ustrrchr(subject, '@');
-
-/* The only case where a subject may not have a domain is if the subject is
-empty. Otherwise, a subject with no domain is a serious configuration error. */
-
-if (sdomain == NULL && *subject != 0)
-  {
-  log_write(0, LOG_MAIN|LOG_PANIC, "no @ found in the subject of an "
-    "address list match: subject=\"%s\" pattern=\"%s\"", subject, pattern);
-  return FAIL;
-  }
-
-/* Handle a regular expression, which must match the entire incoming address.
-This may be the empty address. */
+/* Handle a regular expression, which must match the entire incoming address. */
 
 if (*pattern == '^')
   return match_check_string(subject, pattern, cb->expand_setup, TRUE,
     cb->caseless, FALSE, NULL);
 
-/* Handle a pattern that is just a lookup. Skip over possible lookup names
-(letters, digits, hyphens). Skip over a possible * or *@ at the end. Then we
-must have a semicolon for it to be a lookup. */
+/* If the subject is the empty string, the only pattern it can match (other
+than a regular expression) is the empty pattern. */
+
+if (*subject == 0) return (*pattern == 0)? OK : FAIL;
+
+/* Find the domain in the subject */
+
+sdomain = Ustrrchr(subject, '@');
+
+/* Handle the case of a pattern that is just a lookup. Skip over possible
+lookup names (letters, digits, hyphens). Skip over a possible * or *@ at
+the end. Then we must have a semicolon for it to be a lookup. */
 
 for (s = pattern; isalnum(*s) || *s == '-'; s++);
 if (*s == '*') s++;
 if (*s == '@') s++;
 
-/* If it is a straight lookup, do a lookup for the whole address. This may be
-the empty address. Partial matching doesn't make sense here, so we ignore it,
-but write a panic log entry. However, *@ matching will be honoured. */
+/* If it is a straight lookup, do a lookup for the whole address. Partial
+matching doesn't make sense here, so we ignore it, but write a panic log entry.
+However, *@ matching will be honoured. */
 
 if (*s == ';')
   {
@@ -1056,15 +851,9 @@ if (*s == ';')
     log_write(0, LOG_MAIN|LOG_PANIC, "partial matching is not applicable to "
       "whole-address lookups: ignored \"partial-\" in \"%s\"", pattern);
   return match_check_string(subject, pattern, -1, FALSE, cb->caseless, FALSE,
-    valueptr);
+    NULL);
   }
 
-/* For the remaining cases, an empty subject matches only an empty pattern,
-because other patterns expect to have a local part and a domain to match
-against. */
-
-if (*subject == 0) return (*pattern == 0)? OK : FAIL;
-
 /* If the pattern starts with "@@" we have a split lookup, where the domain is
 looked up to obtain a list of local parts. If the subject's local part is just
 "*" (called from retry) the match always fails. */
@@ -1072,8 +861,7 @@ looked up to obtain a list of local parts. If the subject's local part is just
 if (pattern[0] == '@' && pattern[1] == '@')
   {
   int watchdog = 50;
-  const uschar *key;
-  uschar *list, *ss;
+  uschar *list, *key, *ss;
   uschar buffer[1024];
 
   if (sdomain == subject + 1 && *subject == '*') return FAIL;
@@ -1086,7 +874,7 @@ if (pattern[0] == '@' && pattern[1] == '@')
     int sep = 0;
 
     if ((rc = match_check_string(key, pattern + 2, -1, TRUE, FALSE, FALSE,
-      CUSS &list)) != OK) return rc;
+      &list)) != OK) return rc;
 
     /* Check for chaining from the last item; set up the next key if one
     is found. */
@@ -1105,7 +893,8 @@ if (pattern[0] == '@' && pattern[1] == '@')
     /* Look up the local parts provided by the list; negation is permitted.
     If a local part has to begin with !, a regex can be used. */
 
-    while ((ss = string_nextinlist(CUSS &list, &sep, buffer, sizeof(buffer))))
+    while ((ss = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
+           != NULL)
       {
       int local_yield;
 
@@ -1117,8 +906,7 @@ if (pattern[0] == '@' && pattern[1] == '@')
       else local_yield = OK;
 
       *sdomain = 0;
-      rc = match_check_string(subject, ss, -1, TRUE, cb->caseless, FALSE,
-        valueptr);
+      rc = match_check_string(subject, ss, -1, TRUE, cb->caseless, FALSE, NULL);
       *sdomain = '@';
 
       switch(rc)
@@ -1231,15 +1019,15 @@ listptr = (pdomain == NULL)? pattern : pdomain + 1;
 if (valueptr != NULL) *valueptr = NULL;
 
 return match_check_list(
-  &listptr,                  /* list of one item */
-  UCHAR_MAX+1,               /* impossible separator; single item */
-  &domainlist_anchor,        /* it's a domain list */
-  &null,                     /* ptr to NULL means no caching */
-  check_string,              /* the function to do one test */
-  &csb,                      /* its data */
-  MCL_DOMAIN + MCL_NOEXPAND, /* domain list; don't expand */
-  csb.subject,               /* string for messages */
-  valueptr);                 /* where to pass back lookup data */
+  &listptr,              /* list of one item */
+  UCHAR_MAX + 2,         /* impossible separator; don't expand */
+  &domainlist_anchor,    /* it's a domain list */
+  &null,                 /* ptr to NULL means no caching */
+  check_string,          /* the function to do one test */
+  &csb,                  /* its data */
+  MCL_DOMAIN,            /* type of list */
+  csb.subject,           /* string for messages */
+  valueptr);             /* where to pass back lookup data */
 }
 
 
@@ -1264,7 +1052,6 @@ callback to check_address(). During caseless matching, it will recognize
 Arguments:
   address         address to test
   caseless        TRUE to start in caseless state
-  expand          TRUE to allow list expansion
   listptr         list to check against
   cache_bits      points to cache bits for named address lists, or NULL
   expand_setup    controls setting up of $n variables - passed through
@@ -1272,17 +1059,15 @@ Arguments:
   sep             separator character for the list;
                   may be 0 to get separator from the list;
                   may be UCHAR_MAX+1 for one-item list
-  valueptr        where to return a lookup value, or NULL
 
 Returns:          OK    for a positive match, or end list after a negation;
                   FAIL  for a negative match, or end list after non-negation;
                   DEFER if a lookup deferred
 */
 
-int
-match_address_list(const uschar *address, BOOL caseless, BOOL expand,
-  const uschar **listptr, unsigned int *cache_bits, int expand_setup, int sep,
-  const uschar **valueptr)
+BOOL
+match_address_list(uschar *address, BOOL caseless,
+  uschar **listptr, unsigned int *cache_bits, int expand_setup, int sep)
 {
 uschar *p;
 check_address_block ab;
@@ -1322,8 +1107,7 @@ ab.expand_setup = expand_setup;
 ab.caseless = caseless;
 
 return match_check_list(listptr, sep, &addresslist_anchor, &local_cache_bits,
-  check_address, &ab, MCL_ADDRESS + (expand? 0:MCL_NOEXPAND), address,
-    valueptr);
+  check_address, &ab, MCL_ADDRESS, address, NULL);
 }
 
 /* End of match.c */
