@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -13,34 +13,19 @@ local_scan.h includes it and exim.h includes them both (to get this earlier). */
 #ifndef MYTYPES_H
 #define MYTYPES_H
 
-#ifndef FALSE
+
 #define FALSE         0
-#endif
-
-#ifndef TRUE
 #define TRUE          1
-#endif
-
-#ifndef TRUE_UNSET
 #define TRUE_UNSET    2
-#endif
 
 
 /* If gcc is being used to compile Exim, we can use its facility for checking
 the arguments of printf-like functions. This is done by a macro. */
 
-#if defined(__GNUC__) || defined(__clang__)
-#define PRINTF_FUNCTION(A,B)  __attribute__((format(printf,A,B)))
-#define ARG_UNUSED  __attribute__((__unused__))
+#ifdef __GNUC__
+#define PRINTF_FUNCTION  __attribute__((format(printf,1,2)))
 #else
-#define PRINTF_FUNCTION(A,B)
-#define ARG_UNUSED  /**/
-#endif
-
-#ifdef WANT_DEEPER_PRINTF_CHECKS
-#define ALMOST_PRINTF(A, B) PRINTF_FUNCTION(A, B)
-#else
-#define ALMOST_PRINTF(A, B)
+#define PRINTF_FUNCTION
 #endif
 
 
@@ -48,10 +33,8 @@ the arguments of printf-like functions. This is done by a macro. */
 the standard header files, so we use "uschar". Solaris has u_char in
 sys/types.h. This is just a typing convenience, of course. */
 
-typedef unsigned char uschar;
 typedef int BOOL;
-/* We also have SIGNAL_BOOL, which requires signal.h be included, so is defined
-elsewhere */
+typedef unsigned char uschar;
 
 
 /* These macros save typing for the casting that is needed to cope with the
@@ -66,7 +49,6 @@ almost always literal strings. */
 #define US   (unsigned char *)
 #define CUS  (const unsigned char *)
 #define USS  (unsigned char **)
-#define CUSS (const unsigned char **)
 
 /* The C library string functions expect "char *" arguments. Use macros to
 avoid having to write a cast each time. We do this for string and file
