@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #ifdef STAND_ALONE
@@ -71,7 +71,7 @@ printf("Used SIG_IGN\n");
 #endif
 }
 
-#endif  /* OS_RESTARTING_SIGNAL */
+#endif	/* OS_RESTARTING_SIGNAL */
 
 
 #ifndef OS_NON_RESTARTING_SIGNAL
@@ -131,7 +131,7 @@ printf("Used default signal()\n");
 #endif
 }
 
-#endif  /* OS_NON_RESTARTING_SIGNAL */
+#endif	/* OS_NON_RESTARTING_SIGNAL */
 
 
 
@@ -165,8 +165,8 @@ that have nothing. It provides a basic translation for the common standard
 signal numbers. I've been extra cautious with the ifdef's here. Probably more
 than is necessary... */
 
-const char *
-os_strsignal(const int n)
+char *
+os_strsignal(int n)
 {
 switch (n)
   {
@@ -282,8 +282,8 @@ switch (n)
 exit codes into text, but this function is implemented this way so that if any
 OS does have such a thing, it could be used instead of this build-in one. */
 
-const char *
-os_strexit(const int n)
+char *
+os_strexit(int n)
 {
 switch (n)
   {
@@ -461,75 +461,6 @@ calls the common function; on Linux it calls the Linux function.
 This function finds the addresses of all the running interfaces on the machine.
 A chain of blocks containing the textual form of the addresses is returned.
 
-getifaddrs() provides a sane consistent way to query this on modern OSs,
-otherwise fall back to a maze of twisty ioctl() calls
-
-Arguments:    none
-Returns:      a chain of ip_address_items, each pointing to a textual
-              version of an IP address, with the port field set to zero
-*/
-
-
-#ifndef NO_FIND_INTERFACES
-
-#ifdef HAVE_GETIFADDRS
-
-#include <ifaddrs.h>
-
-ip_address_item *
-os_common_find_running_interfaces(void)
-{
-struct ifaddrs *ifalist = NULL;
-ip_address_item *yield = NULL;
-ip_address_item *last = NULL;
-ip_address_item  *next;
-
-if (getifaddrs(&ifalist) != 0)
-  log_write(0, LOG_PANIC_DIE, "Unable to call getifaddrs: %d %s",
-    errno, strerror(errno));
-
-struct ifaddrs *ifa;
-for (ifa = ifalist; ifa != NULL; ifa = ifa->ifa_next)
-  {
-  if (ifa->ifa_addr->sa_family != AF_INET
-#if HAVE_IPV6
-    && ifa->ifa_addr->sa_family != AF_INET6
-#endif /* HAVE_IPV6 */
-    )
-    continue;
-
-  if ( !(ifa->ifa_flags & IFF_UP) ) /* Only want 'UP' interfaces */
-    continue;
-
-  /* Create a data block for the address, fill in the data, and put it on the
-  chain. */
-
-  next = store_get(sizeof(ip_address_item));
-  next->next = NULL;
-  next->port = 0;
-  (void)host_ntoa(-1, ifa->ifa_addr, next->address, NULL);
-
-  if (yield == NULL)
-    yield = last = next;
-  else
-    {
-    last->next = next;
-    last = next;
-    }
-
-  DEBUG(D_interface) debug_printf("Actual local interface address is %s (%s)\n",
-    last->address, ifa->ifa_name);
-  }
-
-/* free the list of addresses, and return the chain of data blocks. */
-
-freeifaddrs (ifalist);
-return yield;
-}
-
-#else /* HAVE_GETIFADDRS */
-
-/*
 Problems:
 
   (1) Solaris 2 has the SIOGIFNUM call to get the number of interfaces, but
@@ -553,8 +484,15 @@ Problems:
   the former, calling the latter does no harm, but it causes grief on Linux and
   BSD systems in the case of IP aliasing, so a means of cutting it out is
   provided.
+
+Arguments:    none
+Returns:      a chain of ip_address_items, each pointing to a textual
+              version of an IP address, with the port field set to zero
 */
 
+
+#ifndef NO_FIND_INTERFACES
+
 /* If there is IPv6 support, and SIOCGLIFCONF is defined, define macros to
 use these new, longer versions of the old IPv4 interfaces. Otherwise, define
 the macros to use the historical versions. */
@@ -616,7 +554,7 @@ char *cp;
 char buf[MAX_INTERFACES*sizeof(struct V_ifreq)];
 struct sockaddr *addrp;
 size_t len = 0;
-char addrbuf[512];
+char addrbuf[256];
 
 /* We have to create a socket in order to do ioctls on it to find out
 what we want to know. */
@@ -703,27 +641,16 @@ for (cp = buf; cp < buf + ifc.V_ifc_len; cp += len)
   #endif
     ) continue;
 
-  /* Get the interface flags, and if the interface is down, continue. Formerly,
-  we treated the inability to get the flags as a panic-die error. However, it
-  seems that on some OS (Solaris 9 being the case noted), it is possible to
-  have an interface in this list for which this call fails because the
-  interface hasn't been "plumbed" to any protocol (IPv4 or IPv6). Therefore,
-  we now just treat this case as "down" as well. */
+  /* Get the interface flags, and if the interface is down, continue */
 
   if (ioctl(vs, V_GIFFLAGS, (char *)&ifreq) < 0)
-    {
-    continue;
-    /*************
     log_write(0, LOG_PANIC_DIE, "Unable to get flags for %s interface: %d %s",
       ifreq.V_ifr_name, errno, strerror(errno));
-    *************/
-    }
   if ((ifreq.V_ifr_flags & IFF_UP) == 0) continue;
 
   /* On some operating systems we have to get the IP address of the interface
-  by another call. On others, it's already there, but we must copy the full
-  length because we only copied the basic length above, and anyway,
-  GIFFLAGS may have wrecked the data. */
+  by another call. On others, it's already there, but we must reinstate the
+  data in ifreq, because SIOCGIFFLAGS may wreck it. */
 
   #ifndef SIOCGIFCONF_GIVES_ADDR
   if (ioctl(vs, V_GIFADDR, (char *)&ifreq) < 0)
@@ -732,8 +659,8 @@ for (cp = buf; cp < buf + ifc.V_ifc_len; cp += len)
   addrp = &ifreq.V_ifr_addr;
 
   #else
-  memcpy(addrbuf, cp + offsetof(struct V_ifreq, V_ifr_addr),
-    len - sizeof(ifreq.V_ifr_name));
+  memcpy((char *)&ifreq, cp, sizeof(ifreq));
+  memcpy(addrbuf, (char *)&(ifreq.V_ifr_addr), len - sizeof(ifreq.V_ifr_name));
   addrp = (struct sockaddr *)addrbuf;
   #endif
 
@@ -757,12 +684,10 @@ for (cp = buf; cp < buf + ifc.V_ifc_len; cp += len)
 
 /* Close the socket, and return the chain of data blocks. */
 
-(void)close(vs);
+close(vs);
 return yield;
 }
 
-#endif /* HAVE_GETIFADDRS */
-
 #else  /* NO_FIND_INTERFACES */
 
 /* Some experimental or developing OS (e.g. GNU/Hurd) do not have the ioctls,
@@ -795,98 +720,6 @@ return yield;
 
 
 
-/* ----------------------------------------------------------------------- */
-
-/***********************************************************
-*                 DNS Resolver Base Finder                 *
-***********************************************************/
-
-/* We need to be able to set options for the system resolver(5), historically
-made available as _res.  At least one OS (NetBSD) now no longer provides this
-directly, instead making you call a function per thread to get a handle.
-Other OSs handle thread-safe resolver differently, in ways which fail if the
-programmer creates their own structs. */
-
-#if !defined(OS_GET_DNS_RESOLVER_RES) && !defined(COMPILE_UTILITY)
-
-#include <resolv.h>
-
-/* confirmed that res_state is typedef'd as a struct* on BSD and Linux, will
-find out how unportable it is on other OSes, but most resolver implementations
-should be descended from ISC's bind.
-
-Linux and BSD do:
-  define _res (*__res_state())
-identically.  We just can't rely on __foo functions.  It's surprising that use
-of _res has been as portable as it has, for so long.
-
-So, since _res works everywhere, and everything can decode the struct, I'm
-going to gamble that res_state is a typedef everywhere and use that as the
-return type.
-*/
-
-res_state
-os_get_dns_resolver_res(void)
-{
-  return &_res;
-}
-
-#endif /* OS_GET_DNS_RESOLVER_RES */
-
-/* ----------------------------------------------------------------------- */
-
-/***********************************************************
-*                 unsetenv()                               *
-***********************************************************/
-
-/* Most modern systems define int unsetenv(const char*),
-* some don't. */
-
-#if !defined(OS_UNSETENV)
-int
-os_unsetenv(const char *name)
-{
-  return unsetenv(name);
-}
-#endif
-
-/* ----------------------------------------------------------------------- */
-
-/***********************************************************
-*               getcwd()                                   *
-***********************************************************/
-
-/* Glibc allows getcwd(NULL, 0) to do auto-allocation. Some systems
-do auto-allocation, but need the size of the buffer, and others
-may not even do this. If the OS supports getcwd(NULL, 0) we'll use
-this, for all other systems we provide our own getcwd() */
-
-#if !defined(OS_GETCWD)
-char *
-os_getcwd(char *buffer, size_t size)
-{
-return getcwd(buffer, size);
-}
-#else
-#ifndef PATH_MAX
-# define PATH_MAX 4096
-#endif
-char *
-os_getcwd(char *buffer, size_t size)
-{
-void *rc;
-
-if (!size) size = PATH_MAX;
-if (!buffer && !(buffer = (char*) malloc(size))) return NULL;
-if (!(buffer = getcwd(buffer, size))) return NULL;
-return realloc(buffer, strlen(buffer) + 1);
-}
-#endif
-
-/* ----------------------------------------------------------------------- */
-
-
-
 
 /*************************************************
 **************************************************
