@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions that operate on the input queue. */
@@ -12,58 +12,6 @@
 
 
 
-/* The number of nodes to use for the bottom-up merge sort when a list of queue
-items is to be ordered. The code for this sort was contributed as a patch by
-Michael Haardt. */
-
-#define LOG2_MAXNODES 32
-
-
-
-/*************************************************
-*  Helper sort function for queue_get_spool_list *
-*************************************************/
-
-/* This function is used when sorting the queue list in the function
-queue_get_spool_list() below.
-
-Arguments:
-  a            points to an ordered list of queue_filename items
-  b            points to another ordered list
-
-Returns:       a pointer to a merged ordered list
-*/
-
-static queue_filename *
-merge_queue_lists(queue_filename *a, queue_filename *b)
-{
-queue_filename *first = NULL;
-queue_filename **append = &first;
-
-while (a != NULL && b != NULL)
-  {
-  if (Ustrcmp(a->text, b->text) < 0)
-    {
-    *append = a;
-    append= &a->next;
-    a = a->next;
-    }
-  else
-    {
-    *append = b;
-    append= &b->next;
-    b = b->next;
-    }
-  }
-
-*append=((a != NULL)? a : b);
-return first;
-}
-
-
-
-
-
 /*************************************************
 *             Get list of spool files            *
 *************************************************/
@@ -86,15 +34,14 @@ If the randomize argument is TRUE, messages are returned in "randomized" order.
 Actually, the order is anything but random, but the algorithm is cheap, and the
 point is simply to ensure that the same order doesn't occur every time, in case
 a particular message is causing a remote MTA to barf - we would like to try
-other messages to that MTA first.
+other messages to that MTA first. If the argument is FALSE, sort the list
+according to the file name. This should give the order in which the messages
+arrived. It is normally used only for presentation to humans, in which case the
+insertion sort that it does is not part of the normal operational code. This
+case also has the first argument as NULL, so all messages are included.
 
-If the randomize argument is FALSE, sort the list according to the file name.
-This should give the order in which the messages arrived. It is normally used
-only for presentation to humans, in which case the (possibly expensive) sort
-that it does is not part of the normal operational code. However, if
-queue_run_in_order is set, sorting has to take place for queue runs as well.
-When randomize is FALSE, the first argument is normally -1, so all messages are
-included.
+However, if the queue_run_in order is set, sorting happens, but for multiple
+directories will apply only per-directory.
 
 Arguments:
   subdiroffset   sub-directory character offset, or 0 or -1 (see above)
@@ -118,15 +65,12 @@ queue_filename *last = NULL;
 struct dirent *ent;
 DIR *dd;
 uschar buffer[256];
-queue_filename *root[LOG2_MAXNODES];
 
-/* When randomizing, the file names are added to the start or end of the list
-according to the bits of the flags variable. Get a collection of bits from the
-current time. Use the bottom 16 and just keep re-using them if necessary. When
-not randomizing, initialize the sublists for the bottom-up merge sort. */
+/* The file names are added onto the start or end of the list according to the
+bits of the flags variable. When randomizing, get a collection of bits from the
+current time. Use the bottom 16 and just keep re-using them if necessary. */
 
 if (randomize) resetflags = time(NULL) & 0xFFFF;
-  else for (i = 0; i < LOG2_MAXNODES; i++) root[i] = NULL;
 
 /* If processing the full queue, or just the top-level, start at the base
 directory, and initialize the first subdirectory name (as none). Otherwise,
@@ -195,56 +139,59 @@ for (; i <= *subcount; i++)
       Ustrcpy(next->text, name);
       next->dir_uschar = subdirchar;
 
-      /* Handle the creation of a randomized list. The first item becomes both
-      the top and bottom of the list. Subsequent items are inserted either at
-      the top or the bottom, randomly. This is, I argue, faster than doing a
-      sort by allocating a random number to each item, and it also saves having
-      to store the number with each item. */
+      /* First item becomes the top and bottom of the list. */
 
-      if (randomize)
+      if (yield == NULL)
         {
-        if (yield == NULL)
+        next->next = NULL;
+        yield = last = next;
+        }
+
+      /* If randomizing, insert at either top or bottom, randomly. This is, I
+      argue, faster than doing a sort by allocating a random number to each item,
+      and it also saves having to store the number with each item. */
+
+      else if (randomize)
+        {
+        if (flags == 0) flags = resetflags;
+        if ((flags & 1) == 0)
           {
-          next->next = NULL;
-          yield = last = next;
+          next->next = yield;
+          yield = next;
           }
         else
           {
-          if (flags == 0) flags = resetflags;
-          if ((flags & 1) == 0)
-            {
-            next->next = yield;
-            yield = next;
-            }
-          else
-            {
-            next->next = NULL;
-            last->next = next;
-            last = next;
-            }
-          flags = flags >> 1;
+          next->next = NULL;
+          last->next = next;
+          last = next;
           }
+        flags = flags >> 1;
         }
 
-      /* Otherwise do a bottom-up merge sort based on the name. */
+      /* Otherwise do an insertion sort based on the name. First see if
+      it should go before the first item. */
+
+      else if (Ustrcmp(next->text, yield->text) < 0)
+        {
+        next->next = yield;
+        yield = next;
+        }
+
+      /* Otherwise find the item it should go after; check the last one
+      first, because that will often be the case. */
 
       else
         {
-        int j;
-        next->next = NULL;
-        for (j = 0; j < LOG2_MAXNODES; j++)
+        queue_filename *this;
+        if (Ustrcmp(next->text, last->text) < 0)
           {
-          if (root[j] != NULL)
-            {
-            next = merge_queue_lists(next, root[j]);
-            root[j] = (j == LOG2_MAXNODES - 1)? next : NULL;
-            }
-          else
-            {
-            root[j] = next;
-            break;
-            }
+          for (this = yield; this != last; this = this->next)
+            if (Ustrcmp(next->text, this->next->text) < 0) break;
           }
+        else this = last;
+        next->next = this->next;
+        this->next = next;
+        if (this == last) last = next;
         }
       }
     }
@@ -278,14 +225,9 @@ for (; i <= *subcount; i++)
     {
     if (subdiroffset == 0) break;
     }
-  }    /* Loop for multiple subdirectories */
+  }
 
-/* When using a bottom-up merge sort, do the final merging of the sublists.
-Then pass back the final list of file items. */
-
-if (!randomize)
-  for (i = 0; i < LOG2_MAXNODES; ++i)
-    yield = merge_queue_lists(yield, root[i]);
+/* Pass back the list of file items */
 
 return yield;
 }
@@ -451,8 +393,7 @@ for (i  = (queue_run_in_order? -1 : 0);
       load_average = os_getloadavg();
       if (load_average > deliver_queue_load_max)
         {
-        log_write(L_queue_run, LOG_MAIN, "Abandon queue run: %s (load %.2f, max %.2f)",
-          log_detail,
+        log_write(0, LOG_MAIN, "abandon queue run (load %.2f, max %.2f)",
           (double)load_average/1000.0,
           (double)deliver_queue_load_max/1000.0);
         i = subcount;                 /* Don't process other directories */
@@ -593,7 +534,7 @@ for (i  = (queue_run_in_order? -1 : 0);
     if (queue_run_pipe == 0)
       {
       queue_run_pipe = dup(queue_run_pipe);
-      (void)close(0);
+      close(0);
       }
 
     /* Before forking to deliver the message, ensure any open and cached
@@ -607,17 +548,14 @@ for (i  = (queue_run_in_order? -1 : 0);
     search_tidyup();
 
     /* Now deliver the message; get the id by cutting the -H off the file
-    name. The return of the process is zero if a delivery was attempted. */
+    name. The return of the process is the yield of deliver_message(). */
 
     set_process_info("running queue: %s", f->text);
     f->text[SPOOL_NAME_LENGTH-2] = 0;
     if ((pid = fork()) == 0)
       {
-      int rc;
-      if (running_in_test_harness) millisleep(100);
       (void)close(pfd[pipe_read]);
-      rc = deliver_message(f->text, force_delivery, FALSE);
-      _exit(rc == DELIVER_NOT_ATTEMPTED);
+      _exit(deliver_message(f->text, force_delivery, FALSE));
       }
     if (pid < 0)
       log_write(0, LOG_MAIN|LOG_PANIC_DIE, "fork of delivery process from "
@@ -626,12 +564,12 @@ for (i  = (queue_run_in_order? -1 : 0);
     /* Close the writing end of the synchronizing pipe in this process,
     then wait for the first level process to terminate. */
 
-    (void)close(pfd[pipe_write]);
+    close(pfd[pipe_write]);
     set_process_info("running queue: waiting for %s (%d)", f->text, pid);
     while (wait(&status) != pid);
 
-    /* A zero return means a delivery was attempted; turn off the force flag
-    for any subsequent calls unless queue_force is set. */
+    /* A successful return means a delivery was attempted; turn off the
+    force flag for any subsequent calls unless queue_force is set. */
 
     if ((status & 0xffff) == 0) force_delivery = queue_run_force;
 
@@ -650,8 +588,7 @@ for (i  = (queue_run_in_order? -1 : 0);
     the mere fact that read() unblocks is enough. */
 
     set_process_info("running queue: waiting for children of %d", pid);
-    if (read(pfd[pipe_read], buffer, sizeof(buffer)) > 0)
-      log_write(0, LOG_MAIN|LOG_PANIC, "queue run: unexpected data on pipe");
+    (void)read(pfd[pipe_read], buffer, sizeof(buffer));
     (void)close(pfd[pipe_read]);
     set_process_info("running queue");
 
@@ -873,7 +810,7 @@ for (; f != NULL; f = f->next)
         big_buffer[n-1] = 0;
         tree_add_nonrecipient(big_buffer);
         }
-      (void)fclose(jread);
+      fclose(jread);
       }
     }
 
@@ -895,8 +832,10 @@ for (; f != NULL; f = f->next)
       sprintf(CS big_buffer, "%s/input/%s/%s", spool_directory, message_subdir,
         f->text);
       if (Ustat(big_buffer, &statbuf) == 0)
-        printf("*** spool format error: size=" OFF_T_FMT " ***",
-          statbuf.st_size);
+        {
+        int size = statbuf.st_size;    /* Because might be a long */
+        printf("*** spool format error: size=%d ***", size);
+        }
       else printf("*** spool format error ***");
       }
     else printf("*** spool read error: %s ***", strerror(save_errno));
@@ -935,9 +874,9 @@ for (; f != NULL; f = f->next)
 *             Act on a specific message          *
 *************************************************/
 
-/* Actions that require a list of addresses make use of argv/argc/
-recipients_arg. Other actions do not. This function does its own
-authority checking.
+/* Actions that require a list of addresses make use of
+argv/argc/recipients_arg. Other actions do not. This function does its
+own authority checking.
 
 Arguments:
   id              id of the message to work on
@@ -1019,9 +958,9 @@ if (action >= MSG_SHOW_BODY)
     }
 
   while((rc = read(fd, big_buffer, big_buffer_size)) > 0)
-    rc = write(fileno(stdout), big_buffer, rc);
+    write(fileno(stdout), big_buffer, rc);
 
-  (void)close(fd);
+  close(fd);
   return TRUE;
   }
 
@@ -1064,7 +1003,7 @@ if (spool_read_header(spoolname, TRUE, FALSE) != spool_read_OK)
     printf("Spool format error for %s\n", spoolname);
   if (action != MSG_REMOVE || !admin_user)
     {
-    (void)close(deliver_datafile);
+    close(deliver_datafile);
     deliver_datafile = -1;
     return FALSE;
     }
@@ -1079,7 +1018,7 @@ why we leave this check until after the headers are read. */
 if (!admin_user && (action != MSG_REMOVE || real_uid != originator_uid))
   {
   printf("Permission denied\n");
-  (void)close(deliver_datafile);
+  close(deliver_datafile);
   deliver_datafile = -1;
   return FALSE;
   }
@@ -1092,17 +1031,10 @@ username = (pw != NULL)?
 
 /* Take the necessary action. */
 
-if (action != MSG_SHOW_COPY) printf("Message %s ", id);
+printf("Message %s ", id);
 
 switch(action)
   {
-  case MSG_SHOW_COPY:
-  deliver_in_buffer = store_malloc(DELIVER_IN_BUFFER_SIZE);
-  deliver_out_buffer = store_malloc(DELIVER_OUT_BUFFER_SIZE);
-  transport_write_message(NULL, 1, 0, 0, NULL, NULL, NULL, NULL, NULL, 0);
-  break;
-
-
   case MSG_FREEZE:
   if (deliver_freeze)
     {
@@ -1196,11 +1128,7 @@ switch(action)
 
   if (deliver_datafile >= 0) printf("has been removed\n");
     else printf("has been removed or did not exist\n");
-  if (removed)
-    {
-    log_write(0, LOG_MAIN, "removed by %s", username);
-    log_write(0, LOG_MAIN, "Completed");
-    }
+  if (removed) log_write(0, LOG_MAIN, "removed by %s", username);
   break;
 
 
@@ -1274,9 +1202,6 @@ switch(action)
       {
       if (action == MSG_ADD_RECIPIENT)
         {
-#ifdef EXPERIMENTAL_INTERNATIONAL
-	if (string_is_utf8(recipient)) allow_utf8_domains = message_smtputf8 = TRUE;
-#endif
         receive_add_recipient(recipient, -1);
         log_write(0, LOG_MAIN, "recipient <%s> added by %s",
           recipient, username);
@@ -1300,9 +1225,6 @@ switch(action)
         }
       else  /* MSG_EDIT_SENDER */
         {
-#ifdef EXPERIMENTAL_INTERNATIONAL
-	if (string_is_utf8(recipient)) allow_utf8_domains = message_smtputf8 = TRUE;
-#endif
         sender_address = recipient;
         log_write(0, LOG_MAIN, "sender address changed to <%s> by %s",
           recipient, username);
@@ -1327,7 +1249,7 @@ switch(action)
 /* Closing the datafile releases the lock and permits other processes
 to operate on the message (if it still exists). */
 
-(void)close(deliver_datafile);
+close(deliver_datafile);
 deliver_datafile = -1;
 return yield;
 }
@@ -1351,8 +1273,7 @@ queue_check_only(void)
 BOOL *set;
 int sep = 0;
 struct stat statbuf;
-const uschar *s;
-uschar *ss, *name;
+uschar *s, *ss, *name;
 uschar buffer[1024];
 
 if (queue_only_file == NULL) return;
