@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This module contains code for extracting addresses from a forwarding list
@@ -297,17 +297,22 @@ if (fread(filebuf, 1, statbuf.st_size, fwd) != statbuf.st_size)
   }
 filebuf[statbuf.st_size] = 0;
 
+/* Don't pass statbuf.st_size directly to debug_printf. On some systems it
+is a long, which may not be the same as an int. */
+
 DEBUG(D_route)
-  debug_printf(OFF_T_FMT " bytes read from %s\n", statbuf.st_size, filename);
+  {
+  int size = (int)statbuf.st_size;
+  debug_printf("%d bytes read from %s\n", size, filename);
+  }
 
-(void)fclose(fwd);
 return filebuf;
 
 /* Return an error: the string is already set up. */
 
 ERROR_RETURN:
 *yield = FF_ERROR;
-(void)fclose(fwd);
+fclose(fwd);
 return NULL;
 }
 
@@ -321,30 +326,23 @@ return NULL;
 forwarding list, or to run a filter file and get addresses from there.
 
 Arguments:
-  rdata                     the redirection block
-  options                   the options bits
-  include_directory         restrain to this directory
-  sieve_vacation_directory  passed to sieve_interpret
-  sieve_enotify_mailto_owner passed to sieve_interpret
-  sieve_useraddress         passed to sieve_interpret
-  sieve_subaddress          passed to sieve_interpret
-  generated                 where to hang generated addresses
-  error                     for error messages
-  eblockp                   for details of skipped syntax errors
-                              (NULL => no skip)
-  filtertype                set to the filter type:
-                              FILTER_FORWARD => a traditional .forward file
-                              FILTER_EXIM    => an Exim filter file
-                              FILTER_SIEVE   => a Sieve filter file
-                            a system filter is always forced to be FILTER_EXIM
+  rdata               the redirection block
+  options             the options bits
+  include_directory   restrain to this directory
+  generated           where to hang generated addresses
+  error               for error messages
+  eblockp             for details of skipped syntax errors (NULL => no skip)
+  filtertype          set to the filter type:
+                        FILTER_FORWARD => a traditional .forward file
+                        FILTER_EXIM    => an Exim filter file
+                        FILTER_SIEVE   => a Sieve filter file
+                      a system filter is always forced to be FILTER_EXIM
 
-Returns:                    a suitable return for rda_interpret()
+Returns:              a suitable return for rda_interpret()
 */
 
 static int
 rda_extract(redirect_block *rdata, int options, uschar *include_directory,
-  uschar *sieve_vacation_directory, uschar *sieve_enotify_mailto_owner,
-  uschar *sieve_useraddress, uschar *sieve_subaddress,
   address_item **generated, uschar **error, error_block **eblockp,
   int *filtertype)
 {
@@ -352,7 +350,7 @@ uschar *data;
 
 if (rdata->isfile)
   {
-  int yield = 0;
+  int yield;
   data = rda_get_file_contents(rdata, options, error, &yield);
   if (data == NULL) return yield;
   }
@@ -361,9 +359,8 @@ else data = rdata->string;
 *filtertype = system_filtering? FILTER_EXIM : rda_is_filter(data);
 
 /* Filter interpretation is done by a general function that is also called from
-the filter testing option (-bf). There are two versions: one for Exim filtering
-and one for Sieve filtering. Several features of string expansion may be locked
-out at sites that don't trust users. This is done by setting flags in
+the filter testing option (-bf). Several features of string expansion may be
+locked out at sites that don't trust users. This is done by setting flags in
 expand_forbid that the expander inspects. */
 
 if (*filtertype != FILTER_FORWARD)
@@ -371,44 +368,20 @@ if (*filtertype != FILTER_FORWARD)
   int frc;
   int old_expand_forbid = expand_forbid;
 
-  DEBUG(D_route) debug_printf("data is %s filter program\n",
-    (*filtertype == FILTER_EXIM)? "an Exim" : "a Sieve");
-
-  /* RDO_FILTER is an "allow" bit */
-
   if ((options & RDO_FILTER) == 0)
     {
     *error = US"filtering not enabled";
     return FF_ERROR;
     }
 
+  DEBUG(D_route) debug_printf("data is %s filter program\n",
+    (*filtertype == FILTER_EXIM)? "an Exim" : "a Sieve");
+
   expand_forbid =
     (expand_forbid & ~RDO_FILTER_EXPANSIONS) |
     (options & RDO_FILTER_EXPANSIONS);
-
-  /* RDO_{EXIM,SIEVE}_FILTER are forbid bits */
-
-  if (*filtertype == FILTER_EXIM)
-    {
-    if ((options & RDO_EXIM_FILTER) != 0)
-      {
-      *error = US"Exim filtering not enabled";
-      return FF_ERROR;
-      }
-    frc = filter_interpret(data, options, generated, error);
-    }
-  else
-    {
-    if ((options & RDO_SIEVE_FILTER) != 0)
-      {
-      *error = US"Sieve filtering not enabled";
-      return FF_ERROR;
-      }
-    frc = sieve_interpret(data, options, sieve_vacation_directory,
-      sieve_enotify_mailto_owner, sieve_useraddress, sieve_subaddress,
-      generated, error);
-    }
-
+  frc = ((*filtertype == FILTER_EXIM)? filter_interpret : sieve_interpret)
+    (data, options, generated, error);
   expand_forbid = old_expand_forbid;
   return frc;
   }
@@ -433,24 +406,22 @@ return parse_forward_list(data,
 *         Write string down pipe                 *
 *************************************************/
 
-/* This function is used for transferring a string down a pipe between
+/* This function is used for tranferring a string down a pipe between
 processes. If the pointer is NULL, a length of zero is written.
 
 Arguments:
   fd         the pipe
   s          the string
 
-Returns:     -1 on error, else 0
+Returns:     nothing
 */
 
-static int
-rda_write_string(int fd, const uschar *s)
+static void
+rda_write_string(int fd, uschar *s)
 {
 int len = (s == NULL)? 0 : Ustrlen(s) + 1;
-return (  write(fd, &len, sizeof(int)) != sizeof(int)
-       || (s != NULL  &&  write(fd, s, len) != len)
-       )
-       ? -1 : 0;
+write(fd, &len, sizeof(int));
+if (s != NULL) write(fd, s, len);
 }
 
 
@@ -474,12 +445,11 @@ rda_read_string(int fd, uschar **sp)
 int len;
 
 if (read(fd, &len, sizeof(int)) != sizeof(int)) return FALSE;
-if (len == 0)
-  *sp = NULL;
-else
-  /* We know we have enough memory so disable the error on "len" */
-  /* coverity[tainted_data] */
-  if (read(fd, *sp = store_get(len), len) != len) return FALSE;
+if (len == 0) *sp = NULL; else
+  {
+  *sp = store_get(len);
+  if (read(fd, *sp, len) != len) return FALSE;
+  }
 return TRUE;
 }
 
@@ -511,25 +481,21 @@ sub-process is NOT used when:
     process (which is usually running as root).
 
 Arguments:
-  rdata                     redirect data (file + constraints, or data string)
-  options                   options to pass to the extraction functions,
-                              plus ENOTDIR and EACCES handling bits
-  include_directory         restrain :include: to this directory
-  sieve_vacation_directory  directory passed to sieve_interpret
-  sieve_enotify_mailto_owner passed to sieve_interpret
-  sieve_useraddress         passed to sieve_interpret
-  sieve_subaddress          passed to sieve_interpret
-  ugid                      uid/gid to run under - if NULL, no change
-  generated                 where to hang generated addresses, initially NULL
-  error                     pointer for error message
-  eblockp                   for skipped syntax errors; NULL if no skipping
-  filtertype                set to the type of file:
-                              FILTER_FORWARD => traditional .forward file
-                              FILTER_EXIM    => an Exim filter file
-                              FILTER_SIEVE   => a Sieve filter file
-                            a system filter is always forced to be FILTER_EXIM
-  rname                     router name for error messages in the format
-                              "xxx router" or "system filter"
+  rdata              redirect data (file + constraints, or data string)
+  options            options to pass to the extraction functions,
+                       plus ENOTDIR and EACCES handling bits
+  include_directory  restrain :include: to this directory
+  ugid               uid/gid to run under - if NULL, no change
+  generated          where to hang generated addresses, initially NULL
+  error              pointer for error message
+  eblockp            for skipped syntax errors; NULL if no skipping
+  filtertype         set to the type of file:
+                       FILTER_FORWARD => traditional .forward file
+                       FILTER_EXIM    => an Exim filter file
+                       FILTER_SIEVE   => a Sieve filter file
+                     a system filter is always forced to be FILTER_EXIM
+  rname              router name for error messages in the format
+                       "xxx router" or "system filter"
 
 Returns:        values from extraction function, or FF_NONEXIST:
                   FF_DELIVERED     success, a significant action was taken
@@ -545,17 +511,13 @@ Returns:        values from extraction function, or FF_NONEXIST:
 
 int
 rda_interpret(redirect_block *rdata, int options, uschar *include_directory,
-  uschar *sieve_vacation_directory, uschar *sieve_enotify_mailto_owner,
-  uschar *sieve_useraddress, uschar *sieve_subaddress, ugid_block *ugid,
-  address_item **generated, uschar **error, error_block **eblockp,
-  int *filtertype, uschar *rname)
+  ugid_block *ugid, address_item **generated, uschar **error,
+  error_block **eblockp, int *filtertype, uschar *rname)
 {
 int fd, rc, pfd[2];
 int yield, status;
-BOOL had_disaster = FALSE;
 pid_t pid;
 uschar *data;
-uschar *readerror = US"";
 void (*oldsignal)(int);
 
 DEBUG(D_route) debug_printf("rda_interpret (%s): %s\n",
@@ -591,9 +553,8 @@ if (!ugid->uid_set ||                         /* Either there's no uid, or */
      rda_is_filter(data) == FILTER_FORWARD && /* It's not a filter script, */
      Ustrstr(data, ":include:") == NULL))     /* and there's no :include: */
   {
-  return rda_extract(rdata, options, include_directory,
-    sieve_vacation_directory, sieve_enotify_mailto_owner, sieve_useraddress,
-    sieve_subaddress, generated, error, eblockp, filtertype);
+  return rda_extract(rdata, options, include_directory, generated, error,
+    eblockp, filtertype);
   }
 
 /* We need to run the processing code in a sub-process. However, if we can
@@ -613,19 +574,15 @@ if (pipe(pfd) != 0)
 
 /* Ensure that SIGCHLD is set to SIG_DFL before forking, so that the child
 process can be waited for. We sometimes get here with it set otherwise. Save
-the old state for resetting on the wait. Ensure that all cached resources are
-freed so that the subprocess starts with a clean slate and doesn't interfere
-with the parent process. */
+the old state for resetting on the wait. */
 
 oldsignal = signal(SIGCHLD, SIG_DFL);
-search_tidyup();
-
 if ((pid = fork()) == 0)
   {
   header_line *waslast = header_last;   /* Save last header */
 
   fd = pfd[pipe_write];
-  (void)close(pfd[pipe_read]);
+  close(pfd[pipe_read]);
   exim_setugid(ugid->uid, ugid->gid, FALSE, rname);
 
   /* Addresses can get rewritten in filters; if we are not root or the exim
@@ -636,23 +593,20 @@ if ((pid = fork()) == 0)
     {
     DEBUG(D_rewrite) debug_printf("turned off address rewrite logging (not "
       "root or exim in this process)\n");
-    BIT_CLEAR(log_selector, log_selector_size, Li_address_rewrite);
+    log_selector &= ~L_address_rewrite;
     }
 
   /* Now do the business */
 
-  yield = rda_extract(rdata, options, include_directory,
-    sieve_vacation_directory, sieve_enotify_mailto_owner, sieve_useraddress,
-    sieve_subaddress, generated, error, eblockp, filtertype);
+  yield = rda_extract(rdata, options, include_directory, generated, error,
+    eblockp, filtertype);
 
   /* Pass back whether it was a filter, and the return code and any overall
   error text via the pipe. */
 
-  if (  write(fd, filtertype, sizeof(int)) != sizeof(int)
-     || write(fd, &yield, sizeof(int)) != sizeof(int)
-     || rda_write_string(fd, *error) != 0
-     )
-    goto bad;
+  write(fd, filtertype, sizeof(int));
+  write(fd, &yield, sizeof(int));
+  rda_write_string(fd, *error);
 
   /* Pass back the contents of any syntax error blocks if we have a pointer */
 
@@ -660,12 +614,11 @@ if ((pid = fork()) == 0)
     {
     error_block *ep;
     for (ep = *eblockp; ep != NULL; ep = ep->next)
-      if (  rda_write_string(fd, ep->text1) != 0
-         || rda_write_string(fd, ep->text2) != 0
-	 )
-	goto bad;
-    if (rda_write_string(fd, NULL) != 0)    /* Indicates end of eblocks */
-      goto bad;
+      {
+      rda_write_string(fd, ep->text1);
+      rda_write_string(fd, ep->text2);
+      }
+    rda_write_string(fd, NULL);    /* Indicates end of eblocks */
     }
 
   /* If this is a system filter, we have to pass back the numbers of any
@@ -677,34 +630,24 @@ if ((pid = fork()) == 0)
     int i = 0;
     header_line *h;
     for (h = header_list; h != waslast->next; i++, h = h->next)
-      if (  h->type == htype_old
-         && write(fd, &i, sizeof(i)) != sizeof(i)
-	 )
-	goto bad;
-
+      {
+      if (h->type == htype_old) write(fd, &i, sizeof(i));
+      }
     i = -1;
-    if (write(fd, &i, sizeof(i)) != sizeof(i))
-	goto bad;
+    write(fd, &i, sizeof(i));
 
     while (waslast != header_last)
       {
       waslast = waslast->next;
       if (waslast->type != htype_old)
-	if (  rda_write_string(fd, waslast->text) != 0
-           || write(fd, &(waslast->type), sizeof(waslast->type))
-	      != sizeof(waslast->type)
-	   )
-	  goto bad;
+        {
+        rda_write_string(fd, waslast->text);
+        write(fd, &(waslast->type), sizeof(waslast->type));
+        }
       }
-    if (rda_write_string(fd, NULL) != 0)    /* Indicates end of added headers */
-      goto bad;
+    rda_write_string(fd, NULL);    /* Indicates end of added headers */
     }
 
-  /* Write the contents of the $n variables */
-
-  if (write(fd, filter_n, sizeof(filter_n)) != sizeof(filter_n))
-    goto bad;
-
   /* If the result was DELIVERED or NOTDELIVERED, we pass back the generated
   addresses, and their associated information, through the pipe. This is
   just tedious, but it seems to be the only safe way. We do this also for
@@ -719,71 +662,50 @@ if ((pid = fork()) == 0)
       {
       int reply_options = 0;
 
-      if (  rda_write_string(fd, addr->address) != 0
-         || write(fd, &(addr->mode), sizeof(addr->mode))
-	    != sizeof(addr->mode)
-         || write(fd, &(addr->flags), sizeof(addr->flags))
-	    != sizeof(addr->flags)
-         || rda_write_string(fd, addr->prop.errors_address) != 0
-	 )
-	goto bad;
+      rda_write_string(fd, addr->address);
+      write(fd, &(addr->mode), sizeof(addr->mode));
+      write(fd, &(addr->flags), sizeof(addr->flags));
+      rda_write_string(fd, addr->p.errors_address);
 
       if (addr->pipe_expandn != NULL)
         {
         uschar **pp;
         for (pp = addr->pipe_expandn; *pp != NULL; pp++)
-          if (rda_write_string(fd, *pp) != 0)
-	    goto bad;
+          rda_write_string(fd, *pp);
         }
-      if (rda_write_string(fd, NULL) != 0)
-        goto bad;
+      rda_write_string(fd, NULL);
 
       if (addr->reply == NULL)
-	{
-        if (write(fd, &reply_options, sizeof(int)) != sizeof(int))    /* 0 means no reply */
-	  goto bad;
-	}
+        write(fd, &reply_options, sizeof(int));    /* 0 means no reply */
       else
         {
         reply_options |= REPLY_EXISTS;
         if (addr->reply->file_expand) reply_options |= REPLY_EXPAND;
         if (addr->reply->return_message) reply_options |= REPLY_RETURN;
-        if (  write(fd, &reply_options, sizeof(int)) != sizeof(int)
-           || write(fd, &(addr->reply->expand_forbid), sizeof(int))
-	      != sizeof(int)
-           || write(fd, &(addr->reply->once_repeat), sizeof(time_t))
-	      != sizeof(time_t)
-           || rda_write_string(fd, addr->reply->to) != 0
-           || rda_write_string(fd, addr->reply->cc) != 0
-           || rda_write_string(fd, addr->reply->bcc) != 0
-           || rda_write_string(fd, addr->reply->from) != 0
-           || rda_write_string(fd, addr->reply->reply_to) != 0
-           || rda_write_string(fd, addr->reply->subject) != 0
-           || rda_write_string(fd, addr->reply->headers) != 0
-           || rda_write_string(fd, addr->reply->text) != 0
-           || rda_write_string(fd, addr->reply->file) != 0
-           || rda_write_string(fd, addr->reply->logfile) != 0
-           || rda_write_string(fd, addr->reply->oncelog) != 0
-	   )
-	  goto bad;
+        write(fd, &reply_options, sizeof(int));
+        write(fd, &(addr->reply->expand_forbid), sizeof(int));
+        write(fd, &(addr->reply->once_repeat), sizeof(time_t));
+        rda_write_string(fd, addr->reply->to);
+        rda_write_string(fd, addr->reply->cc);
+        rda_write_string(fd, addr->reply->bcc);
+        rda_write_string(fd, addr->reply->from);
+        rda_write_string(fd, addr->reply->reply_to);
+        rda_write_string(fd, addr->reply->subject);
+        rda_write_string(fd, addr->reply->headers);
+        rda_write_string(fd, addr->reply->text);
+        rda_write_string(fd, addr->reply->file);
+        rda_write_string(fd, addr->reply->logfile);
+        rda_write_string(fd, addr->reply->oncelog);
         }
       }
 
-    if (rda_write_string(fd, NULL) != 0)   /* Marks end of addresses */
-      goto bad;
+    rda_write_string(fd, NULL);   /* Marks end of addresses */
     }
 
-  /* OK, this process is now done. Free any cached resources. Must use _exit()
-  and not exit() !! */
+  /* OK, this process is now done. Must use _exit() and not exit() !! */
 
-out:
-  (void)close(fd);
-  search_tidyup();
-  _exit(0);
-
-bad:
-  DEBUG(D_rewrite) debug_printf("rda_interpret: failed write to pipe\n");
-  goto out;
+  close(fd);
+  _exit(yield);
   }
 
 /* Back in the main process: panic if the fork did not succeed. */
@@ -795,7 +717,7 @@ if (pid < 0)
 writing end must be closed first, as otherwise read() won't return zero on an
 empty pipe. Afterwards, close the reading end. */
 
-(void)close(pfd[pipe_write]);
+close(pfd[pipe_write]);
 
 /* Read initial data, including yield and contents of *error */
 
@@ -804,6 +726,9 @@ if (read(fd, filtertype, sizeof(int)) != sizeof(int) ||
     read(fd, &yield, sizeof(int)) != sizeof(int) ||
     !rda_read_string(fd, error)) goto DISASTER;
 
+DEBUG(D_route)
+  debug_printf("rda_interpret: subprocess yield=%d error=%s\n", yield, *error);
+
 /* Read the contents of any syntax error blocks if we have a pointer */
 
 if (eblockp != NULL)
@@ -842,7 +767,7 @@ if (system_filtering)
       {
       hn++;
       h = h->next;
-      if (h == NULL) goto DISASTER_NO_HEADER;
+      if (h == NULL) goto DISASTER;
       }
     h->type = htype_old;
     }
@@ -858,10 +783,6 @@ if (system_filtering)
     }
   }
 
-/* Read the values of the $n variables */
-
-if (read(fd, filter_n, sizeof(filter_n)) != sizeof(filter_n)) goto DISASTER;
-
 /* If the yield is DELIVERED, NOTDELIVERED, FAIL, or FREEZE there may follow
 addresses and data to go with them. Keep them in the same order in the
 generated chain. */
@@ -893,7 +814,7 @@ if (yield == FF_DELIVERED || yield == FF_NOTDELIVERED ||
 
     if (read(fd, &(addr->mode), sizeof(addr->mode)) != sizeof(addr->mode) ||
         read(fd, &(addr->flags), sizeof(addr->flags)) != sizeof(addr->flags) ||
-        !rda_read_string(fd, &(addr->prop.errors_address))) goto DISASTER;
+        !rda_read_string(fd, &(addr->p.errors_address))) goto DISASTER;
 
     /* Next comes a possible setting for $thisaddress and any numerical
     variables for pipe expansion, terminated by a NULL string. The maximum
@@ -912,7 +833,7 @@ if (yield == FF_DELIVERED || yield == FF_NOTDELIVERED ||
 
     if (i > 0)
       {
-      addr->pipe_expandn = store_get((i+1) * sizeof(uschar *));
+      addr->pipe_expandn = store_get((i+1) * sizeof(uschar **));
       addr->pipe_expandn[i] = NULL;
       while (--i >= 0) addr->pipe_expandn[i] = expandn[i];
       }
@@ -956,49 +877,20 @@ while ((rc = wait(&status)) != pid)
   if (rc < 0 && errno == ECHILD)      /* Process has vanished */
     {
     log_write(0, LOG_MAIN, "redirection process %d vanished unexpectedly", pid);
-    goto FINAL_EXIT;
+    break;
     }
   }
 
-DEBUG(D_route)
-  debug_printf("rda_interpret: subprocess yield=%d error=%s\n", yield, *error);
-
-if (had_disaster)
-  {
-  *error = string_sprintf("internal problem in %s: failure to transfer "
-    "data from subprocess: status=%04x%s%s%s", rname,
-    status, readerror,
-    (*error == NULL)? US"" : US": error=",
-    (*error == NULL)? US"" : *error);
-  log_write(0, LOG_MAIN|LOG_PANIC, "%s", *error);
-  }
-else if (status != 0)
-  {
-  log_write(0, LOG_MAIN|LOG_PANIC, "internal problem in %s: unexpected status "
-    "%04x from redirect subprocess (but data correctly received)", rname,
-    status);
-  }
-
-FINAL_EXIT:
-(void)close(fd);
+close(fd);
 signal(SIGCHLD, oldsignal);   /* restore */
 return yield;
 
-
-/* Come here if the data indicates removal of a header that we can't find */
-
-DISASTER_NO_HEADER:
-readerror = US" readerror=bad header identifier";
-had_disaster = TRUE;
-yield = FF_ERROR;
-goto WAIT_EXIT;
-
-/* Come here is there's a shambles in transferring the data over the pipe. The
-value of errno should still be set. */
+/* Come here is there's a shambles in transferring the data over the pipe. */
 
 DISASTER:
-readerror = string_sprintf(" readerror='%s'", strerror(errno));
-had_disaster = TRUE;
+*error = string_sprintf("internal problem in %s: failure to transfer "
+  "data from subprocess", rname);
+log_write(0, LOG_MAIN|LOG_PANIC, "%s", *error);
 yield = FF_ERROR;
 goto WAIT_EXIT;
 }
