@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* This file contains a function for decoding message header lines that may
@@ -144,13 +144,7 @@ for (;; string = mimeword + 2)
 *    Decode and convert an RFC 2047 string       *
 *************************************************/
 
-/* There are two functions defined here. The original one was rfc2047_decode()
-and it was documented in the local_scan() interface. I needed to add an extra
-argument for use by expand_string(), so I created rfc2047_decode2() for that
-purpose. The original function became a stub that just supplies NULL for the
-new argument (sizeptr).
-
-An RFC 2047-encoded string may contain one or more "words", each of the
+/* An RFC 2047-encoded string may contain one or more "words", each of the
 form  =?...?.?...?=  with the first ... specifying the character code, the
 second being Q (for quoted printable) or B for Base64 encoding. The third ...
 is the actual data.
@@ -173,9 +167,6 @@ Arguments:
     zeroval      the value to use for binary zero bytes
     lenptr       if not NULL, the length of the result is returned via
                    this variable
-    sizeptr      if not NULL, the length of a new store block in which the
-                   result is built is placed here; if no new store is obtained,
-                   the value is not changed
     error        for error messages; NULL if no problem; this can be set
                    when the yield is non-NULL if there was a charset
                    translation problem
@@ -185,8 +176,8 @@ Returns:         the decoded, converted string, or NULL on error; if there are
 */
 
 uschar *
-rfc2047_decode2(uschar *string, BOOL lencheck, uschar *target, int zeroval,
-  int *lenptr, int *sizeptr, uschar **error)
+rfc2047_decode(uschar *string, BOOL lencheck, uschar *target, int zeroval, int
+  *lenptr, uschar **error)
 {
 int ptr = 0;
 int size = Ustrlen(string);
@@ -277,7 +268,7 @@ while (mimeword != NULL)
       else
         {
         DEBUG(D_any) debug_printf("iconv error translating \"%.*s\" to %s: "
-        "%s\n", (int)(endword + 2 - mimeword), mimeword, target, strerror(errno));
+        "%s\n", endword + 2 - mimeword, mimeword, target, strerror(errno));
         }
       }
 
@@ -331,19 +322,7 @@ the length as well if requested. */
 yield = string_cat(yield, &size, &ptr, string, Ustrlen(string));
 yield[ptr] = 0;
 if (lenptr != NULL) *lenptr = ptr;
-if (sizeptr != NULL) *sizeptr = size;
 return yield;
 }
 
-
-/* This is the stub that provides the original interface without the sizeptr
-argument. */
-
-uschar *
-rfc2047_decode(uschar *string, BOOL lencheck, uschar *target, int zeroval,
-  int *lenptr, uschar **error)
-{
-return rfc2047_decode2(string, lencheck, target, zeroval, lenptr, NULL, error);
-}
-
 /* End of rfc2047.c */
