@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -14,9 +14,6 @@ typedef struct {
   BOOL rewrite_headers;
   uschar *widen_domains;
   uschar *mx_domains;
-  uschar *mx_fail_domains;
-  uschar *srv_fail_domains;
-  uschar *check_srv;
 } dnslookup_router_options_block;
 
 /* Data for reading the private options. */
@@ -31,7 +28,7 @@ extern dnslookup_router_options_block dnslookup_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int dnslookup_router_entry(router_instance *, address_item *,
-  struct passwd *, int, address_item **, address_item **,
+  struct passwd *, BOOL, address_item **, address_item **,
   address_item **, address_item **);
 
 extern void dnslookup_router_init(router_instance *);
