@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -26,7 +26,7 @@ extern ipliteral_router_options_block ipliteral_router_option_defaults;
 /* The main and initialization entry points for the router */
 
 extern int ipliteral_router_entry(router_instance *, address_item *,
-  struct passwd *, int, address_item **, address_item **,
+  struct passwd *, BOOL, address_item **, address_item **,
   address_item **, address_item **);
 
 extern void ipliteral_router_init(router_instance *);
