@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -135,16 +135,15 @@ iplookup_router_entry(
   router_instance *rblock,        /* data for this instantiation */
   address_item *addr,             /* address we are working on */
   struct passwd *pw,              /* passwd entry after check_local_user */
-  int verify,                     /* v_none/v_recipient/v_sender/v_expn */
+  BOOL verify,                    /* TRUE when verifying */
   address_item **addr_local,      /* add it to this if it's local */
   address_item **addr_remote,     /* add it to this if it's remote */
   address_item **addr_new,        /* put new addresses on here */
   address_item **addr_succeed)    /* put old address here on success */
 {
 uschar *query = NULL;
-uschar *reply;
-uschar *hostname, *reroute, *domain;
-const uschar *listptr;
+uschar reply[256];
+uschar *hostname, *reroute, *domain, *listptr;
 uschar host_buffer[256];
 host_item *host = store_get(sizeof(host_item));
 address_item *new_addr;
@@ -162,8 +161,6 @@ pw = pw;
 DEBUG(D_route) debug_printf("%s router called for %s: domain = %s\n",
   rblock->name, addr->address, addr->domain);
 
-reply = store_get(256);
-
 /* Build the query string to send. If not explicitly given, a default of
 "user@domain user@domain" is used. */
 
@@ -199,16 +196,14 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
 
   host->name = hostname;
   host->address = NULL;
-  host->port = PORT_NONE;
   host->mx = MX_NONE;
   host->next = NULL;
 
-  if (string_is_ip_address(host->name, NULL) != 0)
+  if (string_is_ip_address(host->name, NULL))
     host->address = host->name;
   else
     {
-/*XXX might want dnssec request/require on an iplookup router? */
-    int rc = host_find_byname(host, NULL, HOST_FIND_QUALIFY_SINGLE, NULL, TRUE);
+    int rc = host_find_byname(host, NULL, NULL, TRUE);
     if (rc == HOST_FIND_FAILED || rc == HOST_FIND_AGAIN) continue;
     }
 
@@ -242,7 +237,6 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
 
     if (ip_connect(query_socket, host_af, h->address,ob->port, ob->timeout) < 0)
       {
-      close(query_socket);
       DEBUG(D_route)
         debug_printf("connection to %s failed: %s\n", h->address,
           strerror(errno));
@@ -254,7 +248,7 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
     if (send(query_socket, query, query_len, 0) < 0)
       {
       DEBUG(D_route) debug_printf("send to %s failed\n", h->address);
-      (void)close(query_socket);
+      close(query_socket);
       continue;
       }
 
@@ -262,7 +256,7 @@ while ((hostname = string_nextinlist(&listptr, &sep, host_buffer,
     next IP address. */
 
     count = ip_recv(query_socket, reply, sizeof(reply) - 1, ob->timeout);
-    (void)close(query_socket);
+    close(query_socket);
     if (count <= 0)
       {
       DEBUG(D_route) debug_printf("%s from %s\n", (errno == ETIMEDOUT)?
@@ -378,11 +372,8 @@ new_addr = deliver_make_addr(reroute, TRUE);
 new_addr->parent = addr;
 
 copyflag(new_addr, addr, af_propagate);
-new_addr->prop = addr->prop;
+new_addr->p = addr->p;
 
-if (addr->child_count == SHRT_MAX)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s router generated more than %d "
-    "child addresses for <%s>", rblock->name, SHRT_MAX, addr->address);
 addr->child_count++;
 new_addr->next = *addr_new;
 *addr_new = new_addr;
@@ -390,11 +381,11 @@ new_addr->next = *addr_new;
 /* Set up the errors address, if any, and the additional and removeable headers
 for this new address. */
 
-rc = rf_get_errors_address(addr, rblock, verify, &new_addr->prop.errors_address);
+rc = rf_get_errors_address(addr, rblock, verify, &(new_addr->p.errors_address));
 if (rc != OK) return rc;
 
-rc = rf_get_munge_headers(addr, rblock, &new_addr->prop.extra_headers,
-  &new_addr->prop.remove_headers);
+rc = rf_get_munge_headers(addr, rblock, &(new_addr->p.extra_headers),
+  &(new_addr->p.remove_headers));
 if (rc != OK) return rc;
 
 return OK;
