@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -14,8 +14,6 @@
 /* Options specific to the manualroute router. */
 
 optionlist manualroute_router_options[] = {
-  { "host_all_ignored", opt_stringptr,
-      (void *)(offsetof(manualroute_router_options_block, host_all_ignored)) },
   { "host_find_failed", opt_stringptr,
       (void *)(offsetof(manualroute_router_options_block, host_find_failed)) },
   { "hosts_randomize",  opt_bool,
@@ -37,30 +35,14 @@ int manualroute_router_options_count =
 /* Default private options block for the manualroute router. */
 
 manualroute_router_options_block manualroute_router_option_defaults = {
-  -1,           /* host_all_ignored code (unset) */
-  -1,           /* host_find_failed code (unset) */
+  hff_freeze,   /* host_find_failed code */
   FALSE,        /* hosts_randomize */
-  US"defer",    /* host_all_ignored */
   US"freeze",   /* host_find_failed */
   NULL,         /* route_data */
   NULL          /* route_list */
 };
 
 
-/* Names and values for host_find_failed and host_all_ignored.  */
-
-static uschar *hff_names[] = {
-  US"ignore",  /* MUST be first - not valid for host_all_ignored */
-  US"decline",
-  US"defer",
-  US"fail",
-  US"freeze",
-  US"pass" };
-
-static int hff_codes[] = { hff_ignore, hff_decline, hff_defer, hff_fail,
-  hff_freeze, hff_pass };
-
-static int hff_count= sizeof(hff_codes)/sizeof(int);
 
 
 
@@ -76,34 +58,23 @@ manualroute_router_init(router_instance *rblock)
 {
 manualroute_router_options_block *ob =
   (manualroute_router_options_block *)(rblock->options_block);
-int i;
 
 /* Host_find_failed must be a recognized word */
 
-for (i = 0; i < hff_count; i++)
-  {
-  if (Ustrcmp(ob->host_find_failed, hff_names[i]) == 0)
-    {
-    ob->hff_code = hff_codes[i];
-    break;
-    }
-  }
-if (ob->hff_code < 0)
+if      (Ustrcmp(ob->host_find_failed, "freeze") == 0)
+  ob->hff_code = hff_freeze;
+else if (Ustrcmp(ob->host_find_failed, "decline") == 0)
+  ob->hff_code = hff_decline;
+else if (Ustrcmp(ob->host_find_failed, "defer") == 0)
+  ob->hff_code = hff_defer;
+else if (Ustrcmp(ob->host_find_failed, "pass") == 0)
+  ob->hff_code = hff_pass;
+else if (Ustrcmp(ob->host_find_failed, "fail") == 0)
+  ob->hff_code = hff_fail;
+else
   log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
     "unrecognized setting for host_find_failed option", rblock->name);
 
-for (i = 1; i < hff_count; i++)   /* NB starts at 1 to skip "ignore" */
-  {
-  if (Ustrcmp(ob->host_all_ignored, hff_names[i]) == 0)
-    {
-    ob->hai_code = hff_codes[i];
-    break;
-    }
-  }
-if (ob->hai_code < 0)
-  log_write(0, LOG_PANIC_DIE|LOG_CONFIG_FOR, "%s router:\n  "
-    "unrecognized setting for host_all_ignored option", rblock->name);
-
 /* One of route_list or route_data must be specified */
 
 if ((ob->route_list == NULL && ob->route_data == NULL) ||
@@ -139,8 +110,8 @@ Returns:    FALSE if domain expected and string is empty;
 */
 
 static BOOL
-parse_route_item(const uschar *s, const uschar **domain, const uschar **hostlist,
-  const uschar **options)
+parse_route_item(uschar *s, uschar **domain, uschar **hostlist,
+  uschar **options)
 {
 while (*s != 0 && isspace(*s)) s++;
 
@@ -218,7 +189,7 @@ manualroute_router_entry(
   router_instance *rblock,        /* data for this instantiation */
   address_item *addr,             /* address we are working on */
   struct passwd *pw,              /* passwd entry after check_local_user */
-  int verify,                     /* v_none/v_recipient/v_sender/v_expn */
+  BOOL verify,                    /* TRUE when verifying */
   address_item **addr_local,      /* add it to this if it's local */
   address_item **addr_remote,     /* add it to this if it's remote */
   address_item **addr_new,        /* put new addresses on here */
@@ -226,14 +197,13 @@ manualroute_router_entry(
 {
 int rc, lookup_type;
 uschar *route_item = NULL;
-const uschar *options = NULL;
-const uschar *hostlist = NULL;
-const uschar *domain;
-uschar *newhostlist;
-const uschar *listptr;
+uschar *options = NULL;
+uschar *hostlist = NULL;
+uschar *domain, *newhostlist, *listptr;
 manualroute_router_options_block *ob =
   (manualroute_router_options_block *)(rblock->options_block);
 transport_instance *transport = NULL;
+uschar route_buffer[1024];
 BOOL individual_transport_set = FALSE;
 BOOL randomize = ob->hosts_randomize;
 
@@ -251,7 +221,8 @@ if (ob->route_list != NULL)
   int sep = -(';');             /* Default is semicolon */
   listptr = ob->route_list;
 
-  while ((route_item = string_nextinlist(&listptr, &sep, NULL, 0)) != NULL)
+  while ((route_item = string_nextinlist(&listptr, &sep, route_buffer,
+         sizeof(route_buffer))) != NULL)
     {
     int rc;
 
@@ -262,7 +233,7 @@ if (ob->route_list != NULL)
     /* Check the current domain; if it matches, break the loop */
 
     if ((rc = match_isinlist(addr->domain, &domain, UCHAR_MAX+1,
-           &domainlist_anchor, NULL, MCL_DOMAIN, TRUE, CUSS &lookup_value)) == OK)
+           &domainlist_anchor, NULL, MCL_DOMAIN, TRUE, &lookup_value)) == OK)
       break;
 
     /* If there was a problem doing the check, defer */
@@ -292,8 +263,8 @@ else
 single host or a list of hosts; options is pointing to the rest of the
 routelist item, which is either empty or contains various option words. */
 
-DEBUG(D_route) debug_printf("original list of hosts = \"%s\" options = %s\n",
-  hostlist, options);
+DEBUG(D_route)
+  debug_printf("original hostlist=%s options=%s\n", hostlist, options);
 
 newhostlist = expand_string_copy(hostlist);
 lookup_value = NULL;                        /* Finished with */
@@ -306,13 +277,13 @@ if (newhostlist == NULL)
   {
   if (expand_string_forcedfail) return DECLINE;
   addr->message = string_sprintf("%s router: failed to expand \"%s\": %s",
-    rblock->name, hostlist, expand_string_message);
+    rblock->name, hostlist , expand_string_message);
   return DEFER;
   }
 else hostlist = newhostlist;
 
-DEBUG(D_route) debug_printf("expanded list of hosts = \"%s\" options = %s\n",
-  hostlist, options);
+DEBUG(D_route) debug_printf("expanded hostlist=\"%s\" options=%s\n", hostlist,
+  options);
 
 /* Set default lookup type and scan the options */
 
@@ -320,26 +291,28 @@ lookup_type = lk_default;
 
 while (*options != 0)
   {
-  unsigned n;
-  const uschar *s = options;
+  int term;
+  uschar *s = options;
   while (*options != 0 && !isspace(*options)) options++;
-  n = options-s;
+  term = *options;
+  *options = 0;
 
-  if (Ustrncmp(s, "randomize", n) == 0) randomize = TRUE;
-  else if (Ustrncmp(s, "no_randomize", n) == 0) randomize = FALSE;
-  else if (Ustrncmp(s, "byname", n) == 0) lookup_type = lk_byname;
-  else if (Ustrncmp(s, "bydns", n) == 0) lookup_type = lk_bydns;
+  if (Ustrcmp(s, "randomize") == 0) randomize = TRUE;
+  else if (Ustrcmp(s, "no_randomize") == 0) randomize = FALSE;
+  else if (Ustrcmp(s, "byname") == 0) lookup_type = lk_byname;
+  else if (Ustrcmp(s, "bydns") == 0) lookup_type = lk_bydns;
   else
     {
     transport_instance *t;
     for (t = transports; t != NULL; t = t->next)
+      {
       if (Ustrcmp(t->name, s) == 0)
         {
         transport = t;
         individual_transport_set = TRUE;
         break;
         }
-
+      }
     if (t == NULL)
       {
       s = string_sprintf("unknown routing option or transport name \"%s\"", s);
@@ -349,7 +322,7 @@ while (*options != 0)
       }
     }
 
-  if (*options)
+  if (term != 0)
     {
     options++;
     while (*options != 0 && isspace(*options)) options++;
@@ -358,13 +331,13 @@ while (*options != 0)
 
 /* Set up the errors address, if any. */
 
-rc = rf_get_errors_address(addr, rblock, verify, &addr->prop.errors_address);
+rc = rf_get_errors_address(addr, rblock, verify, &(addr->p.errors_address));
 if (rc != OK) return rc;
 
 /* Set up the additional and removeable headers for this address. */
 
-rc = rf_get_munge_headers(addr, rblock, &addr->prop.extra_headers,
-  &addr->prop.remove_headers);
+rc = rf_get_munge_headers(addr, rblock, &(addr->p.extra_headers),
+  &(addr->p.remove_headers));
 if (rc != OK) return rc;
 
 /* If an individual transport is not set, get the transport for this router, if
@@ -390,7 +363,6 @@ if (transport != NULL && transport->info->local)
     addr->host_list = h = store_get(sizeof(host_item));
     h->name = string_copy(hostlist);
     h->address = NULL;
-    h->port = PORT_NONE;
     h->mx = MX_NONE;
     h->status = hstatus_unknown;
     h->why = hwhy_unknown;
@@ -413,7 +385,7 @@ address is just accepted. */
 
 if (hostlist[0] == 0)
   {
-  if (verify != v_none) goto ROUTED;
+  if (verify) goto ROUTED;
   addr->message = string_sprintf("error in %s router: no host(s) specified "
     "for domain %s", rblock->name, domain);
   log_write(0, LOG_MAIN, "%s", addr->message);
@@ -428,35 +400,11 @@ rc = rf_lookup_hostlist(rblock, addr, rblock->ignore_target_hosts, lookup_type,
   ob->hff_code, addr_new);
 if (rc != OK) return rc;
 
-/* If host_find_failed is set to "ignore", it is possible for all the hosts to
-be ignored, in which case we will end up with an empty host list. What happens
-is controlled by host_all_ignored. */
-
-if (addr->host_list == NULL)
-  {
-  int i;
-  DEBUG(D_route) debug_printf("host_find_failed ignored every host\n");
-  if (ob->hai_code == hff_decline) return DECLINE;
-  if (ob->hai_code == hff_pass) return PASS;
-
-  for (i = 0; i < hff_count; i++)
-    if (ob->hai_code == hff_codes[i]) break;
-
-  addr->message = string_sprintf("lookup failed for all hosts in %s router: "
-    "host_find_failed=ignore host_all_ignored=%s", rblock->name, hff_names[i]);
-
-  if (ob->hai_code == hff_defer) return DEFER;
-  if (ob->hai_code == hff_fail) return FAIL;
-
-  addr->special_action = SPECIAL_FREEZE;
-  return DEFER;
-  }
-
 /* Finally, since we have done all the routing here, there must be a transport
 defined for these hosts. It will be a remote one, as a local transport is
 dealt with above. However, we don't need one if verifying only. */
 
-if (transport == NULL && verify == v_none)
+if (transport == NULL && !verify)
     {
     log_write(0, LOG_MAIN, "Error in %s router: no transport defined",
       rblock->name);
