@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 #include "../exim.h"
@@ -24,7 +24,7 @@ configuration.
 Arguments:
   addr         the input address
   rblock       the router instance
-  verify       v_none / v_recipient / v_sender / v_expn
+  verify       TRUE when verifying
   errors_to    point the errors address here
 
 Returns:       OK if no problem
@@ -34,11 +34,11 @@ Returns:       OK if no problem
 
 int
 rf_get_errors_address(address_item *addr, router_instance *rblock,
-  int verify, uschar **errors_to)
+  BOOL verify, uschar **errors_to)
 {
 uschar *s;
 
-*errors_to = addr->prop.errors_address;
+*errors_to = addr->p.errors_address;
 if (rblock->errors_to == NULL) return OK;
 
 s = expand_string(rblock->errors_to);
@@ -73,7 +73,7 @@ of routers by checking the sender address. When testing an address, there may
 not be a sender address. We also need to save and restore the expansion values
 associated with an address. */
 
-if (verify != v_none)
+if (verify)
   {
   *errors_to = s;
   DEBUG(D_route)
@@ -84,8 +84,8 @@ else
   BOOL save_address_test_mode = address_test_mode;
   int save1 = 0;
   int i;
-  const uschar ***p;
-  const uschar *address_expansions_save[ADDRESS_EXPANSIONS_COUNT];
+  uschar ***p;
+  uschar *address_expansions_save[ADDRESS_EXPANSIONS_COUNT];
   address_item *snew = deliver_make_addr(s, FALSE);
 
   if (sender_address != NULL)
@@ -98,23 +98,10 @@ else
     address_expansions_save[i++] = **p++;
   address_test_mode = FALSE;
 
-  /* NOTE: the address is verified as a recipient, not a sender. This is
-  perhaps confusing. It isn't immediately obvious what to do: we want to have
-  some confidence that we can deliver to the address, in which case it will be
-  a recipient, but on the other hand, it will be passed on in SMTP deliveries
-  as a sender. However, I think on balance recipient is right because sender
-  verification is really about the *incoming* sender of the message.
-
-  If this code is changed, note that you must set vopt_fake_sender instead of
-  vopt_is_recipient, as otherwise sender_address may be altered because
-  verify_address() thinks it is dealing with *the* sender of the message. */
-
   DEBUG(D_route|D_verify)
     debug_printf("------ Verifying errors address %s ------\n", s);
-  if (verify_address(snew, NULL,
-      vopt_is_recipient /* vopt_fake_sender is the alternative */
-      | vopt_qualify, -1, -1, -1, NULL, NULL, NULL) == OK)
-    *errors_to = snew->address;
+  if (verify_address(snew, NULL, vopt_is_recipient | vopt_qualify, -1, NULL)
+    == OK) *errors_to = snew->address;
   DEBUG(D_route|D_verify)
     debug_printf("------ End verifying errors address %s ------\n", s);
 
