@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -25,12 +25,6 @@ case, MX records are looked up for the name, and the list of hosts obtained
 replaces the incoming "host". In other words, "x/MX" is shorthand for "those
 hosts pointed to by x's MX records".
 
-It is also possible for a port to be specified along with the host name or IP
-address. The syntax is to add ":port" on to the end. This doesn't work with
-IPv6 addresses, so we allow IP addresses to be enclosed in [] in order to make
-this work. The specification of the port must come last, that is, after "/MX"
-if that is present.
-
 Arguments:
   rblock               the router block
   addr                 the address being routed
@@ -62,22 +56,17 @@ list of MX hosts. If the first host is the local host, act according to the
 "self" option in the configuration. */
 
 prev = NULL;
-for (h = addr->host_list; h != NULL; h = next_h)
+for (h = addr->host_list; h != NULL; prev = h, h = next_h)
   {
-  const uschar *canonical_name;
-  int rc, len, port;
+  uschar *canonical_name;
+  int rc, len;
 
   next_h = h->next;
-  if (h->address != NULL) { prev = h; continue; }
+  if (h->address != NULL) continue;
 
   DEBUG(D_route|D_host_lookup)
     debug_printf("finding IP address for %s\n", h->name);
 
-  /* Handle any port setting that may be on the name; it will be removed
-  from the end of the name. */
-
-  port = host_item_get_port(h);
-
   /* If the name ends with "/MX", we interpret it to mean "the list of hosts
   pointed to by MX records with this name". */
 
@@ -86,15 +75,13 @@ for (h = addr->host_list; h != NULL; h = next_h)
     {
     DEBUG(D_route|D_host_lookup)
       debug_printf("doing DNS MX lookup for %s\n", h->name);
-
-    h->name = string_copyn(h->name, len - 3);
+    h->name[len-3] = 0;
     rc = host_find_bydns(h,
-        ignore_target_hosts,
-        HOST_FIND_BY_MX,                /* look only for MX records */
-        NULL,                           /* SRV service not relevant */
-        NULL,                           /* failing srv domains not relevant */
-        NULL,                           /* no special mx failing domains */
-        &rblock->dnssec,		/* dnssec request/require */
+        NULL,                           /* ignore_target_hosts*/
+        TRUE,                           /* mx_only */
+        FALSE,                          /* a_only */
+        FALSE,                          /* qualify_single */
+        FALSE,                          /* search_parents */
         NULL,                           /* fully_qualified_name */
         NULL);                          /* indicate local host removed */
     }
@@ -102,11 +89,10 @@ for (h = addr->host_list; h != NULL; h = next_h)
   /* If explicitly configured to look up by name, or if the "host name" is
   actually an IP address, do a byname lookup. */
 
-  else if (lookup_type == lk_byname || string_is_ip_address(h->name, NULL) != 0)
+  else if (lookup_type == lk_byname || string_is_ip_address(h->name, NULL))
     {
     DEBUG(D_route|D_host_lookup) debug_printf("calling host_find_byname\n");
-    rc = host_find_byname(h, ignore_target_hosts, HOST_FIND_QUALIFY_SINGLE,
-      &canonical_name, TRUE);
+    rc = host_find_byname(h, ignore_target_hosts, &canonical_name, TRUE);
     }
 
   /* Otherwise, do a DNS lookup. If that yields "host not found", and the
@@ -117,9 +103,7 @@ for (h = addr->host_list; h != NULL; h = next_h)
     {
     BOOL removed;
     DEBUG(D_route|D_host_lookup) debug_printf("doing DNS lookup\n");
-    rc = host_find_bydns(h, ignore_target_hosts, HOST_FIND_BY_A, NULL, NULL,
-      NULL,
-      &rblock->dnssec,			/* domains for request/require */
+    rc = host_find_bydns(h, ignore_target_hosts, FALSE, TRUE, FALSE, FALSE,
       &canonical_name, &removed);
     if (rc == HOST_FOUND)
       {
@@ -131,8 +115,7 @@ for (h = addr->host_list; h != NULL; h = next_h)
         {
         DEBUG(D_route|D_host_lookup)
           debug_printf("DNS lookup failed: trying getipnodebyname\n");
-        rc = host_find_byname(h, ignore_target_hosts, HOST_FIND_QUALIFY_SINGLE,
-          &canonical_name, TRUE);
+        rc = host_find_byname(h, ignore_target_hosts, &canonical_name, TRUE);
         }
       }
     }
@@ -158,16 +141,9 @@ for (h = addr->host_list; h != NULL; h = next_h)
 
   if (rc == HOST_FIND_FAILED)
     {
-    if (hff_code == hff_ignore)
-      {
-      if (prev == NULL) addr->host_list = next_h; else prev->next = next_h;
-      continue;   /* With the next host, leave prev unchanged */
-      }
-
     if (hff_code == hff_pass) return PASS;
     if (hff_code == hff_decline) return DECLINE;
 
-    addr->basic_errno = ERRNO_UNKNOWNHOST;
     addr->message =
       string_sprintf("lookup of host \"%s\" failed in %s router%s",
         h->name, rblock->name,
@@ -180,14 +156,6 @@ for (h = addr->host_list; h != NULL; h = next_h)
     return DEFER;
     }
 
-  /* Deal with a port setting */
-
-  if (port != PORT_NONE)
-    {
-    host_item *hh;
-    for (hh = h; hh != next_h; hh = hh->next) hh->port = port;
-    }
-
   /* A local host gets chopped, with its successors, if there are previous
   hosts. Otherwise the self option is used. If it is set to "send", any
   subsequent hosts that are also the local host do NOT get chopped. */
@@ -214,12 +182,6 @@ for (h = addr->host_list; h != NULL; h = next_h)
       }
     self_send = TRUE;
     }
-
-  /* Ensure that prev is the host before next_h; this will not be h if a lookup
-  found multiple addresses or multiple MX records. */
-
-  prev = h;
-  while (prev->next != next_h) prev = prev->next;
   }
 
 return OK;
