@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* A set of functions to search databases in various formats. An open
@@ -39,10 +39,6 @@ static tree_node *open_bot = NULL;
 
 static int open_filecount = 0;
 
-/* Allow us to reset store used for lookups and lookup caching */
-
-static void *search_reset_point = NULL;
-
 
 
 /*************************************************
@@ -62,14 +58,14 @@ Returns:     +ve => valid lookup name; value is offset in lookup_list
 */
 
 int
-search_findtype(const uschar *name, int len)
+search_findtype(uschar *name, int len)
 {
 int bot = 0;
 int top = lookup_list_count;
 while (top > bot)
   {
   int mid = (top + bot)/2;
-  int c = Ustrncmp(name, lookup_list[mid]->name, len);
+  int c = Ustrncmp(name, lookup_list[mid].name, len);
 
   /* If c == 0 we have matched the incoming name with the start of the search
   type name. However, some search types are substrings of others (e.g. nis and
@@ -79,9 +75,9 @@ while (top > bot)
   are testing. By leaving c == 0 when the lengths are different, and doing a
   > 0 test below, this all falls out correctly. */
 
-  if (c == 0 && Ustrlen(lookup_list[mid]->name) == len)
+  if (c == 0 && Ustrlen(lookup_list[mid].name) == len)
     {
-    if (lookup_list[mid]->find != NULL) return mid;
+    if (lookup_list[mid].find != NULL) return mid;
     search_error_message  = string_sprintf("lookup type \"%.*s\" is not "
       "available (not in the binary - check buildtime LOOKUP configuration)",
       len, name);
@@ -121,12 +117,12 @@ Returns:     +ve => valid lookup name; value is offset in lookup_list
 */
 
 int
-search_findtype_partial(const uschar *name, int *ptypeptr, const uschar **ptypeaff,
+search_findtype_partial(uschar *name, int *ptypeptr, uschar **ptypeaff,
   int *afflen, int *starflags)
 {
 int len, stype;
 int pv = -1;
-const uschar *ss = name;
+uschar *ss = name;
 
 *starflags = 0;
 *ptypeaff = NULL;
@@ -182,26 +178,18 @@ else if (len >= 1 && ss[len-1]  == '*')
   }
 
 /* Check for the individual search type. Only those that are actually in the
-binary are valid. For query-style types, "partial" and default types are
-erroneous. */
+binary are valid. For query-style types, "partial" is an error. */
 
 stype = search_findtype(ss, len);
-if (stype >= 0 && mac_islookup(stype, lookup_querystyle))
+if (pv >= 0 && mac_islookup(stype, lookup_querystyle))
   {
-  if (pv >= 0)
-    {
-    search_error_message = string_sprintf("\"partial\" is not permitted "
-      "for lookup type \"%s\"", ss);
-    return -1;
-    }
-  if ((*starflags & (SEARCH_STAR|SEARCH_STARAT)) != 0)
-    {
-    search_error_message = string_sprintf("defaults using \"*\" or \"*@\" are "
-      "not permitted for lookup type \"%s\"", ss);
-    return -1;
-    }
+  search_error_message = string_sprintf("\"partial\" is not permitted "
+    "for lookup type \"%s\"", ss);
+  return -1;
   }
 
+/* All is well; pass back the partial type and return the lookup type. */
+
 *ptypeptr = pv;
 return stype;
 }
@@ -223,20 +211,26 @@ for the lookup driver, if it exists.
 
 First, there is an internal, recursive subroutine.
 
-Argument:    a pointer to a search_openfile tree node
-Returns:     nothing
+Argument:   a pointer to a search_openfile tree node
+Returns:    nothing
 */
 
 static void
 tidyup_subtree(tree_node *t)
 {
+int search_type = t->name[0] - '0';
 search_cache *c = (search_cache *)(t->data.ptr);
+
 if (t->left != NULL) tidyup_subtree(t->left);
 if (t->right != NULL) tidyup_subtree(t->right);
-if (c != NULL &&
-    c->handle != NULL &&
-    lookup_list[c->search_type]->close != NULL)
-  lookup_list[c->search_type]->close(c->handle);
+
+if (c->handle != NULL && lookup_list[search_type].close != NULL)
+  lookup_list[search_type].close(c->handle);
+
+if (c->key != NULL) store_free(c->key);
+if (c->data != NULL) store_free(c->data);
+store_free(c);
+store_free(t);
 }
 
 
@@ -250,13 +244,11 @@ void
 search_tidyup(void)
 {
 int i;
-int old_pool = store_pool;
 
 DEBUG(D_lookup) debug_printf("search_tidyup called\n");
 
 /* Close individually each cached open file. */
 
-store_pool = POOL_SEARCH;
 if (search_tree != NULL)
   {
   tidyup_subtree(search_tree);
@@ -268,11 +260,7 @@ open_filecount = 0;
 /* Call the general tidyup entry for any drivers that have one. */
 
 for (i = 0; i < lookup_list_count; i++)
-  if (lookup_list[i]->tidy != NULL) (lookup_list[i]->tidy)();
-
-if (search_reset_point != NULL) store_reset(search_reset_point);
-search_reset_point = NULL;
-store_pool = old_pool;
+  if (lookup_list[i].tidy != NULL) (lookup_list[i].tidy)();
 }
 
 
@@ -333,14 +321,8 @@ search_open(uschar *filename, int search_type, int modemask, uid_t *owners,
 void *handle;
 tree_node *t;
 search_cache *c;
-lookup_info *lk = lookup_list[search_type];
+lookup_info *lk = lookup_list + search_type;
 uschar keybuffer[256];
-int old_pool = store_pool;
-
-/* Change to the search store pool and remember our reset point */
-
-store_pool = POOL_SEARCH;
-if (search_reset_point == NULL) search_reset_point = store_get(0);
 
 DEBUG(D_lookup) debug_printf("search_open: %s \"%s\"\n", lk->name,
   (filename == NULL)? US"NULL" : filename);
@@ -359,7 +341,6 @@ if ((t = tree_search(search_tree, keybuffer)) != NULL)
   if (c->handle != NULL)
     {
     DEBUG(D_lookup) debug_printf("  cached open\n");
-    store_pool = old_pool;
     return t;
     }
   DEBUG(D_lookup) debug_printf("  cached closed\n");
@@ -386,7 +367,7 @@ if (lk->type == lookup_absfile && open_filecount >= lookup_open_max)
       ((search_cache *)(open_bot->data.ptr))->down = NULL;
     else
       open_top = NULL;
-    ((lookup_list[c->search_type])->close)(c->handle);
+    ((lookup_list + c->search_type)->close)(c->handle);
     c->handle = NULL;
     open_filecount--;
     }
@@ -395,19 +376,14 @@ if (lk->type == lookup_absfile && open_filecount >= lookup_open_max)
 /* If opening is successful, call the file-checking function if there is one,
 and if all is still well, enter the open database into the tree. */
 
-handle = (lk->open)(filename, &search_error_message);
-if (handle == NULL)
-  {
-  store_pool = old_pool;
-  return NULL;
-  }
+handle = lk->open(filename, &search_error_message);
+if (handle == NULL) return handle;
 
 if (lk->check != NULL &&
    !lk->check(handle, filename, modemask, owners, owngroups,
      &search_error_message))
   {
   lk->close(handle);
-  store_pool = old_pool;
   return NULL;
   }
 
@@ -421,9 +397,10 @@ count alone. */
 
 if (t == NULL)
   {
-  t = store_get(sizeof(tree_node) + Ustrlen(keybuffer));
-  t->data.ptr = c = store_get(sizeof(search_cache));
-  c->item_cache = NULL;
+  t = store_malloc(sizeof(tree_node) + Ustrlen(keybuffer));
+  t->data.ptr = c = store_malloc(sizeof(search_cache));
+  c->key = NULL;
+  c->data = NULL;
   Ustrcpy(t->name, keybuffer);
   tree_insertnode(&search_tree, t);
   }
@@ -433,7 +410,6 @@ c->handle = handle;
 c->search_type = search_type;
 c->up = c->down = NULL;
 
-store_pool = old_pool;
 return t;
 }
 
@@ -466,10 +442,7 @@ internal_search_find(void *handle, uschar *filename, uschar *keystring)
 {
 tree_node *t = (tree_node *)handle;
 search_cache *c = (search_cache *)(t->data.ptr);
-expiring_data *e;
-uschar *data = NULL;
 int search_type = t->name[0] - '0';
-int old_pool = store_pool;
 
 /* Lookups that return DEFER may not always set an error message. So that
 the callers don't have to test for NULL, set an empty string. */
@@ -479,100 +452,85 @@ search_find_defer = FALSE;
 
 DEBUG(D_lookup) debug_printf("internal_search_find: file=\"%s\"\n  "
   "type=%s key=\"%s\"\n", filename,
-  lookup_list[search_type]->name, keystring);
+  lookup_list[search_type].name, keystring);
 
 /* Insurance. If the keystring is empty, just fail. */
 
 if (keystring[0] == 0) return NULL;
 
-/* Use the special store pool for search data */
-
-store_pool = POOL_SEARCH;
-
 /* Look up the data for the key, unless it is already in the cache for this
-file. No need to check c->item_cache for NULL, tree_search will do so. */
+file. */
 
-if (  (t = tree_search(c->item_cache, keystring))
-   && (!(e = t->data.ptr)->expiry || e->expiry > time(NULL))
-   )
-  { /* Data was in the cache already; set the pointer from the tree node */
-  data = e->ptr;
-  DEBUG(D_lookup) debug_printf("cached data used for lookup of %s%s%s\n",
-    keystring,
-    filename ? US"\n  in " : US"", filename ? filename : US"");
-  }
-else
+if (c->key == NULL || Ustrcmp(c->key, keystring) != 0)
   {
-  uint do_cache = UINT_MAX;
+  uschar *data = NULL;
   int keylength = Ustrlen(keystring);
 
   DEBUG(D_lookup)
     {
-    if (t) debug_printf("cached data found but past valid time; ");
-    debug_printf("%s lookup required for %s%s%s\n",
-      filename ? US"file" : US"database",
-      keystring,
-      filename ? US"\n  in " : US"", filename ? filename : US"");
+    if (filename != NULL)
+      debug_printf("file lookup required for %s\n  in %s\n",
+        keystring, filename);
+    else
+      debug_printf("database lookup required for %s\n", keystring);
     }
 
+  /* Free any previous cache */
+
+  if (c->key != NULL) store_free(c->key);
+  if (c->data != NULL) store_free(c->data);
+  c->key = c->data = NULL;
+
   /* Call the code for the different kinds of search. DEFER is handled
   like FAIL, except that search_find_defer is set so the caller can
   distinguish if necessary. */
 
-  if (lookup_list[search_type]->find(c->handle, filename, keystring, keylength,
-      &data, &search_error_message, &do_cache) == DEFER)
-    search_find_defer = TRUE;
-
-  /* A record that has been found is now in data, which is either NULL
-  or points to a bit of dynamic store. Cache the result of the lookup if
-  caching is permitted. Lookups can disable caching, when they did something
-  that changes their data. The mysql and pgsql lookups do this when an
-  UPDATE/INSERT query was executed. */
-
-  else if (do_cache)
+  if (lookup_list[search_type].find(c->handle, filename, keystring, keylength,
+      &data, &search_error_message) == DEFER)
     {
-    int len = keylength + 1;
-
-    if (t)	/* Previous, out-of-date cache entry.  Update with the */
-      { 	/* new result and forget the old one */
-      e->expiry = do_cache == UINT_MAX ? 0 : time(NULL)+do_cache;
-      e->ptr = data;
-      }
-    else
-      {
-      e = store_get(sizeof(expiring_data) + sizeof(tree_node) + len);
-      e->expiry = do_cache == UINT_MAX ? 0 : time(NULL)+do_cache;
-      e->ptr = data;
-      t = (tree_node *)(e+1);
-      memcpy(t->name, keystring, len);
-      t->data.ptr = e;
-      tree_insertnode(&c->item_cache, t);
-      }
+    search_find_defer = TRUE;
     }
 
-  /* If caching was disabled, empty the cache tree. We just set the cache
-  pointer to NULL here, because we cannot release the store at this stage. */
+  /* A record that has been found is now in data, which is either NULL
+  or points to a bit of dynamic store. Cache the result of the lookup. */
 
   else
     {
-    DEBUG(D_lookup) debug_printf("lookup forced cache cleanup\n");
-    c->item_cache = NULL;
+    int len = keylength + 1;
+    c->key = store_malloc(len);
+    memcpy(c->key, keystring, len);
+    if (data != NULL)
+      {
+      len = Ustrlen(data) + 1;
+      c->data = store_malloc(len);
+      memcpy(c->data, data, len);
+      }
     }
   }
 
+/* Data was in the cache already */
+
+else DEBUG(D_lookup) debug_printf("cached data used for lookup of %s%s%s\n",
+  keystring,
+  (filename == NULL)? US"" : US"\n  in ",
+  (filename == NULL)? US"" : filename);
+
+/* Debug: output the answer */
+
 DEBUG(D_lookup)
   {
-  if (data)
-    debug_printf("lookup yielded: %s\n", data);
-  else if (search_find_defer)
-    debug_printf("lookup deferred: %s\n", search_error_message);
-  else debug_printf("lookup failed\n");
+  if (c->data == NULL)
+    {
+    if (search_find_defer) debug_printf("lookup deferred: %s\n",
+      search_error_message);
+    else debug_printf("lookup failed\n");
+    }
+  else debug_printf("lookup yielded: %s\n", c->data);
   }
 
-/* Return it in new dynamic store in the regular pool */
+/* Return it in new dynamic store */
 
-store_pool = old_pool;
-return data ? string_copy(data) : NULL;
+return (c->data == NULL)? NULL : string_copy(c->data);
 }
 
 
@@ -606,7 +564,7 @@ Returns:         a pointer to a dynamic string containing the answer,
 
 uschar *
 search_find(void *handle, uschar *filename, uschar *keystring, int partial,
-  const uschar *affix, int affixlen, int starflags, int *expand_setup)
+  uschar *affix, int affixlen, int starflags, int *expand_setup)
 {
 tree_node *t = (tree_node *)handle;
 BOOL set_null_wild = FALSE;
@@ -625,7 +583,7 @@ DEBUG(D_lookup)
 that opens real files. */
 
 if (open_top != (tree_node *)handle &&
-    lookup_list[t->name[0]-'0']->type == lookup_absfile)
+    lookup_list[t->name[0]-'0'].type == lookup_absfile)
   {
   search_cache *c = (search_cache *)(t->data.ptr);
   tree_node *up = c->up;
