@@ -2,14 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* A number of functions for driving outgoing SMTP calls. */
 
 
 #include "exim.h"
-#include "transports/smtp.h"
 
 
 
@@ -26,6 +25,7 @@ Arguments:
                which case the function does nothing
   host_af    AF_INET or AF_INET6 for the outgoing IP address
   addr       the mail address being handled (for setting errors)
+  changed    if not NULL, set TRUE if expansion actually changed istring
   interface  point this to the interface
   msg        to add to any error message
 
@@ -35,9 +35,9 @@ Returns:     TRUE on success, FALSE on failure, with error message
 
 BOOL
 smtp_get_interface(uschar *istring, int host_af, address_item *addr,
-  uschar **interface, uschar *msg)
+  BOOL *changed, uschar **interface, uschar *msg)
 {
-const uschar * expint;
+uschar *expint;
 uschar *iface;
 int sep = 0;
 
@@ -53,6 +53,8 @@ if (expint == NULL)
   return FALSE;
   }
 
+if (changed != NULL) *changed = expint != istring;
+
 while (isspace(*expint)) expint++;
 if (*expint == 0) return TRUE;
 
@@ -105,7 +107,7 @@ if (pstring == NULL)
   {
   addr->transport_return = PANIC;
   addr->message = string_sprintf("failed to expand \"%s\" (\"port\" option) "
-    "for %s: %s", rstring, msg, expand_string_message);
+    "for %s: %s", msg, expand_string_message);
   return FALSE;
   }
 
@@ -141,105 +143,6 @@ return TRUE;
 
 
 
-int
-smtp_sock_connect(host_item * host, int host_af, int port, uschar * interface,
-  transport_instance * tb, int timeout)
-{
-smtp_transport_options_block * ob =
-  (smtp_transport_options_block *)tb->options_block;
-const uschar * dscp = ob->dscp;
-int dscp_value;
-int dscp_level;
-int dscp_option;
-int sock;
-int on = 1;
-int save_errno = 0;
-
-#ifndef DISABLE_EVENT
-deliver_host_address = host->address;
-deliver_host_port = port;
-if (event_raise(tb->event_action, US"tcp:connect", NULL)) return -1;
-#endif
-
-if ((sock = ip_socket(SOCK_STREAM, host_af)) < 0) return -1;
-
-/* Set TCP_NODELAY; Exim does its own buffering. */
-
-if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, US &on, sizeof(on)))
-  HDEBUG(D_transport|D_acl|D_v)
-    debug_printf("failed to set NODELAY: %s ", strerror(errno));
-
-/* Set DSCP value, if we can. For now, if we fail to set the value, we don't
-bomb out, just log it and continue in default traffic class. */
-
-if (dscp && dscp_lookup(dscp, host_af, &dscp_level, &dscp_option, &dscp_value))
-  {
-  HDEBUG(D_transport|D_acl|D_v)
-    debug_printf("DSCP \"%s\"=%x ", dscp, dscp_value);
-  if (setsockopt(sock, dscp_level, dscp_option, &dscp_value, sizeof(dscp_value)) < 0)
-    HDEBUG(D_transport|D_acl|D_v)
-      debug_printf("failed to set DSCP: %s ", strerror(errno));
-  /* If the kernel supports IPv4 and IPv6 on an IPv6 socket, we need to set the
-  option for both; ignore failures here */
-  if (host_af == AF_INET6 &&
-      dscp_lookup(dscp, AF_INET, &dscp_level, &dscp_option, &dscp_value))
-    (void) setsockopt(sock, dscp_level, dscp_option, &dscp_value, sizeof(dscp_value));
-  }
-
-/* Bind to a specific interface if requested. Caller must ensure the interface
-is the same type (IPv4 or IPv6) as the outgoing address. */
-
-if (interface && ip_bind(sock, host_af, interface, 0) < 0)
-  {
-  save_errno = errno;
-  HDEBUG(D_transport|D_acl|D_v)
-    debug_printf("unable to bind outgoing SMTP call to %s: %s", interface,
-    strerror(errno));
-  }
-
-/* Connect to the remote host, and add keepalive to the socket before returning
-it, if requested. */
-
-else if (ip_connect(sock, host_af, host->address, port, timeout) < 0)
-  save_errno = errno;
-
-/* Either bind() or connect() failed */
-
-if (save_errno != 0)
-  {
-  HDEBUG(D_transport|D_acl|D_v)
-    {
-    debug_printf("failed: %s", CUstrerror(save_errno));
-    if (save_errno == ETIMEDOUT)
-      debug_printf(" (timeout=%s)", readconf_printtime(timeout));
-    debug_printf("\n");
-    }
-  (void)close(sock);
-  errno = save_errno;
-  return -1;
-  }
-
-/* Both bind() and connect() succeeded */
-
-else
-  {
-  union sockaddr_46 interface_sock;
-  EXIM_SOCKLEN_T size = sizeof(interface_sock);
-  HDEBUG(D_transport|D_acl|D_v) debug_printf("connected\n");
-  if (getsockname(sock, (struct sockaddr *)(&interface_sock), &size) == 0)
-    sending_ip_address = host_ntoa(-1, &interface_sock, NULL, &sending_port);
-  else
-    {
-    log_write(0, LOG_MAIN | ((errno == ECONNRESET)? 0 : LOG_PANIC),
-      "getsockname() failed: %s", strerror(errno));
-    close(sock);
-    return -1;
-    }
-  if (ob->keepalive) ip_keepalive(sock, host->address, TRUE);
-  return sock;
-  }
-}
-
 /*************************************************
 *           Connect to remote host               *
 *************************************************/
@@ -249,60 +152,65 @@ detected by checking for a colon in the address. AF_INET6 is defined even on
 non-IPv6 systems, to enable the code to be less messy. However, on such systems
 host->address will always be an IPv4 address.
 
-The port field in the host item is used if it is set (usually router from SRV
-records or elsewhere). In other cases, the default passed as an argument is
-used, and the host item is updated with its value.
-
 Arguments:
-  host        host item containing name and address (and sometimes port)
+  host        host item containing name and address
   host_af     AF_INET or AF_INET6
-  port        default remote port to connect to, in host byte order, for those
-                hosts whose port setting is PORT_NONE
+  port        remote port to connect to, in host byte order
   interface   outgoing interface address or NULL
   timeout     timeout value or 0
-  tb          transport
+  keepalive   TRUE to use keepalive
 
 Returns:      connected socket number, or -1 with errno set
 */
 
 int
 smtp_connect(host_item *host, int host_af, int port, uschar *interface,
-  int timeout, transport_instance * tb)
+  int timeout, BOOL keepalive)
 {
-#ifdef SUPPORT_SOCKS
-smtp_transport_options_block * ob =
-  (smtp_transport_options_block *)tb->options_block;
-#endif
-
-if (host->port != PORT_NONE)
-  {
-  HDEBUG(D_transport|D_acl|D_v)
-    debug_printf("Transport port=%d replaced by host-specific port=%d\n", port,
-      host->port);
-  port = host->port;
-  }
-else host->port = port;    /* Set the port actually used */
-
-callout_address = string_sprintf("[%s]:%d", host->address, port);
+int on = 1;
+int sock;
 
 HDEBUG(D_transport|D_acl|D_v)
   {
-  uschar * s = US" ";
-  if (interface) s = string_sprintf(" from %s ", interface);
-#ifdef SUPPORT_SOCKS
-  if (ob->socks_proxy) s = string_sprintf("%svia proxy ", s);
-#endif
-  debug_printf("Connecting to %s %s%s... ", host->name, callout_address, s);
+  if (interface == NULL)
+    debug_printf("Connecting to %s [%s]:%d ... ",host->name,host->address,port);
+  else
+    debug_printf("Connecting to %s [%s]:%d from %s ... ", host->name,
+      host->address, port, interface);
   }
 
-/* Create and connect the socket */
+/* Create the socket */
 
-#ifdef SUPPORT_SOCKS
-if (ob->socks_proxy)
-  return socks_sock_connect(host, host_af, port, interface, tb, timeout);
-#endif
+if ((sock = ip_socket(SOCK_STREAM, host_af)) < 0) return -1;
 
-return smtp_sock_connect(host, host_af, port, interface, tb, timeout);
+/* Set TCP_NODELAY; Exim does its own buffering. */
+
+setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (uschar *)(&on), sizeof(on));
+
+/* Bind to a specific interface if requested. Caller must ensure the interface
+is the same type (IPv4 or IPv6) as the outgoing address. */
+
+if (interface != NULL && ip_bind(sock, host_af, interface, 0) < 0)
+  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "unable to bind outgoing SMTP call "
+    "to %s: %s", interface, strerror(errno));
+
+/* Connect to the remote host, and add keepalive to the socket before returning
+it, if requested. */
+
+if (ip_connect(sock, host_af, host->address, port, timeout) < 0)
+  {
+  int save_errno = errno;
+  HDEBUG(D_transport|D_acl|D_v) debug_printf("failed\n");
+  close(sock);
+  errno = save_errno;
+  return -1;
+  }
+else
+  {
+  HDEBUG(D_transport|D_acl|D_v) debug_printf("connected\n");
+  if (keepalive) ip_keepalive(sock, host->address, TRUE);
+  return sock;
+  }
 }
 
 
@@ -326,8 +234,8 @@ flush_buffer(smtp_outblock *outblock)
 int rc;
 
 #ifdef SUPPORT_TLS
-if (tls_out.active == outblock->sock)
-  rc = tls_write(FALSE, outblock->buffer, outblock->ptr - outblock->buffer);
+if (tls_active == outblock->sock)
+  rc = tls_write(outblock->buffer, outblock->ptr - outblock->buffer);
 else
 #endif
 
@@ -365,7 +273,7 @@ Returns:     0 if command added to pipelining buffer, with nothing transmitted
 */
 
 int
-smtp_write_command(smtp_outblock *outblock, BOOL noflush, const char *format, ...)
+smtp_write_command(smtp_outblock *outblock, BOOL noflush, char *format, ...)
 {
 int count;
 int rc = 0;
@@ -378,10 +286,6 @@ if (!string_vformat(big_buffer, big_buffer_size, CS format, ap))
 va_end(ap);
 count = Ustrlen(big_buffer);
 
-if (count > outblock->buffersize)
-  log_write(0, LOG_MAIN|LOG_PANIC_DIE, "overlong write_command in outgoing "
-    "SMTP");
-
 if (count > outblock->buffersize - (outblock->ptr - outblock->buffer))
   {
   rc = outblock->cmd_count;                 /* flush resets */
@@ -595,5 +499,3 @@ return buffer[0] == okdigit;
 }
 
 /* End of smtp_out.c */
-/* vi: aw ai sw=2
-*/
