@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2016 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions for writing spool files, and moving them about. */
@@ -17,7 +17,7 @@
 *************************************************/
 
 /* This function is called immediately after errors in writing the spool, with
-errno still set. It creates an error message, depending on the circumstances.
+errno still set. It creates and error message, depending on the circumstances.
 If errmsg is NULL, it logs the message and panic-dies. Otherwise errmsg is set
 to point to the message, and -1 is returned. This function makes the code of
 spool_write_header() a bit neater.
@@ -36,21 +36,22 @@ static int
 spool_write_error(int where, uschar **errmsg, uschar *s, uschar *temp_name,
   FILE *f)
 {
-uschar *msg = where == SW_RECEIVING
-  ? string_sprintf("spool file %s error while receiving from %s: %s", s,
-      sender_fullhost ? sender_fullhost : sender_ident,
-      strerror(errno))
-  : string_sprintf("spool file %s error while %s: %s", s,
-      where == SW_DELIVERING ? "delivering" : "modifying",
-      strerror(errno));
+uschar *msg = (where == SW_RECEIVING)?
+  string_sprintf("spool file %s error while receiving from %s: %s", s,
+    (sender_fullhost != NULL)? sender_fullhost : sender_ident,
+    strerror(errno))
+  :
+  string_sprintf("spool file %s error while %s: %s", s,
+    (where == SW_DELIVERING)? "delivering" : "modifying",
+    strerror(errno));
 
-if (temp_name) Uunlink(temp_name);
-if (f) (void)fclose(f);
+if (temp_name != NULL) Uunlink(temp_name);
+if (f != NULL) fclose(f);
 
-if (errmsg)
-  *errmsg = msg;
-else
+if (errmsg == NULL)
   log_write(0, LOG_MAIN|LOG_PANIC_DIE, "%s", msg);
+else
+  *errmsg = msg;
 
 return -1;
 }
@@ -92,12 +93,10 @@ double-check the mode because the group setting doesn't always get set
 automatically. */
 
 if (fd >= 0)
-  if (fchown(fd, exim_uid, exim_gid) || fchmod(fd, SPOOL_MODE))
-    {
-    DEBUG(D_any) debug_printf("failed setting perms on %s\n", temp_name);
-    (void) close(fd); fd = -1;
-    Uunlink(temp_name);
-    }
+  {
+  fchown(fd, exim_uid, exim_gid);
+  fchmod(fd, SPOOL_MODE);
+  }
 
 return fd;
 }
@@ -133,16 +132,15 @@ int size_correction;
 FILE *f;
 header_line *h;
 struct stat statbuf;
-uschar * tname;
-uschar * fname;
+uschar name[256];
+uschar temp_name[256];
 
-tname = spool_fname(US"input", message_subdir,
-		    string_sprintf("hdr.%d", (int)getpid()), US"");
-
-if ((fd = spool_open_temp(tname)) < 0)
-  return spool_write_error(where, errmsg, US"open", NULL, NULL);
+sprintf(CS temp_name, "%s/input/%s/hdr.%d", spool_directory, message_subdir,
+  (int)getpid());
+fd = spool_open_temp(temp_name);
+if (fd < 0) return spool_write_error(where, errmsg, US"open", NULL, NULL);
 f = fdopen(fd, "wb");
-DEBUG(D_receive|D_deliver) debug_printf("Writing spool header file: %s\n", tname);
+DEBUG(D_receive|D_deliver) debug_printf("Writing spool header file\n");
 
 /* We now have an open file to which the header data is to be written. Start
 with the file's leaf name, to make the file self-identifying. Continue with the
@@ -176,9 +174,6 @@ if (sender_host_address != NULL)
 if (interface_address != NULL)
   fprintf(f, "-interface_address %s.%d\n", interface_address, interface_port);
 
-if (smtp_active_hostname != primary_hostname)
-  fprintf(f, "-active_hostname %s\n", smtp_active_hostname);
-
 /* Likewise for any ident information; for local messages this is
 likely to be the same as originator_login, but will be different if
 the originator was root, forcing a different ident. */
@@ -190,78 +185,39 @@ if (sender_ident != NULL) fprintf(f, "-ident %s\n", sender_ident);
 if (received_protocol != NULL)
   fprintf(f, "-received_protocol %s\n", received_protocol);
 
-/* Preserve any ACL variables that are set. */
+/* Preserve any ACL variables that are set. Because the values may contain
+newlines, we use an explicit length. */
 
-tree_walk(acl_var_c, &acl_var_write, f);
-tree_walk(acl_var_m, &acl_var_write, f);
+for (i = 0; i < ACL_C_MAX + ACL_M_MAX; i++)
+  {
+  if (acl_var[i] != NULL)
+    fprintf(f, "-acl %d %d\n%s\n", i, Ustrlen(acl_var[i]), acl_var[i]);
+  }
 
 /* Now any other data that needs to be remembered. */
 
 fprintf(f, "-body_linecount %d\n", body_linecount);
-fprintf(f, "-max_received_linelength %d\n", max_received_linelength);
-
-if (body_zerocount > 0) fprintf(f, "-body_zerocount %d\n", body_zerocount);
 
 if (authenticated_id != NULL)
   fprintf(f, "-auth_id %s\n", authenticated_id);
 if (authenticated_sender != NULL)
   fprintf(f, "-auth_sender %s\n", authenticated_sender);
-
-if (allow_unqualified_recipient) fprintf(f, "-allow_unqualified_recipient\n");
-if (allow_unqualified_sender) fprintf(f, "-allow_unqualified_sender\n");
 if (deliver_firsttime) fprintf(f, "-deliver_firsttime\n");
-if (deliver_freeze) fprintf(f, "-frozen " TIME_T_FMT "\n", deliver_frozen_at);
+if (deliver_freeze) fprintf(f, "-frozen %d\n", deliver_frozen_at);
 if (dont_deliver) fprintf(f, "-N\n");
-if (host_lookup_deferred) fprintf(f, "-host_lookup_deferred\n");
 if (host_lookup_failed) fprintf(f, "-host_lookup_failed\n");
 if (sender_local) fprintf(f, "-local\n");
 if (local_error_message) fprintf(f, "-localerror\n");
 if (local_scan_data != NULL) fprintf(f, "-local_scan %s\n", local_scan_data);
-#ifdef WITH_CONTENT_SCAN
-if (spam_bar)       fprintf(f,"-spam_bar %s\n",       spam_bar);
-if (spam_score)     fprintf(f,"-spam_score %s\n",     spam_score);
-if (spam_score_int) fprintf(f,"-spam_score_int %s\n", spam_score_int);
-#endif
 if (deliver_manual_thaw) fprintf(f, "-manual_thaw\n");
 if (sender_set_untrusted) fprintf(f, "-sender_set_untrusted\n");
 
-#ifdef EXPERIMENTAL_BRIGHTMAIL
-if (bmi_verdicts != NULL) fprintf(f, "-bmi_verdicts %s\n", bmi_verdicts);
-#endif
-
 #ifdef SUPPORT_TLS
-if (tls_in.certificate_verified) fprintf(f, "-tls_certificate_verified\n");
-if (tls_in.cipher)       fprintf(f, "-tls_cipher %s\n", tls_in.cipher);
-if (tls_in.peercert)
-  {
-  (void) tls_export_cert(big_buffer, big_buffer_size, tls_in.peercert);
-  fprintf(f, "-tls_peercert %s\n", CS big_buffer);
-  }
-if (tls_in.peerdn)       fprintf(f, "-tls_peerdn %s\n", string_printing(tls_in.peerdn));
-if (tls_in.sni)		 fprintf(f, "-tls_sni %s\n",    string_printing(tls_in.sni));
-if (tls_in.ourcert)
-  {
-  (void) tls_export_cert(big_buffer, big_buffer_size, tls_in.ourcert);
-  fprintf(f, "-tls_ourcert %s\n", CS big_buffer);
-  }
-if (tls_in.ocsp)	 fprintf(f, "-tls_ocsp %d\n",   tls_in.ocsp);
+if (tls_certificate_verified) fprintf(f, "-tls_certificate_verified\n");
+if (tls_cipher != NULL) fprintf(f, "-tls_cipher %s\n", tls_cipher);
+if (tls_peerdn != NULL) fprintf(f, "-tls_peerdn %s\n", tls_peerdn);
 #endif
 
-#ifdef SUPPORT_I18N
-if (message_smtputf8)
-  {
-  fprintf(f, "-smtputf8\n");
-  if (message_utf8_downconvert)
-    fprintf(f, "-utf8_%sdowncvt\n", message_utf8_downconvert < 0 ? "opt" : "");
-  }
-#endif
-
-/* Write the dsn flags to the spool header file */
-DEBUG(D_deliver) debug_printf("DSN: Write SPOOL :-dsn_envid %s\n", dsn_envid);
-if (dsn_envid != NULL) fprintf(f, "-dsn_envid %s\n", dsn_envid);
-DEBUG(D_deliver) debug_printf("DSN: Write SPOOL :-dsn_ret %d\n", dsn_ret);
-if (dsn_ret != 0) fprintf(f, "-dsn_ret %d\n", dsn_ret);
-
 /* To complete the envelope, write out the tree of non-recipients, followed by
 the list of recipients. These won't be disjoint the first time, when no
 checking has been done. If a recipient is a "one-time" alias, it is followed by
@@ -272,25 +228,14 @@ fprintf(f, "%d\n", recipients_count);
 for (i = 0; i < recipients_count; i++)
   {
   recipient_item *r = recipients_list + i;
-
-  DEBUG(D_deliver) debug_printf("DSN: Flags :%d\n", r->dsn_flags);
-
-  if (r->pno < 0 && r->errors_to == NULL && r->dsn_flags == 0)
+  if (r->pno < 0 && r->errors_to == NULL)
     fprintf(f, "%s\n", r->address);
   else
     {
-    uschar * errors_to = r->errors_to ? r->errors_to : US"";
-    /* for DSN SUPPORT extend exim 4 spool in a compatible way by
-    adding new values upfront and add flag 0x02 */
-    uschar * orcpt = r->orcpt ? r->orcpt : US"";
-
-    fprintf(f, "%s %s %d,%d %s %d,%d#3\n", r->address, orcpt, Ustrlen(orcpt),
-      r->dsn_flags, errors_to, Ustrlen(errors_to), r->pno);
+    uschar *errors_to = (r->errors_to == NULL)? US"" : r->errors_to;
+    fprintf(f, "%s %s %d,%d#1\n", r->address, errors_to,
+      Ustrlen(errors_to), r->pno);
     }
-
-    DEBUG(D_deliver) debug_printf("DSN: **** SPOOL_OUT - "
-      "address: |%s| errorsto: |%s| orcpt: |%s| dsn_flags: %d\n",
-      r->address, r->errors_to, r->orcpt, r->dsn_flags);
   }
 
 /* Put a blank line before the headers */
@@ -301,8 +246,7 @@ fprintf(f, "\n");
 to get the actual size of the headers. */
 
 fflush(f);
-if (fstat(fd, &statbuf))
-  return spool_write_error(where, errmsg, US"fstat", tname, f);
+fstat(fd, &statbuf);
 size_correction = statbuf.st_size;
 
 /* Finally, write out the message's headers. To make it easier to read them
@@ -323,30 +267,28 @@ for (h = header_list; h != NULL; h = h->next)
 /* Flush and check for any errors while writing */
 
 if (fflush(f) != 0 || ferror(f))
-  return spool_write_error(where, errmsg, US"write", tname, f);
+  return spool_write_error(where, errmsg, US"write", temp_name, f);
 
 /* Force the file's contents to be written to disk. Note that fflush()
 just pushes it out of C, and fclose() doesn't guarantee to do the write
 either. That's just the way Unix works... */
 
-if (EXIMfsync(fileno(f)) < 0)
-  return spool_write_error(where, errmsg, US"sync", tname, f);
+if (fsync(fileno(f)) < 0)
+  return spool_write_error(where, errmsg, US"sync", temp_name, f);
 
 /* Get the size of the file, and close it. */
 
-if (fstat(fd, &statbuf) != 0)
-  return spool_write_error(where, errmsg, US"fstat", tname, NULL);
+fstat(fd, &statbuf);
 if (fclose(f) != 0)
-  return spool_write_error(where, errmsg, US"close", tname, NULL);
+  return spool_write_error(where, errmsg, US"close", temp_name, NULL);
 
 /* Rename the file to its correct name, thereby replacing any previous
 incarnation. */
 
-fname = spool_fname(US"input", message_subdir, id, US"-H");
-DEBUG(D_receive|D_deliver) debug_printf("Renaming spool header file: %s\n", fname);
+sprintf(CS name, "%s/input/%s/%s-H", spool_directory, message_subdir, id);
 
-if (Urename(tname, fname) < 0)
-  return spool_write_error(where, errmsg, US"rename", tname, NULL);
+if (Urename(temp_name, name) < 0)
+  return spool_write_error(where, errmsg, US"rename", temp_name, NULL);
 
 /* Linux (and maybe other OS?) does not automatically sync a directory after
 an operation like rename. We therefore have to do it forcibly ourselves in
@@ -360,20 +302,20 @@ these cases. One hack on top of another... but that's life. */
 
 #ifdef NEED_SYNC_DIRECTORY
 
-tname = spool_fname(US"input", message_subdir, US".", US"");
+sprintf(CS temp_name, "%s/input/%s/.", spool_directory, message_subdir);
 
-# ifndef O_DIRECTORY
-#  define O_DIRECTORY 0
-# endif
+#ifndef O_DIRECTORY
+#define O_DIRECTORY 0
+#endif
 
-if ((fd = Uopen(tname, O_RDONLY|O_DIRECTORY, 0)) < 0)
-  return spool_write_error(where, errmsg, US"directory open", fname, NULL);
+if ((fd = Uopen(temp_name, O_RDONLY|O_DIRECTORY, 0)) < 0)
+  return spool_write_error(where, errmsg, US"directory open", name, NULL);
 
-if (EXIMfsync(fd) < 0 && errno != EINVAL)
-  return spool_write_error(where, errmsg, US"directory sync", fname, NULL);
+if (fsync(fd) < 0 && errno != EINVAL)
+  return spool_write_error(where, errmsg, US"directory sync", name, NULL);
 
 if (close(fd) < 0)
-  return spool_write_error(where, errmsg, US"directory close", fname, NULL);
+  return spool_write_error(where, errmsg, US"directory close", name, NULL);
 
 #endif  /* NEED_SYNC_DIRECTORY */
 
@@ -414,12 +356,13 @@ static BOOL
 make_link(uschar *dir, uschar *subdir, uschar *id, uschar *suffix, uschar *from,
   uschar *to, BOOL noentok)
 {
-uschar * fname = spool_fname(string_sprintf("%s%s", from, dir), subdir, id, suffix);
-uschar * tname = spool_fname(string_sprintf("%s%s", to,   dir), subdir, id, suffix);
-if (Ulink(fname, tname) < 0 && (!noentok || errno != ENOENT))
+uschar f[256], t[256];
+sprintf(CS f, "%s/%s%s/%s/%s%s", spool_directory, from, dir, subdir, id, suffix);
+sprintf(CS t, "%s/%s%s/%s/%s%s", spool_directory, to, dir, subdir, id, suffix);
+if (Ulink(f, t) < 0 && (!noentok || errno != ENOENT))
   {
   log_write(0, LOG_MAIN|LOG_PANIC, "link(\"%s\", \"%s\") failed while moving "
-    "message: %s", fname, tname, strerror(errno));
+    "message: %s", f, t, strerror(errno));
   return FALSE;
   }
 return TRUE;
@@ -451,11 +394,12 @@ static BOOL
 break_link(uschar *dir, uschar *subdir, uschar *id, uschar *suffix, uschar *from,
   BOOL noentok)
 {
-uschar * fname = spool_fname(string_sprintf("%s%s", from, dir), subdir, id, suffix);
-if (Uunlink(fname) < 0 && (!noentok || errno != ENOENT))
+uschar f[256];
+sprintf(CS f, "%s/%s%s/%s/%s%s", spool_directory, from, dir, subdir, id, suffix);
+if (Uunlink(f) < 0 && (!noentok || errno != ENOENT))
   {
   log_write(0, LOG_MAIN|LOG_PANIC, "unlink(\"%s\") failed while moving "
-    "message: %s", fname, strerror(errno));
+    "message: %s", f, strerror(errno));
   return FALSE;
   }
 return TRUE;
@@ -464,7 +408,7 @@ return TRUE;
 
 
 /************************************************
-*            Move message files                 *
+*	     Move message files			*
 ************************************************/
 
 /* Move the files for a message (-H, -D, and msglog) from one directory (or
@@ -487,19 +431,17 @@ spool_move_message(uschar *id, uschar *subdir, uschar *from, uschar *to)
 {
 /* Create any output directories that do not exist. */
 
-(void) directory_make(spool_directory,
-  spool_sname(string_sprintf("%sinput", to), subdir),
-  INPUT_DIRECTORY_MODE, TRUE);
-(void) directory_make(spool_directory,
-  spool_sname(string_sprintf("%smsglog", to), subdir),
-  INPUT_DIRECTORY_MODE, TRUE);
+sprintf(CS big_buffer, "%sinput/%s", to, subdir);
+(void)directory_make(spool_directory, big_buffer, INPUT_DIRECTORY_MODE, TRUE);
+sprintf(CS big_buffer, "%smsglog/%s", to, subdir);
+(void)directory_make(spool_directory, big_buffer, INPUT_DIRECTORY_MODE, TRUE);
 
 /* Move the message by first creating new hard links for all the files, and
 then removing the old links. When moving messages onto the main spool, the -H
 file should be set up last, because that's the one that tells Exim there is a
 message to be delivered, so we create its new link last and remove its old link
 first. Programs that look at the alternate directories should follow the same
-rule of waiting for a -H file before doing anything. When moving messages off
+rule of waiting for a -H file before doing anything. When moving messsages off
 the mail spool, the -D file should be open and locked at the time, thus keeping
 Exim's hands off. */
 
@@ -522,5 +464,3 @@ return TRUE;
 #endif
 
 /* End of spool_out.c */
-/* vi: aw ai sw=2
-*/
