@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Miscellaneous string-handling functions. Some are not required for
@@ -20,13 +20,13 @@ utilities and tests, and are cut out by the COMPILE_UTILITY macro. */
 /* This used just to be a regular expression, but with IPv6 things are a bit
 more complicated. If the address contains a colon, it is assumed to be a v6
 address (assuming HAVE_IPV6 is set). If a mask is permitted and one is present,
-and maskptr is not NULL, its offset is placed there.
+the '/' in the input gets changed to 0, and maskptr points to its offset.
 
 Arguments:
   s         a string
   maskptr   NULL if no mask is permitted to follow
-            otherwise, points to an int where the offset of '/' is placed
-            if there is no / followed by trailing digits, *maskptr is set 0
+            otherwise, points to an int where the '/' offset is put, or
+            zero if there is no mask given
 
 Returns:    0 if the string is not a textual representation of an IP address
             4 if it is an IPv4 address
@@ -34,22 +34,26 @@ Returns:    0 if the string is not a textual representation of an IP address
 */
 
 int
-string_is_ip_address(const uschar *s, int *maskptr)
+string_is_ip_address(uschar *s, int *maskptr)
 {
 int i;
 int yield = 4;
 
-/* If an optional mask is permitted, check for it. If found, pass back the
-offset. */
+/* If an optional mask is permitted, check for it. If found, change the / to
+a zero, and pass back the offset where this was done. */
 
 if (maskptr != NULL)
   {
-  const uschar *ss = s + Ustrlen(s);
+  uschar *ss = s + Ustrlen(s);
   *maskptr = 0;
   if (s != ss && isdigit(*(--ss)))
     {
     while (ss > s && isdigit(ss[-1])) ss--;
-    if (ss > s && *(--ss) == '/') *maskptr = ss - s;
+    if (ss > s && *(--ss) == '/')
+      {
+      *ss = 0;
+      *maskptr = ss - s;
+      }
     }
   }
 
@@ -75,11 +79,16 @@ if (Ustrchr(s, ':') != NULL)
   for (count = 0; count < 8; count++)
     {
     /* If the end of the string is reached before reading 8 components, the
-    address is valid provided a double colon has been read. This also applies
-    if we hit the / that introduces a mask or the % that introduces the
+    address is valid provided a double colon has been read. Otherwise is
+    is invalid. */
+
+    if (*s == 0) return had_double_colon? yield : 0;
+
+    /* If the left of the current position is a valid IPv6 address and we
+    encounter a percent sign, the right of the current position is the
     interface specifier (scope id) of a link-local address. */
 
-    if (*s == 0 || *s == '%' || *s == '/') return had_double_colon? yield : 0;
+    if (*s == '%') return had_double_colon? yield : 0;
 
     /* If a component starts with an additional colon, we have hit a double
     colon. This is permitted to appear once only, and counts as at least
@@ -122,13 +131,10 @@ if (Ustrchr(s, ':') != NULL)
     }
 
   /* If about to handle a trailing IPv4 address, drop through. Otherwise
-  all is well if we are at the end of the string or at the mask or at a percent
-  sign, which introduces the interface specifier (scope id) of a link local
-  address. */
+  all is well if we are at the end of the string or at a percent sign, which
+  introduces the interface specifier (scope id) of a link local address. */
 
-  if (!v4end)
-    return (*s == 0 || *s == '%' ||
-           (*s == '/' && maskptr != NULL && *maskptr != 0))? yield : 0;
+  if (!v4end) return (*s == 0 || *s == '%')? yield : 0;
   }
 
 /* Test for IPv4 address, which may be the tail-end of an IPv6 address. */
@@ -140,8 +146,7 @@ for (i = 0; i < 4; i++)
   if (isdigit(*s) && isdigit(*(++s))) s++;
   }
 
-return (*s == 0 || (*s == '/' && maskptr != NULL && *maskptr != 0))?
-  yield : 0;
+return (*s == 0)? yield : 0;
 }
 #endif  /* COMPILE_UTILITY */
 
@@ -165,7 +170,7 @@ Returns:      pointer to the buffer
 uschar *
 string_format_size(int size, uschar *buffer)
 {
-if (size == 0) Ustrcpy(buffer, "     ");
+if (size == 0) Ustrcpy(CS buffer, "     ");
 else if (size < 1024) sprintf(CS buffer, "%5d", size);
 else if (size < 10*1024)
   sprintf(CS buffer, "%4.1fK", (double)size / 1024.0);
@@ -210,6 +215,7 @@ return yield;
 
 
 
+#ifndef COMPILE_UTILITY
 /*************************************************
 *          Interpret escape sequence             *
 *************************************************/
@@ -224,13 +230,10 @@ Returns:   the value of the character escape
 */
 
 int
-string_interpret_escape(const uschar **pp)
+string_interpret_escape(uschar **pp)
 {
-#ifdef COMPILE_UTILITY
-const uschar *hex_digits= CUS"0123456789abcdef";
-#endif
 int ch;
-const uschar *p = *pp;
+uschar *p = *pp;
 ch = *(++p);
 if (isdigit(ch) && ch != '8' && ch != '9')
   {
@@ -244,12 +247,9 @@ if (isdigit(ch) && ch != '8' && ch != '9')
   }
 else switch(ch)
   {
-  case 'b':  ch = '\b'; break;
-  case 'f':  ch = '\f'; break;
   case 'n':  ch = '\n'; break;
   case 'r':  ch = '\r'; break;
   case 't':  ch = '\t'; break;
-  case 'v':  ch = '\v'; break;
   case 'x':
   ch = 0;
   if (isxdigit(p[1]))
@@ -264,6 +264,7 @@ else switch(ch)
 *pp = p;
 return ch;
 }
+#endif  /* COMPILE_UTILITY */
 
 
 
@@ -274,28 +275,24 @@ return ch;
 
 /* This function is called for critical strings. It checks for any
 non-printing characters, and if any are found, it makes a new copy
-of the string with suitable escape sequences. It is most often called by the
-macro string_printing(), which sets allow_tab TRUE.
+of the string with suitable escape sequences.
 
-Arguments:
-  s             the input string
-  allow_tab     TRUE to allow tab as a printing character
-
-Returns:        string with non-printers encoded as printing sequences
+Argument:    input string
+Returns:     string with non-printers encoded as printing sequences
 */
 
-const uschar *
-string_printing2(const uschar *s, BOOL allow_tab)
+uschar *
+string_printing(uschar *s)
 {
 int nonprintcount = 0;
 int length = 0;
-const uschar *t = s;
+uschar *t = s;
 uschar *ss, *tt;
 
 while (*t != 0)
   {
   int c = *t++;
-  if (!mac_isprint(c) || (!allow_tab && c == '\t')) nonprintcount++;
+  if (!mac_isprint(c)) nonprintcount++;
   length++;
   }
 
@@ -304,7 +301,7 @@ if (nonprintcount == 0) return s;
 /* Get a new block of store guaranteed big enough to hold the
 expanded string. */
 
-ss = store_get(length + nonprintcount * 3 + 1);
+ss = store_get(length + nonprintcount * 4 + 1);
 
 /* Copy everying, escaping non printers. */
 
@@ -314,7 +311,7 @@ tt = ss;
 while (*t != 0)
   {
   int c = *t;
-  if (mac_isprint(c) && (allow_tab || c != '\t')) *tt++ = *t++; else
+  if (mac_isprint(c)) *tt++ = *t++; else
     {
     *tt++ = '\\';
     switch (*t)
@@ -324,7 +321,6 @@ while (*t != 0)
       case '\b': *tt++ = 'b'; break;
       case '\v': *tt++ = 'v'; break;
       case '\f': *tt++ = 'f'; break;
-      case '\t': *tt++ = 't'; break;
       default: sprintf(CS tt, "%03o", *t); tt += 3; break;
       }
     t++;
@@ -335,73 +331,6 @@ return ss;
 }
 #endif  /* COMPILE_UTILITY */
 
-/*************************************************
-*        Undo printing escapes in string         *
-*************************************************/
-
-/* This function is the reverse of string_printing2.  It searches for
-backslash characters and if any are found, it makes a new copy of the
-string with escape sequences parsed.  Otherwise it returns the original
-string.
-
-Arguments:
-  s             the input string
-
-Returns:        string with printing escapes parsed back
-*/
-
-uschar *
-string_unprinting(uschar *s)
-{
-uschar *p, *q, *r, *ss;
-int len, off;
-
-p = Ustrchr(s, '\\');
-if (!p) return s;
-
-len = Ustrlen(s) + 1;
-ss = store_get(len);
-
-q = ss;
-off = p - s;
-if (off)
-  {
-  memcpy(q, s, off);
-  q += off;
-  }
-
-while (*p)
-  {
-  if (*p == '\\')
-    {
-    *q++ = string_interpret_escape((const uschar **)&p);
-    p++;
-    }
-  else
-    {
-    r = Ustrchr(p, '\\');
-    if (!r)
-      {
-      off = Ustrlen(p);
-      memcpy(q, p, off);
-      p += off;
-      q += off;
-      break;
-      }
-    else
-      {
-      off = r - p;
-      memcpy(q, p, off);
-      q += off;
-      p = r;
-      }
-    }
-  }
-*q = '\0';
-
-return ss;
-}
-
 
 
 
@@ -409,18 +338,16 @@ return ss;
 *            Copy and save string                *
 *************************************************/
 
-/* This function assumes that memcpy() is faster than strcpy().
-
+/*
 Argument: string to copy
 Returns:  copy of string in new store
 */
 
 uschar *
-string_copy(const uschar *s)
+string_copy(uschar *s)
 {
-int len = Ustrlen(s) + 1;
-uschar *ss = store_get(len);
-memcpy(ss, s, len);
+uschar *ss = store_get(Ustrlen(s) + 1);
+Ustrcpy(ss, s);
 return ss;
 }
 
@@ -430,18 +357,16 @@ return ss;
 *     Copy and save string in malloc'd store     *
 *************************************************/
 
-/* This function assumes that memcpy() is faster than strcpy().
-
+/*
 Argument: string to copy
 Returns:  copy of string in new store
 */
 
 uschar *
-string_copy_malloc(const uschar *s)
+string_copy_malloc(uschar *s)
 {
-int len = Ustrlen(s) + 1;
-uschar *ss = store_malloc(len);
-memcpy(ss, s, len);
+uschar *ss = store_malloc(Ustrlen(s) + 1);
+Ustrcpy(ss, s);
 return ss;
 }
 
@@ -457,7 +382,7 @@ Returns:  copy of string in new store, with letters lowercased
 */
 
 uschar *
-string_copylc(const uschar *s)
+string_copylc(uschar *s)
 {
 uschar *ss = store_get(Ustrlen(s) + 1);
 uschar *p = ss;
@@ -483,7 +408,7 @@ Returns:    copy of string in new store
 */
 
 uschar *
-string_copyn(const uschar *s, int n)
+string_copyn(uschar *s, int n)
 {
 uschar *ss = store_get(n + 1);
 Ustrncpy(ss, s, n);
@@ -518,67 +443,6 @@ return ss;
 
 
 
-/*************************************************
-*    Copy string if long, inserting newlines     *
-*************************************************/
-
-/* If the given string is longer than 75 characters, it is copied, and within
-the copy, certain space characters are converted into newlines.
-
-Argument:  pointer to the string
-Returns:   pointer to the possibly altered string
-*/
-
-uschar *
-string_split_message(uschar *msg)
-{
-uschar *s, *ss;
-
-if (msg == NULL || Ustrlen(msg) <= 75) return msg;
-s = ss = msg = string_copy(msg);
-
-for (;;)
-  {
-  int i = 0;
-  while (i < 75 && *ss != 0 && *ss != '\n') ss++, i++;
-  if (*ss == 0) break;
-  if (*ss == '\n')
-    s = ++ss;
-  else
-    {
-    uschar *t = ss + 1;
-    uschar *tt = NULL;
-    while (--t > s + 35)
-      {
-      if (*t == ' ')
-        {
-        if (t[-1] == ':') { tt = t; break; }
-        if (tt == NULL) tt = t;
-        }
-      }
-
-    if (tt == NULL)          /* Can't split behind - try ahead */
-      {
-      t = ss + 1;
-      while (*t != 0)
-        {
-        if (*t == ' ' || *t == '\n')
-          { tt = t; break; }
-        t++;
-        }
-      }
-
-    if (tt == NULL) break;   /* Can't find anywhere to split */
-    *tt = '\n';
-    s = ss = tt+1;
-    }
-  }
-
-return msg;
-}
-
-
-
 /*************************************************
 *   Copy returned DNS domain name, de-escaping   *
 *************************************************/
@@ -639,9 +503,9 @@ Returns:   the new string
 */
 
 uschar *
-string_dequote(const uschar **sptr)
+string_dequote(uschar **sptr)
 {
-const uschar *s = *sptr;
+uschar *s = *sptr;
 uschar *t, *yield;
 
 /* First find the end of the string */
@@ -710,15 +574,14 @@ Returns:    pointer to fresh piece of store containing sprintf'ed string
 */
 
 uschar *
-string_sprintf(const char *format, ...)
+string_sprintf(char *format, ...)
 {
 va_list ap;
 uschar buffer[STRING_SPRINTF_BUFFER_SIZE];
 va_start(ap, format);
 if (!string_vformat(buffer, sizeof(buffer), format, ap))
   log_write(0, LOG_MAIN|LOG_PANIC_DIE,
-    "string_sprintf expansion was longer than " SIZE_T_FMT " (%s)",
-    sizeof(buffer), format);
+    "string_sprintf expansion was longer than %d", sizeof(buffer));
 va_end(ap);
 return string_copy(buffer);
 }
@@ -739,7 +602,7 @@ Returns:    < 0, = 0, or > 0, according to the comparison
 */
 
 int
-strncmpic(const uschar *s, const uschar *t, int n)
+strncmpic(uschar *s, uschar *t, int n)
 {
 while (n--)
   {
@@ -763,7 +626,7 @@ Returns:    < 0, = 0, or > 0, according to the comparison
 */
 
 int
-strcmpic(const uschar *s, const uschar *t)
+strcmpic(uschar *s, uschar *t)
 {
 while (*s != 0)
   {
@@ -834,26 +697,19 @@ return NULL;
 /* Leading and trailing space is removed from each item. The separator in the
 list is controlled by the int pointed to by the separator argument as follows:
 
-  If the value is > 0 it is used as the separator. This is typically used for
-  sublists such as slash-separated options. The value is always a printing
-  character.
-
-    (If the value is actually > UCHAR_MAX there is only one item in the list.
+  If its value is > 0 it is used as the delimiter.
+    (If its value is actually > UCHAR_MAX there is only one item in the list.
     This is used for some cases when called via functions that sometimes
     plough through lists, and sometimes are given single items.)
+  If its value is <= 0, the string is inspected for a leading <x, where
+    x is an ispunct() value. If found, it is used as the delimiter. If not
+    found: (a) if separator == 0, ':' is used
+           (b) if separator <0, then -separator is used
+    In all cases the value of the separator that is used is written back to
+      the int so that it is used on subsequent calls as we progress through
+      the list.
 
-  If the value is <= 0, the string is inspected for a leading <x, where x is an
-  ispunct() or an iscntrl() character. If found, x is used as the separator. If
-  not found:
-
-      (a) if separator == 0, ':' is used
-      (b) if separator <0, -separator is used
-
-  In all cases the value of the separator that is used is written back to the
-  int so that it is used on subsequent calls as we progress through the list.
-
-A literal ispunct() separator can be represented in an item by doubling, but
-there is no way to include an iscntrl() separator as part of the data.
+The separator can always be represented in the string by doubling.
 
 Arguments:
   listptr    points to a pointer to the current start of the list; the
@@ -868,30 +724,22 @@ Returns:     pointer to buffer, containing the next substring,
 */
 
 uschar *
-string_nextinlist(const uschar **listptr, int *separator, uschar *buffer, int buflen)
+string_nextinlist(uschar **listptr, int *separator, uschar *buffer, int buflen)
 {
-int sep = *separator;
-const uschar *s = *listptr;
-BOOL sep_is_special;
+register int p = 0;
+register int sep = *separator;
+register uschar *s = *listptr;
 
 if (s == NULL) return NULL;
-
-/* This allows for a fixed specified separator to be an iscntrl() character,
-but at the time of implementation, this is never the case. However, it's best
-to be conservative. */
-
-while (isspace(*s) && *s != sep) s++;
-
-/* A change of separator is permitted, so look for a leading '<' followed by an
-allowed character. */
+while (isspace(*s)) s++;
 
 if (sep <= 0)
   {
-  if (*s == '<' && (ispunct(s[1]) || iscntrl(s[1])))
+  if (*s == '<' && ispunct(s[1]))
     {
     sep = s[1];
     s += 2;
-    while (isspace(*s) && *s != sep) s++;
+    while (isspace(*s)) s++;
     }
   else
     {
@@ -900,22 +748,15 @@ if (sep <= 0)
   *separator = sep;
   }
 
-/* An empty string has no list elements */
-
 if (*s == 0) return NULL;
 
-/* Note whether whether or not the separator is an iscntrl() character. */
-
-sep_is_special = iscntrl(sep);
-
 /* Handle the case when a buffer is provided. */
 
 if (buffer != NULL)
   {
-  register int p = 0;
   for (; *s != 0; s++)
     {
-    if (*s == sep && (*(++s) != sep || sep_is_special)) break;
+    if (*s == sep && *(++s) != sep) break;
     if (p < buflen - 1) buffer[p++] = *s;
     }
   while (p > 0 && isspace(buffer[p-1])) p--;
@@ -926,37 +767,31 @@ if (buffer != NULL)
 
 else
   {
-  int size = 0;
-  int ptr = 0;
-  const uschar *ss;
-
   /* We know that *s != 0 at this point. However, it might be pointing to a
-  separator, which could indicate an empty string, or (if an ispunct()
-  character) could be doubled to indicate a separator character as data at the
-  start of a string. Avoid getting working memory for an empty item. */
+  separator, which could indicate an empty string, or could be doubled to
+  indicate a separator character as data at the start of a string. */
 
   if (*s == sep)
     {
     s++;
-    if (*s != sep || sep_is_special)
-      {
-      *listptr = s;
-      return string_copy(US"");
-      }
+    if (*s != sep) buffer = string_copy(US"");
     }
 
-  /* Not an empty string; the first character is guaranteed to be a data
-  character. */
-
-  for (;;)
+  if (buffer == NULL)
     {
-    for (ss = s + 1; *ss != 0 && *ss != sep; ss++);
-    buffer = string_cat(buffer, &size, &ptr, s, ss-s);
-    s = ss;
-    if (*s == 0 || *(++s) != sep || sep_is_special) break;
+    int size = 0;
+    int ptr = 0;
+    uschar *ss;
+    for (;;)
+      {
+      for (ss = s + 1; *ss != 0 && *ss != sep; ss++);
+      buffer = string_cat(buffer, &size, &ptr, s, ss-s);
+      s = ss;
+      if (*s == 0 || *(++s) != sep) break;
+      }
+    while (ptr > 0 && isspace(buffer[ptr-1])) ptr--;
+    buffer[ptr] = 0;
     }
-  while (ptr > 0 && isspace(buffer[ptr-1])) ptr--;
-  buffer[ptr] = 0;
   }
 
 /* Update the current pointer and return the new string */
@@ -967,95 +802,6 @@ return buffer;
 #endif  /* COMPILE_UTILITY */
 
 
-#ifndef COMPILE_UTILITY
-/************************************************
-*	Add element to seperated list           *
-************************************************/
-/* This function is used to build a list, returning
-an allocated null-terminated growable string. The
-given element has any embedded seperator characters
-doubled.
-
-Arguments:
-  list	points to the start of the list that is being built, or NULL
-	if this is a new list that has no contents yet
-  sep	list seperator charactoer
-  ele	new lement to be appended to the list
-
-Returns:  pointer to the start of the list, changed if copied for expansion.
-*/
-
-uschar *
-string_append_listele(uschar * list, uschar sep, const uschar * ele)
-{
-uschar * new = NULL;
-int sz = 0, off = 0;
-uschar * sp;
-
-if (list)
-  {
-  new = string_cat(new, &sz, &off, list, Ustrlen(list));
-  new = string_cat(new, &sz, &off, &sep, 1);
-  }
-
-while((sp = Ustrchr(ele, sep)))
-  {
-  new = string_cat(new, &sz, &off, ele, sp-ele+1);
-  new = string_cat(new, &sz, &off, &sep, 1);
-  ele = sp+1;
-  }
-new = string_cat(new, &sz, &off, ele, Ustrlen(ele));
-new[off] = '\0';
-return new;
-}
-
-
-static const uschar *
-Ustrnchr(const uschar * s, int c, unsigned * len)
-{
-unsigned siz = *len;
-while (siz)
-  {
-  if (!*s) return NULL;
-  if (*s == c)
-    {
-    *len = siz;
-    return s;
-    }
-  s++;
-  siz--;
-  }
-return NULL;
-}
-
-uschar *
-string_append_listele_n(uschar * list, uschar sep, const uschar * ele,
-  unsigned len)
-{
-uschar * new = NULL;
-int sz = 0, off = 0;
-const uschar * sp;
-
-if (list)
-  {
-  new = string_cat(new, &sz, &off, list, Ustrlen(list));
-  new = string_cat(new, &sz, &off, &sep, 1);
-  }
-
-while((sp = Ustrnchr(ele, sep, &len)))
-  {
-  new = string_cat(new, &sz, &off, ele, sp-ele+1);
-  new = string_cat(new, &sz, &off, &sep, 1);
-  ele = sp+1;
-  len--;
-  }
-new = string_cat(new, &sz, &off, ele, len);
-new[off] = '\0';
-return new;
-}
-#endif  /* COMPILE_UTILITY */
-
-
 
 #ifndef COMPILE_UTILITY
 /*************************************************
@@ -1121,7 +867,7 @@ if (p + count >= *size)
 
   else if (!store_extend(string, oldsize, *size))
     {
-    BOOL release_ok = store_last_get[store_pool] == string;
+    BOOL release_ok = store_last_get == string;
     uschar *newstring = store_get(*size);
     memcpy(newstring, string, p);
     if (release_ok) store_release(string);
@@ -1141,49 +887,6 @@ return string;
 
 
 
-#ifndef COMPILE_UTILITY
-/*************************************************
-*        Append strings to another string        *
-*************************************************/
-
-/* This function can be used to build a string from many other strings.
-It calls string_cat() to do the dirty work.
-
-Arguments:
-  string   points to the start of the string that is being built, or NULL
-             if this is a new string that has no contents yet
-  size     points to a variable that holds the current capacity of the memory
-             block (updated if changed)
-  ptr      points to a variable that holds the offset at which to add
-             characters, updated to the new offset
-  count    the number of strings to append
-  ...      "count" uschar* arguments, which must be valid zero-terminated
-             C strings
-
-Returns:   pointer to the start of the string, changed if copied for expansion.
-           The string is not zero-terminated - see string_cat() above.
-*/
-
-uschar *
-string_append(uschar *string, int *size, int *ptr, int count, ...)
-{
-va_list ap;
-int i;
-
-va_start(ap, count);
-for (i = 0; i < count; i++)
-  {
-  uschar *t = va_arg(ap, uschar *);
-  string = string_cat(string, size, ptr, t, Ustrlen(t));
-  }
-va_end(ap);
-
-return string;
-}
-#endif
-
-
-
 /*************************************************
 *        Format a string with length checks      *
 *************************************************/
@@ -1196,9 +899,9 @@ on whether the variable length list of data arguments are given explicitly or
 as a va_list item.
 
 The formats are the usual printf() ones, with some omissions (never used) and
-two additions for strings: %S forces lower case, and %#s or %#S prints nothing
-for a NULL string. Without the # "NULL" is printed (useful in debugging). There
-is also the addition of %D and %M, which insert the date in the form used for
+two additions for strings: %S forces lower case, %#s or %#S prints nothing for
+a NULL string. Without the # "NULL" is printed (useful in debugging). There is
+also the addition of %D, which inserts the date in the form used for
 datestamped log files.
 
 Arguments:
@@ -1211,7 +914,7 @@ Returns:       TRUE if the result fitted in the buffer
 */
 
 BOOL
-string_format(uschar *buffer, int buflen, const char *format, ...)
+string_format(uschar *buffer, int buflen, char *format, ...)
 {
 BOOL yield;
 va_list ap;
@@ -1223,30 +926,24 @@ return yield;
 
 
 BOOL
-string_vformat(uschar *buffer, int buflen, const char *format, va_list ap)
+string_vformat(uschar *buffer, int buflen, char *format, va_list ap)
 {
-/* We assume numbered ascending order, C does not guarantee that */
-enum { L_NORMAL=1, L_SHORT=2, L_LONG=3, L_LONGLONG=4, L_LONGDOUBLE=5, L_SIZE=6 };
-
 BOOL yield = TRUE;
 int width, precision;
-const char *fp = format;       /* Deliberately not unsigned */
+char *fp = format;             /* Deliberately not unsigned */
 uschar *p = buffer;
 uschar *last = buffer + buflen - 1;
 
 string_datestamp_offset = -1;  /* Datestamp not inserted */
-string_datestamp_length = 0;   /* Datestamp not inserted */
-string_datestamp_type = 0;     /* Datestamp not inserted */
 
 /* Scan the format and handle the insertions */
 
 while (*fp != 0)
   {
-  int length = L_NORMAL;
   int *nptr;
   int slen;
-  const char *null = "NULL";   /* ) These variables */
-  const char *item_start, *s;  /* ) are deliberately */
+  char *null = "NULL";         /* ) These variables */
+  char *item_start, *s;        /* ) are deliberately */
   char newformat[16];          /* ) not unsigned */
 
   /* Non-% characters just get copied verbatim */
@@ -1296,27 +993,7 @@ while (*fp != 0)
       }
     }
 
-  /* Skip over 'h', 'L', 'l', 'll' and 'z', remembering the item length */
-
-  if (*fp == 'h')
-    { fp++; length = L_SHORT; }
-  else if (*fp == 'L')
-    { fp++; length = L_LONGDOUBLE; }
-  else if (*fp == 'l')
-    {
-    if (fp[1] == 'l')
-      {
-      fp += 2;
-      length = L_LONGLONG;
-      }
-    else
-      {
-      fp++;
-      length = L_LONG;
-      }
-    }
-  else if (*fp == 'z')
-    { fp++; length = L_SIZE; }
+  if (strchr("hlL", *fp) != NULL) fp++;
 
   /* Handle each specific format type. */
 
@@ -1332,22 +1009,10 @@ while (*fp != 0)
     case 'u':
     case 'x':
     case 'X':
-    if (p >= last - ((length > L_LONG)? 24 : 12))
-      { yield = FALSE; goto END_FORMAT; }
+    if (p >= last - 12) { yield = FALSE; goto END_FORMAT; }
     strncpy(newformat, item_start, fp - item_start);
     newformat[fp - item_start] = 0;
-
-    /* Short int is promoted to int when passing through ..., so we must use
-    int for va_arg(). */
-
-    switch(length)
-      {
-      case L_SHORT:
-      case L_NORMAL:   sprintf(CS p, newformat, va_arg(ap, int)); break;
-      case L_LONG:     sprintf(CS p, newformat, va_arg(ap, long int)); break;
-      case L_LONGLONG: sprintf(CS p, newformat, va_arg(ap, LONGLONG_T)); break;
-      case L_SIZE:     sprintf(CS p, newformat, va_arg(ap, size_t)); break;
-      }
+    sprintf(CS p, newformat, va_arg(ap, int));
     while (*p) p++;
     break;
 
@@ -1360,11 +1025,9 @@ while (*fp != 0)
     break;
 
     /* %f format is inherently insecure if the numbers that it may be
-    handed are unknown (e.g. 1e300). However, in Exim, %f is used for
-    printing load averages, and these are actually stored as integers
-    (load average * 1000) so the size of the numbers is constrained.
-    It is also used for formatting sending rates, where the simplicity
-    of the format prevents overflow. */
+    handed are unknown (e.g. 1e300). However, in Exim, the only use of %f
+    is for printing load averages, and these are actually stored as integers
+    (load average * 1000) so the size of the numbers is constrained. */
 
     case 'f':
     case 'e':
@@ -1375,10 +1038,7 @@ while (*fp != 0)
     if (p >= last - precision - 8) { yield = FALSE; goto END_FORMAT; }
     strncpy(newformat, item_start, fp - item_start);
     newformat[fp-item_start] = 0;
-    if (length == L_LONGDOUBLE)
-      sprintf(CS p, newformat, va_arg(ap, long double));
-    else
-      sprintf(CS p, newformat, va_arg(ap, double));
+    sprintf(CS p, newformat, va_arg(ap, double));
     while (*p) p++;
     break;
 
@@ -1394,31 +1054,19 @@ while (*fp != 0)
     *p++ = va_arg(ap, int);
     break;
 
-    case 'D':                   /* Insert daily datestamp for log file names */
-    s = CS tod_stamp(tod_log_datestamp_daily);
+    case 'D':                   /* Insert datestamp for log file names */
+    s = CS tod_stamp(tod_log_datestamp);
     string_datestamp_offset = p - buffer;   /* Passed back via global */
-    string_datestamp_length = Ustrlen(s);   /* Passed back via global */
-    string_datestamp_type = tod_log_datestamp_daily;
-    slen = string_datestamp_length;
-    goto INSERT_STRING;
-
-    case 'M':                   /* Insert monthly datestamp for log file names */
-    s = CS tod_stamp(tod_log_datestamp_monthly);
-    string_datestamp_offset = p - buffer;   /* Passed back via global */
-    string_datestamp_length = Ustrlen(s);   /* Passed back via global */
-    string_datestamp_type = tod_log_datestamp_monthly;
-    slen = string_datestamp_length;
     goto INSERT_STRING;
 
     case 's':
     case 'S':                   /* Forces *lower* case */
     s = va_arg(ap, char *);
 
+    INSERT_STRING:              /* Come to from %D above */
     if (s == NULL) s = null;
     slen = Ustrlen(s);
 
-    INSERT_STRING:              /* Come to from %D or %M above */
-
     /* If the width is specified, check that there is a precision
     set; if not, set it to the width to prevent overruns of long
     strings. */
@@ -1444,17 +1092,10 @@ while (*fp != 0)
     not OK, add part of the string (debugging uses this to show as
     much as possible). */
 
-    if (p == last)
-      {
-      yield = FALSE;
-      goto END_FORMAT;
-      }
     if (p >= last - width)
       {
       yield = FALSE;
       width = precision = last - p - 1;
-      if (width < 0) width = 0;
-      if (precision < 0) precision = 0;
       }
     sprintf(CS p, "%*.*s", width, precision, s);
     if (fp[-1] == 'S')
@@ -1485,6 +1126,7 @@ return yield;
 
 
 
+
 #ifndef COMPILE_UTILITY
 /*************************************************
 *       Generate an "open failed" message        *
@@ -1503,7 +1145,7 @@ Returns:        a message, in dynamic store
 */
 
 uschar *
-string_open_failed(int eno, const char *format, ...)
+string_open_failed(int eno, char *format, ...)
 {
 va_list ap;
 uschar buffer[1024];
@@ -1547,35 +1189,14 @@ static uschar *
 string_get_localpart(address_item *addr, uschar *yield, int *sizeptr,
   int *ptrptr)
 {
-uschar * s;
-
-s = addr->prefix;
-if (testflag(addr, af_include_affixes) && s)
-  {
-#ifdef EXPERIMENTAL_INTERNATIONAL
-  if (testflag(addr, af_utf8_downcvt))
-    s = string_localpart_utf8_to_alabel(s, NULL);
-#endif
-  yield = string_cat(yield, sizeptr, ptrptr, s, Ustrlen(s));
-  }
-
-s = addr->local_part;
-#ifdef EXPERIMENTAL_INTERNATIONAL
-if (testflag(addr, af_utf8_downcvt))
-  s = string_localpart_utf8_to_alabel(s, NULL);
-#endif
-yield = string_cat(yield, sizeptr, ptrptr, s, Ustrlen(s));
-
-s = addr->suffix;
-if (testflag(addr, af_include_affixes) && s)
-  {
-#ifdef EXPERIMENTAL_INTERNATIONAL
-  if (testflag(addr, af_utf8_downcvt))
-    s = string_localpart_utf8_to_alabel(s, NULL);
-#endif
-  yield = string_cat(yield, sizeptr, ptrptr, s, Ustrlen(s));
-  }
-
+if (testflag(addr, af_include_affixes) && addr->prefix != NULL)
+  yield = string_cat(yield, sizeptr, ptrptr, addr->prefix,
+    Ustrlen(addr->prefix));
+yield = string_cat(yield, sizeptr, ptrptr, addr->local_part,
+  Ustrlen(addr->local_part));
+if (testflag(addr, af_include_affixes) && addr->suffix != NULL)
+  yield = string_cat(yield, sizeptr, ptrptr, addr->suffix,
+    Ustrlen(addr->suffix));
 return yield;
 }
 
@@ -1636,15 +1257,10 @@ else
   {
   if (addr->local_part != NULL)
     {
-    const uschar * s;
     yield = string_get_localpart(addr, yield, &size, &ptr);
     yield = string_cat(yield, &size, &ptr, US"@", 1);
-    s = addr->domain;
-#ifdef EXPERIMENTAL_INTERNATIONAL
-    if (testflag(addr, af_utf8_downcvt))
-      s = string_localpart_utf8_to_alabel(s, NULL);
-#endif
-    yield = string_cat(yield, &size, &ptr, s, Ustrlen(s) );
+    yield = string_cat(yield, &size, &ptr, addr->domain,
+      Ustrlen(addr->domain) );
     }
   else
     {
@@ -1706,17 +1322,6 @@ return yield;
 #endif  /* COMPILE_UTILITY */
 
 
-#ifndef COMPILE_UTILITY
-/* qsort(3), currently used to sort the environment variables
-for -bP environment output, needs a function to compare two pointers to string
-pointers. Here it is. */
-
-int
-string_compare_by_pointer(const void *a, const void *b)
-{
-return Ustrcmp(* CUSS a, * CUSS b);
-}
-#endif /* COMPILE_UTILITY */
 
 
 
@@ -1782,10 +1387,8 @@ printf("Testing string_format\n");
 while (fgets(CS buffer, sizeof(buffer), stdin) != NULL)
   {
   void *args[3];
-  long long llargs[3];
   double dargs[3];
   int dflag = 0;
-  int llflag = 0;
   int n = 0;
   int count;
   int countset = 0;
@@ -1816,11 +1419,6 @@ while (fgets(CS buffer, sizeof(buffer), stdin) != NULL)
         dflag = 1;
         dargs[n++] = Ustrtod(outbuf, NULL);
         }
-      else if (Ustrstr(outbuf, "ll") != NULL)
-        {
-        llflag = 1;
-        llargs[n++] = strtoull(CS outbuf, NULL, 10);
-        }
       else
         {
         args[n++] = (void *)Uatoi(outbuf);
@@ -1843,16 +1441,11 @@ while (fgets(CS buffer, sizeof(buffer), stdin) != NULL)
     if (*s == ',') s++;
     }
 
-  if (!dflag && !llflag)
-    printf("%s\n", string_format(outbuf, sizeof(outbuf), CS format,
-      args[0], args[1], args[2])? "True" : "False");
-
-  else if (dflag)
-    printf("%s\n", string_format(outbuf, sizeof(outbuf), CS format,
-      dargs[0], dargs[1], dargs[2])? "True" : "False");
+  if (!dflag) printf("%s\n", string_format(outbuf, sizeof(outbuf), CS format,
+    args[0], args[1], args[2])? "True" : "False");
 
   else printf("%s\n", string_format(outbuf, sizeof(outbuf), CS format,
-    llargs[0], llargs[1], llargs[2])? "True" : "False");
+    dargs[0], dargs[1], dargs[2])? "True" : "False");
 
   printf("%s\n", CS outbuf);
   if (countset) printf("count=%d\n", count);
