@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -36,8 +36,6 @@ optionlist autoreply_transport_options[] = {
       (void *)offsetof(autoreply_transport_options_block, logfile) },
   { "mode",              opt_octint,
       (void *)offsetof(autoreply_transport_options_block, mode) },
-  { "never_mail",        opt_stringptr,
-      (void *)offsetof(autoreply_transport_options_block, never_mail) },
   { "once",              opt_stringptr,
       (void *)offsetof(autoreply_transport_options_block, oncelog) },
   { "once_file_size",    opt_int,
@@ -77,7 +75,6 @@ autoreply_transport_options_block autoreply_transport_option_defaults = {
   NULL,           /* logfile */
   NULL,           /* oncelog */
   NULL,           /* once_repeat */
-  NULL,           /* never_mail */
   0600,           /* mode */
   0,              /* once_file_size */
   FALSE,          /* file_expand */
@@ -157,13 +154,12 @@ if (ss == NULL)
 if (type != cke_text) for (t = ss; *t != 0; t++)
   {
   int c = *t;
-  const uschar * sp;
   if (mac_isprint(c)) continue;
-  if (type == cke_hdr && c == '\n' && (t[1] == ' ' || t[1] == '\t')) continue;
-  sp = string_printing(s);
+  if (type == cke_hdr && c == '\n' && isspace(t[1])) continue;
+  s = string_printing(s);
   addr->transport_return = FAIL;
   addr->message = string_sprintf("Expansion of \"%s\" in %s transport "
-    "contains non-printing character %d", sp, name, c);
+    "contains non-printing character %d", s, name, c);
   return NULL;
   }
 
@@ -173,85 +169,6 @@ return ss;
 
 
 
-/*************************************************
-*          Check a header line for never_mail    *
-*************************************************/
-
-/* This is called to check to, cc, and bcc for addresses in the never_mail
-list. Any that are found are removed.
-
-Arguments:
-  listptr     points to the list of addresses
-  never_mail  an address list, already expanded
-
-Returns:      nothing
-*/
-
-static void
-check_never_mail(uschar **listptr, const uschar *never_mail)
-{
-uschar *s = *listptr;
-
-while (*s != 0)
-  {
-  uschar *error, *next;
-  uschar *e = parse_find_address_end(s, FALSE);
-  int terminator = *e;
-  int start, end, domain, rc;
-
-  /* Temporarily terminate the string at the address end while extracting
-  the operative address within. */
-
-  *e = 0;
-  next = parse_extract_address(s, &error, &start, &end, &domain, FALSE);
-  *e = terminator;
-
-  /* If there is some kind of syntax error, just give up on this header
-  line. */
-
-  if (next == NULL) break;
-
-  /* See if the address is on the never_mail list */
-
-  rc = match_address_list(next,         /* address to check */
-                          TRUE,         /* start caseless */
-                          FALSE,        /* don't expand the list */
-                          &never_mail,  /* the list */
-                          NULL,         /* no caching */
-                          -1,           /* no expand setup */
-                          0,            /* separator from list */
-                          NULL);        /* no lookup value return */
-
-  if (rc == OK)                         /* Remove this address */
-    {
-    DEBUG(D_transport)
-      debug_printf("discarding recipient %s (matched never_mail)\n", next);
-    if (terminator == ',') e++;
-    memmove(s, e, Ustrlen(e) + 1);
-    }
-  else                                  /* Skip over this address */
-    {
-    s = e;
-    if (terminator == ',') s++;
-    }
-  }
-
-/* Check to see if we removed the last address, leaving a terminating comma
-that needs to be removed */
-
-s = *listptr + Ustrlen(*listptr);
-while (s > *listptr && (isspace(s[-1]) || s[-1] == ',')) s--;
-*s = 0;
-
-/* Check to see if there any addresses left; if not, set NULL */
-
-s = *listptr;
-while (s != 0 && isspace(*s)) s++;
-if (*s == 0) *listptr = NULL;
-}
-
-
-
 /*************************************************
 *              Main entry point                  *
 *************************************************/
@@ -276,7 +193,6 @@ uschar *from, *reply_to, *to, *cc, *bcc, *subject, *headers, *text, *file;
 uschar *logfile, *oncelog;
 uschar *cache_buff = NULL;
 uschar *cache_time = NULL;
-uschar *message_id = NULL;
 header_line *h;
 time_t now = time(NULL);
 time_t once_repeat_sec = 0;
@@ -375,33 +291,6 @@ else
     }
   }
 
-/* If the never_mail option is set, we have to scan all the recipients and
-remove those that match. */
-
-if (ob->never_mail != NULL)
-  {
-  const uschar *never_mail = expand_string(ob->never_mail);
-
-  if (never_mail == NULL)
-    {
-    addr->transport_return = FAIL;
-    addr->message = string_sprintf("Failed to expand \"%s\" for "
-      "\"never_mail\" in %s transport", ob->never_mail, tblock->name);
-    return FALSE;
-    }
-
-  if (to != NULL) check_never_mail(&to, never_mail);
-  if (cc != NULL) check_never_mail(&cc, never_mail);
-  if (bcc != NULL) check_never_mail(&bcc, never_mail);
-
-  if (to == NULL && cc == NULL && bcc == NULL)
-    {
-    DEBUG(D_transport)
-      debug_printf("*** all recipients removed by never_mail\n");
-    return OK;
-    }
-  }
-
 /* If the -N option is set, can't do any more. */
 
 if (dont_deliver)
@@ -420,7 +309,7 @@ recipient, the effect might not be quite as envisaged. If once_file_size is
 set, instead of a dbm file, we use a regular file containing a circular buffer
 recipient cache. */
 
-if (oncelog != NULL && *oncelog != 0 && to != NULL)
+if (oncelog != NULL && to != NULL)
   {
   time_t then = 0;
 
@@ -530,10 +419,8 @@ if (oncelog != NULL && *oncelog != 0 && to != NULL)
       uschar *ptr = log_buffer;
       sprintf(CS ptr, "%s\n  previously sent to %.200s\n", tod_stamp(tod_log), to);
       while(*ptr) ptr++;
-      if(write(log_fd, log_buffer, ptr - log_buffer) != ptr-log_buffer
-        || close(log_fd))
-        DEBUG(D_transport) debug_printf("Problem writing log file %s for %s "
-          "transport\n", logfile, tblock->name);
+      write(log_fd, log_buffer, ptr - log_buffer);
+      close(log_fd);
       }
     goto END_OFF;
     }
@@ -592,57 +479,9 @@ for (h = header_list; h != NULL; h = h->next)
 
 if (h != NULL)
   {
-  message_id = Ustrchr(h->text, ':') + 1;
-  while (isspace(*message_id)) message_id++;
-  fprintf(f, "In-Reply-To: %s", message_id);
-  }
-
-/* Generate a References header if there is at least one of Message-ID:,
-References:, or In-Reply-To: (see RFC 2822). */
-
-for (h = header_list; h != NULL; h = h->next)
-  if (h->type != htype_old && strncmpic(US"References:", h->text, 11) == 0)
-    break;
-
-if (h == NULL)
-  for (h = header_list; h != NULL; h = h->next)
-    if (h->type != htype_old && strncmpic(US"In-Reply-To:", h->text, 12) == 0)
-      break;
-
-/* We limit the total length of references.  Although there is no fixed
-limit, some systems do not like headers growing beyond recognition.
-Keep the first message ID for the thread root and the last few for
-the position inside the thread, up to a maximum of 12 altogether. */
-
-if (h != NULL || message_id != NULL)
-  {
-  fprintf(f, "References:");
-  if (h != NULL)
-    {
-    uschar *s, *id, *error;
-    uschar *referenced_ids[12];
-    int reference_count = 0;
-    int i;
-
-    s = Ustrchr(h->text, ':') + 1;
-    parse_allow_group = FALSE;
-    while (*s != 0 && (s = parse_message_id(s, &id, &error)) != NULL)
-      {
-      if (reference_count == sizeof(referenced_ids)/sizeof(uschar *))
-        {
-        memmove(referenced_ids + 1, referenced_ids + 2,
-           sizeof(referenced_ids) - 2*sizeof(uschar *));
-        referenced_ids[reference_count - 1] = id;
-        }
-      else referenced_ids[reference_count++] = id;
-      }
-    for (i = 0; i < reference_count; ++i) fprintf(f, " %s", referenced_ids[i]);
-    }
-
-  /* The message id will have a newline on the end of it. */
-
-  if (message_id != NULL) fprintf(f, " %s", message_id);
-    else fprintf(f, "\n");
+  uschar *s = Ustrchr(h->text, ':') + 1;
+  while (isspace(*s)) s++;
+  fprintf(f, "In-Reply-To: %s", s);
   }
 
 /* Add an Auto-Submitted: header */
@@ -677,36 +516,31 @@ if (ff != NULL)
       }
     else fprintf(f, "%s", CS big_buffer);
     }
-  (void) fclose(ff);
   }
 
 /* Copy the original message if required, observing the return size
-limit if we are returning the body. */
+limit. */
 
 if (return_message)
   {
-  uschar *rubric = (tblock->headers_only)?
-    US"------ This is a copy of the message's header lines.\n"
-    : (tblock->body_only)?
-    US"------ This is a copy of the body of the message, without the headers.\n"
-    :
-    US"------ This is a copy of the message, including all the headers.\n";
-
-  if (bounce_return_size_limit > 0 && !tblock->headers_only)
+  if (bounce_return_size_limit > 0)
     {
     struct stat statbuf;
     int max = (bounce_return_size_limit/DELIVER_IN_BUFFER_SIZE + 1) *
       DELIVER_IN_BUFFER_SIZE;
     if (fstat(deliver_datafile, &statbuf) == 0 && statbuf.st_size > max)
       {
-      fprintf(f, "\n%s"
-"------ The body of the message is " OFF_T_FMT " characters long; only the first\n"
-"------ %d or so are included here.\n\n", rubric, statbuf.st_size,
-        (max/1000)*1000);
+      int size = statbuf.st_size;  /* Because might be a long */
+      fprintf(f, "\n"
+"------ This is a copy of the message, including all the headers.\n"
+"------ The body of the message is %d characters long; only the first\n"
+"------ %d or so are included here.\n\n", size, (max/1000)*1000);
       }
-    else fprintf(f, "\n%s\n", rubric);
+    else fprintf(f, "\n"
+"------ This is a copy of the message, including all the headers. ------\n\n");
     }
-  else fprintf(f, "\n%s\n", rubric);
+  else fprintf(f, "\n"
+"------ This is a copy of the message, including all the headers. ------\n\n");
 
   fflush(f);
   transport_count = 0;
@@ -722,7 +556,7 @@ if (return_message)
 
 /* End the message and wait for the child process to end; no timeout. */
 
-(void)fclose(f);
+fclose(f);
 rc = child_close(pid, 0);
 
 /* Update the "sent to" log whatever the yield. This errs on the side of
@@ -757,9 +591,7 @@ if (cache_fd >= 0)
     }
 
   memcpy(cache_time, &now, sizeof(time_t));
-  if(write(cache_fd, from, size) != size)
-    DEBUG(D_transport) debug_printf("Problem writing cache file %s for %s "
-      "transport\n", oncelog, tblock->name);
+  write(cache_fd, from, size);
   }
 
 /* Update DBM file */
@@ -855,10 +687,8 @@ if (logfile != NULL)
         "  %s\n", headers);
       while(*ptr) ptr++;
       }
-    if(write(log_fd, log_buffer, ptr - log_buffer) != ptr-log_buffer
-      || close(log_fd))
-      DEBUG(D_transport) debug_printf("Problem writing log file %s for %s "
-        "transport\n", logfile, tblock->name);
+    write(log_fd, log_buffer, ptr - log_buffer);
+    close(log_fd);
     }
   else DEBUG(D_transport) debug_printf("Failed to open log file %s for %s "
     "transport: %s\n", logfile, tblock->name, strerror(errno));
@@ -866,7 +696,7 @@ if (logfile != NULL)
 
 END_OFF:
 if (dbm_file != NULL) EXIM_DBCLOSE(dbm_file);
-if (cache_fd > 0) (void)close(cache_fd);
+if (cache_fd > 0) close(cache_fd);
 
 DEBUG(D_transport) debug_printf("%s transport succeeded\n", tblock->name);
 
