@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -26,8 +26,6 @@ optionlist lmtp_transport_options[] = {
       (void *)offsetof(transport_instance, batch_max) },
   { "command",           opt_stringptr,
       (void *)offsetof(lmtp_transport_options_block, cmd) },
-  { "ignore_quota",      opt_bool,
-      (void *)offsetof(lmtp_transport_options_block, ignore_quota) },
   { "socket",            opt_stringptr,
       (void *)offsetof(lmtp_transport_options_block, skt) },
   { "timeout",           opt_time,
@@ -46,8 +44,7 @@ lmtp_transport_options_block lmtp_transport_option_defaults = {
   NULL,           /* cmd */
   NULL,           /* skt */
   5*60,           /* timeout */
-  0,              /* options */
-  FALSE           /* ignore_quota */
+  0               /* options */
 };
 
 
@@ -142,9 +139,7 @@ end the DATA. */
 
 if (*errno_value == ERRNO_FILTER_FAIL)
   {
-  *message = string_sprintf("transport filter process failed (%d)%s",
-    more_errno,
-    (more_errno == EX_EXECFAILED)? ": unable to execute command" : "");
+  *message = string_sprintf("transport filter process failed (%d)", more_errno);
   return FALSE;
   }
 
@@ -171,7 +166,7 @@ if (*errno_value == ERRNO_WRITEINCOMPLETE)
 
 if (buffer[0] != 0)
   {
-  const uschar *s = string_printing(buffer);
+  uschar *s = string_printing(buffer);
   *message = string_sprintf("LMTP error after %s: %s", big_buffer, s);
   *yield = buffer[0];
   return TRUE;
@@ -210,7 +205,7 @@ Returns:     TRUE if successful, FALSE if not, with errno set
 */
 
 static BOOL
-lmtp_write_command(int fd, const char *format, ...)
+lmtp_write_command(int fd, char *format, ...)
 {
 int count, rc;
 va_list ap;
@@ -356,8 +351,7 @@ for (;;)
     while (*t != 0)
       {
       while (*t != 0 && *t != '\n') t++;
-      debug_printf("  %s %*s\n", (s == ptr)? "LMTP<<" : "      ",
-        (int)(t-s), s);
+      debug_printf("  %s %*s\n", (s == ptr)? "LMTP<<" : "      ", t-s, s);
       if (*t == 0) break;
       s = t = t + 1;
       }
@@ -458,9 +452,8 @@ int code, save_errno;
 BOOL send_data;
 BOOL yield = FALSE;
 address_item *addr;
-uschar *igquotstr = US"";
 uschar *sockname = NULL;
-const uschar **argv;
+uschar **argv;
 uschar buffer[256];
 
 DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
@@ -469,29 +462,13 @@ DEBUG(D_transport) debug_printf("%s transport entered\n", tblock->name);
 not both. When a command is specified, call the common function for creating an
 argument list and expanding the items. */
 
-if (ob->cmd)
+if (ob->cmd != NULL)
   {
   DEBUG(D_transport) debug_printf("using command %s\n", ob->cmd);
   sprintf(CS buffer, "%.50s transport", tblock->name);
   if (!transport_set_up_command(&argv, ob->cmd, TRUE, PANIC, addrlist, buffer,
        NULL))
     return FALSE;
-
-  /* If the -N option is set, can't do any more. Presume all has gone well. */
-  if (dont_deliver)
-    goto MINUS_N;
-
-/* As this is a local transport, we are already running with the required
-uid/gid and current directory. Request that the new process be a process group
-leader, so we can kill it and all its children on an error. */
-
-  if ((pid = child_open(USS argv, NULL, 0, &fd_in, &fd_out, TRUE)) < 0)
-    {
-    addrlist->message = string_sprintf(
-      "Failed to create child process for %s transport: %s", tblock->name,
-        strerror(errno));
-    return FALSE;
-    }
   }
 
 /* When a socket is specified, expand the string and create a socket. */
@@ -514,11 +491,38 @@ else
         ob->skt, tblock->name, strerror(errno));
     return FALSE;
     }
+  }
 
-  /* If the -N option is set, can't do any more. Presume all has gone well. */
-  if (dont_deliver)
-    goto MINUS_N;
+/* If the -N option is set, can't do any more. Presume all has gone well. */
 
+if (dont_deliver)
+  {
+  DEBUG(D_transport)
+    debug_printf("*** delivery by %s transport bypassed by -N option",
+      tblock->name);
+  addrlist->transport_return = OK;
+  return FALSE;
+  }
+
+/* As this is a local transport, we are already running with the required
+uid/gid and current directory. Request that the new process be a process group
+leader, so we can kill it and all its children on an error. */
+
+if (ob->cmd != NULL)
+  {
+  if ((pid = child_open(argv, NULL, 0, &fd_in, &fd_out, TRUE)) < 0)
+    {
+    addrlist->message = string_sprintf(
+      "Failed to create child process for %s transport: %s", tblock->name,
+        strerror(errno));
+    return FALSE;
+    }
+  }
+
+/* For a socket, try to make the connection */
+
+else
+  {
   sockun.sun_family = AF_UNIX;
   sprintf(sockun.sun_path, "%.*s", (int)(sizeof(sockun.sun_path)-1), sockname);
   if(connect(fd_out, (struct sockaddr *)(&sockun), sizeof(sockun)) == -1)
@@ -530,7 +534,6 @@ else
     }
   }
 
-
 /* Make the output we are going to read into a file. */
 
 out = fdopen(fd_out, "rb");
@@ -553,27 +556,13 @@ if (!lmtp_write_command(fd_in, "%s %s\r\n", "LHLO",
 if (!lmtp_read_response(out, buffer, sizeof(buffer), '2',
      timeout)) goto RESPONSE_FAILED;
 
-/* If the ignore_quota option is set, note whether the server supports the
-IGNOREQUOTA option, and if so, set an appropriate addition for RCPT. */
-
-if (ob->ignore_quota)
-  igquotstr = (pcre_exec(regex_IGNOREQUOTA, NULL, CS buffer,
-    Ustrlen(CS buffer), 0, PCRE_EOPT, NULL, 0) >= 0)? US" IGNOREQUOTA" : US"";
-
 /* Now the envelope sender */
 
 if (!lmtp_write_command(fd_in, "MAIL FROM:<%s>\r\n", return_path))
   goto WRITE_FAILED;
 
 if (!lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
-  {
-  if (errno == 0 && buffer[0] == '4')
-    {
-    errno = ERRNO_MAIL4XX;
-    addrlist->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-    }
   goto RESPONSE_FAILED;
-  }
 
 /* Next, we hand over all the recipients. Some may be permanently or
 temporarily rejected; others may be accepted, for now. */
@@ -581,8 +570,8 @@ temporarily rejected; others may be accepted, for now. */
 send_data = FALSE;
 for (addr = addrlist; addr != NULL; addr = addr->next)
   {
-  if (!lmtp_write_command(fd_in, "RCPT TO:<%s>%s\r\n",
-       transport_rcpt_address(addr, tblock->rcpt_include_affixes), igquotstr))
+  if (!lmtp_write_command(fd_in, "RCPT TO:<%s>\r\n",
+       transport_rcpt_address(addr, tblock->rcpt_include_affixes)))
     goto WRITE_FAILED;
   if (lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
     {
@@ -594,12 +583,7 @@ for (addr = addrlist; addr != NULL; addr = addr->next)
     if (errno != 0 || buffer[0] == 0) goto RESPONSE_FAILED;
     addr->message = string_sprintf("LMTP error after %s: %s", big_buffer,
       string_printing(buffer));
-    setflag(addr, af_pass_message);   /* Allow message to go to user */
-    if (buffer[0] == '5') addr->transport_return = FAIL; else
-      {
-      addr->basic_errno = ERRNO_RCPT4XX;
-      addr->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-      }
+    if (buffer[0] == '5') addr->transport_return = FAIL;
     }
   }
 
@@ -611,14 +595,7 @@ if (send_data)
 
   if (!lmtp_write_command(fd_in, "DATA\r\n")) goto WRITE_FAILED;
   if (!lmtp_read_response(out, buffer, sizeof(buffer), '3', timeout))
-    {
-    if (errno == 0 && buffer[0] == '4')
-      {
-      errno = ERRNO_DATA4XX;
-      addrlist->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-      }
     goto RESPONSE_FAILED;
-    }
 
   sigalrm_seen = FALSE;
   transport_write_timeout = timeout;
@@ -652,15 +629,8 @@ if (send_data)
     if (addr->transport_return != PENDING_OK) continue;
 
     if (lmtp_read_response(out, buffer, sizeof(buffer), '2', timeout))
-      {
       addr->transport_return = OK;
-      if ((log_extra_selector & LX_smtp_confirmation) != 0)
-        {
-        const uschar *s = string_printing(buffer);
-	/* de-const safe here as string_printing known to have alloc'n'copied */
-        addr->message = (s == buffer)? (uschar *)string_copy(s) : US s;
-        }
-      }
+
     /* If the response has failed badly, use it for all the remaining pending
     addresses and give up. */
 
@@ -685,15 +655,9 @@ if (send_data)
 
     else
       {
-      if (buffer[0] == '4')
-        {
-        addr->basic_errno = ERRNO_DATA4XX;
-        addr->more_errno |= ((buffer[1] - '0')*10 + buffer[2] - '0') << 8;
-        }
       addr->message = string_sprintf("LMTP error after %s: %s", big_buffer,
         string_printing(buffer));
       addr->transport_return = (buffer[0] == '5')? FAIL : DEFER;
-      setflag(addr, af_pass_message);   /* Allow message to go to user */
       }
     }
   }
@@ -711,18 +675,16 @@ goto RETURN;
 
 /* Come here if any call to read_response, other than a response after the data
 phase, failed. Put the error in the top address - this will be replicated
-because the yield is still FALSE. (But omit ETIMEDOUT, as there will already be
-a suitable message.) Analyse the error, and if if isn't too bad, send a QUIT
-command. Wait for the response with a short timeout, so we don't wind up this
-process before the far end has had time to read the QUIT. */
+because the yield is still FALSE. Analyse the error, and if if isn't too bad,
+send a QUIT command. Wait for the response with a short timeout, so we don't
+wind up this process before the far end has had time to read the QUIT. */
 
 RESPONSE_FAILED:
 
 save_errno = errno;
-if (errno != ETIMEDOUT && errno != 0) addrlist->basic_errno = errno;
 addrlist->message = NULL;
 
-if (check_response(&save_errno, addrlist->more_errno,
+if(check_response(&save_errno, addrlist->more_errno,
     buffer, &code, &(addrlist->message)))
   {
   (void) lmtp_write_command(fd_in, "QUIT\r\n");
@@ -768,21 +730,13 @@ RETURN:
 
 (void)child_close(pid, timeout);
 
-if (fd_in >= 0) (void)close(fd_in);
-if (fd_out >= 0) (void)fclose(out);
+if (fd_in >= 0) (void) close(fd_in);
+if (fd_out >= 0) (void) fclose(out);
 
 DEBUG(D_transport)
   debug_printf("%s transport yields %d\n", tblock->name, yield);
 
 return yield;
-
-
-MINUS_N:
-  DEBUG(D_transport)
-    debug_printf("*** delivery by %s transport bypassed by -N option",
-      tblock->name);
-  addrlist->transport_return = OK;
-  return FALSE;
 }
 
 /* End of transport/lmtp.c */
