@@ -1,10 +1,8 @@
-/* $Cambridge: exim/src/src/transports/lmtp.h,v 1.6 2009/11/16 19:50:39 nm4 Exp $ */
-
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -14,7 +12,6 @@ typedef struct {
   uschar *skt;
   int   timeout;
   int   options;
-  BOOL  ignore_quota;
 } lmtp_transport_options_block;
 
 /* Data for reading the private options. */
