@@ -2,17 +2,13 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2015 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
 #include "../exim.h"
 #include "pipe.h"
 
-#ifdef HAVE_SETCLASSRESOURCES
-#include <login_cap.h>
-#endif
-
 
 
 /* Options specific to the pipe transport. They must be in alphabetic
@@ -37,12 +33,8 @@ optionlist pipe_transport_options[] = {
       (void *)offsetof(pipe_transport_options_block, environment) },
   { "escape_string",     opt_stringptr,
       (void *)offsetof(pipe_transport_options_block, escape_string) },
-  { "force_command",         opt_bool,
-      (void *)offsetof(pipe_transport_options_block, force_command) },
   { "freeze_exec_fail",  opt_bool,
       (void *)offsetof(pipe_transport_options_block, freeze_exec_fail) },
-  { "freeze_signal",     opt_bool,
-      (void *)offsetof(pipe_transport_options_block, freeze_signal) },
   { "ignore_status",     opt_bool,
       (void *)offsetof(pipe_transport_options_block, ignore_status) },
   { "log_defer_output",  opt_bool | opt_public,
@@ -59,8 +51,6 @@ optionlist pipe_transport_options[] = {
       (void *)offsetof(pipe_transport_options_block, message_suffix) },
   { "path",              opt_stringptr,
       (void *)offsetof(pipe_transport_options_block, path) },
-  { "permit_coredump",   opt_bool,
-      (void *)offsetof(pipe_transport_options_block, permit_coredump) },
   { "pipe_as_creator",   opt_bool | opt_public,
       (void *)offsetof(transport_instance, deliver_as_creator) },
   { "restrict_to_path",  opt_bool,
@@ -73,16 +63,10 @@ optionlist pipe_transport_options[] = {
       (void *)offsetof(pipe_transport_options_block, temp_errors) },
   { "timeout",           opt_time,
       (void *)offsetof(pipe_transport_options_block, timeout) },
-  { "timeout_defer",     opt_bool,
-      (void *)offsetof(pipe_transport_options_block, timeout_defer) },
   { "umask",             opt_octint,
       (void *)offsetof(pipe_transport_options_block, umask) },
   { "use_bsmtp",         opt_bool,
       (void *)offsetof(pipe_transport_options_block, use_bsmtp) },
-  #ifdef HAVE_SETCLASSRESOURCES
-  { "use_classresources", opt_bool,
-      (void *)offsetof(pipe_transport_options_block, use_classresources) },
-  #endif
   { "use_crlf",          opt_bool,
       (void *)offsetof(pipe_transport_options_block, use_crlf) },
   { "use_shell",         opt_bool,
@@ -101,7 +85,7 @@ pipe_transport_options_block pipe_transport_option_defaults = {
   NULL,           /* cmd */
   NULL,           /* allow_commands */
   NULL,           /* environment */
-  US"/bin:/usr/bin",  /* path */
+  US"/usr/bin",   /* path */
   NULL,           /* message_prefix (reset in init if not bsmtp) */
   NULL,           /* message_suffix (ditto) */
   US mac_expanded_string(EX_TEMPFAIL) ":"    /* temp_errors */
@@ -112,94 +96,16 @@ pipe_transport_options_block pipe_transport_option_defaults = {
   20480,          /* max_output */
   60*60,          /* timeout */
   0,              /* options */
-  FALSE,          /* force_command */
   FALSE,          /* freeze_exec_fail */
-  FALSE,          /* freeze_signal */
   FALSE,          /* ignore_status */
-  FALSE,          /* permit_coredump */
   FALSE,          /* restrict_to_path */
-  FALSE,          /* timeout_defer */
   FALSE,          /* use_shell */
   FALSE,          /* use_bsmtp */
-  FALSE,          /* use_classresources */
   FALSE           /* use_crlf */
 };
 
 
 
-/*************************************************
-*              Setup entry point                 *
-*************************************************/
-
-/* Called for each delivery in the privileged state, just before the uid/gid
-are changed and the main entry point is called. In a system that supports the
-login_cap facilities, this function is used to set the class resource limits
-for the user.  It may also re-enable coredumps.
-
-Arguments:
-  tblock     points to the transport instance
-  addrlist   addresses about to be delivered (not used)
-  dummy      not used (doesn't pass back data)
-  uid        the uid that will be set (not used)
-  gid        the gid that will be set (not used)
-  errmsg     where to put an error message
-
-Returns:     OK, FAIL, or DEFER
-*/
-
-static int
-pipe_transport_setup(transport_instance *tblock, address_item *addrlist,
-  transport_feedback *dummy, uid_t uid, gid_t gid, uschar **errmsg)
-{
-pipe_transport_options_block *ob =
-  (pipe_transport_options_block *)(tblock->options_block);
-
-addrlist = addrlist;  /* Keep compiler happy */
-dummy = dummy;
-uid = uid;
-gid = gid;
-errmsg = errmsg;
-ob = ob;
-
-#ifdef HAVE_SETCLASSRESOURCES
-if (ob->use_classresources)
-  {
-  struct passwd *pw = getpwuid(uid);
-  if (pw != NULL)
-    {
-    login_cap_t *lc = login_getpwclass(pw);
-    if (lc != NULL)
-      {
-      setclassresources(lc);
-      login_close(lc);
-      }
-    }
-  }
-#endif
-
-#ifdef RLIMIT_CORE
-if (ob->permit_coredump)
-  {
-  struct rlimit rl;
-  rl.rlim_cur = RLIM_INFINITY;
-  rl.rlim_max = RLIM_INFINITY;
-  if (setrlimit(RLIMIT_CORE, &rl) < 0)
-    {
-#ifdef SETRLIMIT_NOT_SUPPORTED
-    if (errno != ENOSYS && errno != ENOTSUP)
-#endif
-      log_write(0, LOG_MAIN,
-          "delivery setrlimit(RLIMIT_CORE, RLIM_INFINITY) failed: %s",
-          strerror(errno));
-    }
-  }
-#endif
-
-return OK;
-}
-
-
-
 /*************************************************
 *          Initialization entry point            *
 *************************************************/
@@ -214,10 +120,6 @@ pipe_transport_init(transport_instance *tblock)
 pipe_transport_options_block *ob =
   (pipe_transport_options_block *)(tblock->options_block);
 
-/* Set up the setup entry point, to be called in the privileged state */
-
-tblock->setup = pipe_transport_setup;
-
 /* If pipe_as_creator is set, then uid/gid should not be set. */
 
 if (tblock->deliver_as_creator && (tblock->uid_set || tblock->gid_set ||
@@ -326,12 +228,12 @@ Returns:             TRUE if all went well; otherwise an error will be
 */
 
 static BOOL
-set_up_direct_command(const uschar ***argvptr, uschar *cmd,
-  BOOL expand_arguments, int expand_fail, address_item *addr, uschar *tname,
+set_up_direct_command(uschar ***argvptr, uschar *cmd, BOOL expand_arguments,
+  int expand_fail, address_item *addr, uschar *tname,
   pipe_transport_options_block *ob)
 {
 BOOL permitted = FALSE;
-const uschar **argv;
+uschar **argv;
 uschar buffer[64];
 
 /* Set up "transport <name>" to be put in any error messages, and then
@@ -353,11 +255,11 @@ argv = *argvptr;
 if (ob->allow_commands != NULL)
   {
   int sep = 0;
-  const uschar *s;
-  uschar *p;
+  uschar *s, *p;
   uschar buffer[256];
 
-  if (!(s = expand_string(ob->allow_commands)))
+  s = expand_string(ob->allow_commands);
+  if (s == NULL)
     {
     addr->transport_return = DEFER;
     addr->message = string_sprintf("failed to expand string \"%s\" "
@@ -365,8 +267,10 @@ if (ob->allow_commands != NULL)
     return FALSE;
     }
 
-  while ((p = string_nextinlist(&s, &sep, buffer, sizeof(buffer))))
+  while ((p = string_nextinlist(&s, &sep, buffer, sizeof(buffer))) != NULL)
+    {
     if (Ustrcmp(p, argv[0]) == 0) { permitted = TRUE; break; }
+    }
   }
 
 /* If permitted is TRUE it means the command was found in the allowed list, and
@@ -405,7 +309,7 @@ if (argv[0][0] != '/')
   {
   int sep = 0;
   uschar *p;
-  const uschar *listptr = ob->path;
+  uschar *listptr = ob->path;
   uschar buffer[1024];
 
   while ((p = string_nextinlist(&listptr, &sep, buffer, sizeof(buffer))) != NULL)
@@ -451,10 +355,10 @@ Returns:             TRUE if all went well; otherwise an error will be
 */
 
 static BOOL
-set_up_shell_command(const uschar ***argvptr, uschar *cmd,
-  BOOL expand_arguments, int expand_fail, address_item *addr, uschar *tname)
+set_up_shell_command(uschar ***argvptr, uschar *cmd, BOOL expand_arguments,
+  int expand_fail, address_item *addr, uschar *tname)
 {
-const uschar **argv;
+uschar **argv;
 
 *argvptr = argv = store_get((4)*sizeof(uschar *));
 
@@ -549,9 +453,9 @@ pipe_transport_options_block *ob =
 int timeout = ob->timeout;
 BOOL written_ok = FALSE;
 BOOL expand_arguments;
-const uschar **argv;
+uschar **argv;
 uschar *envp[50];
-const uschar *envlist = ob->environment;
+uschar *envlist = ob->environment;
 uschar *cmd, *ss;
 uschar *eol = (ob->use_crlf)? US"\r\n" : US"\n";
 
@@ -570,21 +474,10 @@ options. */
 
 if (testflag(addr, af_pfr) && addr->local_part[0] == '|')
   {
-  if (ob->force_command)
-    {
-    /* Enables expansion of $address_pipe into seperate arguments */
-    setflag(addr, af_force_command);
-    cmd = ob->cmd;
-    expand_arguments = TRUE;
-    expand_fail = PANIC;
-    }
-  else
-    {
-    cmd = addr->local_part + 1;
-    while (isspace(*cmd)) cmd++;
-    expand_arguments = testflag(addr, af_expand_pipe);
-    expand_fail = FAIL;
-    }
+  cmd = addr->local_part + 1;
+  while (isspace(*cmd)) cmd++;
+  expand_arguments = testflag(addr, af_expand_pipe);
+  expand_fail = FAIL;
   }
 else
   {
@@ -593,12 +486,9 @@ else
   expand_fail = PANIC;
   }
 
-/* If no command has been supplied, we are in trouble.
- * We also check for an empty string since it may be
- * coming from addr->local_part[0] == '|'
- */
+/* If no command has been supplied, we are in trouble. */
 
-if (cmd == NULL || *cmd == '\0')
+if (cmd == NULL)
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf("no command specified for %s transport",
@@ -667,9 +557,9 @@ else if (timezone_string != NULL && timezone_string[0] != 0)
 
 /* Add any requested items */
 
-if (envlist)
+if (envlist != NULL)
   {
-  envlist = expand_cstring(envlist);
+  envlist = expand_string(envlist);
   if (envlist == NULL)
     {
     addr->transport_return = DEFER;
@@ -727,7 +617,7 @@ reading of the output pipe. */
 uid/gid and current directory. Request that the new process be a process group
 leader, so we can kill it and all its children on a timeout. */
 
-if ((pid = child_open(USS argv, envp, ob->umask, &fd_in, &fd_out, TRUE)) < 0)
+if ((pid = child_open(argv, envp, ob->umask, &fd_in, &fd_out, TRUE)) < 0)
   {
   addr->transport_return = DEFER;
   addr->message = string_sprintf(
@@ -745,8 +635,8 @@ if ((outpid = fork()) < 0)
   addr->message = string_sprintf(
     "Failed to create process for handling output in %s transport",
       tblock->name);
-  (void)close(fd_in);
-  (void)close(fd_out);
+  close(fd_in);
+  close(fd_out);
   return FALSE;
   }
 
@@ -759,33 +649,28 @@ the subprocess to fail. */
 if (outpid == 0)
   {
   int count = 0;
-  (void)close(fd_in);
+  close(fd_in);
   set_process_info("reading output from |%s", cmd);
   while ((rc = read(fd_out, big_buffer, big_buffer_size)) > 0)
     {
     if (addr->return_file >= 0)
-      if(write(addr->return_file, big_buffer, rc) != rc)
-        DEBUG(D_transport) debug_printf("Problem writing to return_file\n");
+      write(addr->return_file, big_buffer, rc);
     count += rc;
     if (count > ob->max_output)
       {
+      uschar *message = US"\n\n*** Too much output - remainder discarded ***\n";
       DEBUG(D_transport) debug_printf("Too much output from pipe - killed\n");
       if (addr->return_file >= 0)
-	{
-        uschar *message = US"\n\n*** Too much output - remainder discarded ***\n";
-        rc = Ustrlen(message);
-        if(write(addr->return_file, message, rc) != rc)
-          DEBUG(D_transport) debug_printf("Problem writing to return_file\n");
-	}
+        write(addr->return_file, message, Ustrlen(message));
       killpg(pid, SIGKILL);
       break;
       }
     }
-  (void)close(fd_out);
+  close(fd_out);
   _exit(0);
   }
 
-(void)close(fd_out);  /* Not used in this process */
+close(fd_out);  /* Not used in this process */
 
 
 /* Carrying on now with the main parent process. Attempt to write the message
@@ -899,20 +784,17 @@ the child process to 1 second. If the process at the far end of the pipe died
 without reading all of it, we expect an EPIPE error, which should be ignored.
 We used also to ignore WRITEINCOMPLETE but the writing function is now cleverer
 at handling OS where the death of a pipe doesn't give EPIPE immediately. See
-comments therein. */
+comments therein. This change made 04-Sep-98. Clean up this code in a year or
+so. */
 
 if (!written_ok)
   {
   if (errno == ETIMEDOUT)
-    {
-    addr->message = string_sprintf("%stimeout while writing to pipe",
-      transport_filter_timed_out? "transport filter " : "");
-    addr->transport_return = ob->timeout_defer? DEFER : FAIL;
     timeout = 1;
-    }
-  else if (errno == EPIPE)
+  else if (errno == EPIPE /* || errno == ERRNO_WRITEINCOMPLETE */ )
     {
-    debug_printf("transport error EPIPE ignored\n");
+    debug_printf("transport error %s ignored\n",
+      (errno == EPIPE)? "EPIPE" : "WRITEINCOMPLETE");
     }
   else
     {
@@ -923,9 +805,7 @@ if (!written_ok)
         string_sprintf("Failed to expand headers_add or headers_remove: %s",
           expand_string_message);
     else if (errno == ERRNO_FILTER_FAIL)
-      addr->message = string_sprintf("Transport filter process failed (%d)%s",
-      addr->more_errno,
-      (addr->more_errno == EX_EXECFAILED)? ": unable to execute command" : "");
+      addr->message = string_sprintf("Filter process failure");
     else if (errno == ERRNO_WRITEINCOMPLETE)
       addr->message = string_sprintf("Failed repeatedly to write data");
     else
@@ -940,9 +820,6 @@ above timed out. */
 
 if ((rc = child_close(pid, timeout)) != 0)
   {
-  uschar *tmsg = (addr->message == NULL)? US"" :
-    string_sprintf(" (preceded by %s)", addr->message);
-
   /* The process did not complete in time; kill its process group and fail
   the delivery. It appears to be necessary to kill the output process too, as
   otherwise it hangs on for some time if the actual pipe process is sleeping.
@@ -953,8 +830,9 @@ if ((rc = child_close(pid, timeout)) != 0)
     {
     killpg(pid, SIGKILL);
     kill(outpid, SIGKILL);
-    addr->transport_return = ob->timeout_defer? DEFER : FAIL;
-    addr->message = string_sprintf("pipe delivery process timed out%s", tmsg);
+    addr->transport_return = FAIL;
+    addr->message = string_sprintf("pipe delivery process timed out",
+      tblock->name);
     }
 
   /* Wait() failed. */
@@ -963,43 +841,22 @@ if ((rc = child_close(pid, timeout)) != 0)
     {
     addr->transport_return = PANIC;
     addr->message = string_sprintf("Wait() failed for child process of %s "
-      "transport: %s%s", tblock->name, strerror(errno), tmsg);
-    }
-
-  /* Since the transport_filter timed out we assume it has sent the child process
-  a malformed or incomplete data stream.  Kill off the child process
-  and prevent checking its exit status as it will has probably exited in error.
-  This prevents the transport_filter timeout message from getting overwritten
-  by the exit error which is not the cause of the problem. */
-
-  else if (transport_filter_timed_out)
-    {
-    killpg(pid, SIGKILL);
-    kill(outpid, SIGKILL);
+      "transport: %s", tblock->name, strerror(errno));
     }
 
   /* Either the process completed, but yielded a non-zero (necessarily
   positive) status, or the process was terminated by a signal (rc will contain
   the negation of the signal number). Treat killing by signal as failure unless
-  status is being ignored. By default, the message is bounced back, unless
-  freeze_signal is set, in which case it is frozen instead. */
+  status is being ignored. */
 
   else if (rc < 0)
     {
-    if (ob->freeze_signal)
-      {
-      addr->transport_return = DEFER;
-      addr->special_action = SPECIAL_FREEZE;
-      addr->message = string_sprintf("Child process of %s transport (running "
-        "command \"%s\") was terminated by signal %d (%s)%s", tblock->name, cmd,
-        -rc, os_strsignal(-rc), tmsg);
-      }
-    else if (!ob->ignore_status)
+    if (!ob->ignore_status)
       {
       addr->transport_return = FAIL;
       addr->message = string_sprintf("Child process of %s transport (running "
-        "command \"%s\") was terminated by signal %d (%s)%s", tblock->name, cmd,
-        -rc, os_strsignal(-rc), tmsg);
+        "command \"%s\") was terminated by signal %d (%s)", tblock->name, cmd,
+        -rc, os_strsignal(-rc));
       }
     }
 
@@ -1012,9 +869,9 @@ if ((rc = child_close(pid, timeout)) != 0)
   with the description
 
       EX_TEMPFAIL -- temporary failure, indicating something that
-         is not really an error.  In sendmail, this means
-         that a mailer (e.g.) could not create a connection,
-         and the request should be reattempted later.
+	 is not really an error.  In sendmail, this means
+	 that a mailer (e.g.) could not create a connection,
+	 and the request should be reattempted later.
 
   Based on this, we use exit code EX_TEMPFAIL as a default to mean "defer" when
   not ignoring the returned status. However, there is now an option that
@@ -1031,9 +888,9 @@ if ((rc = child_close(pid, timeout)) != 0)
   with the description
 
       EX_UNAVAILABLE -- A service is unavailable.  This can occur
-            if a support program or file does not exist.  This
-            can also be used as a catchall message when something
-            you wanted to do doesn't work, but you don't know why.
+	    if a support program or file does not exist.  This
+	    can also be used as a catchall message when something
+	    you wanted to do doesn't work, but you don't know why.
 
   However, this can be confused with a command that actually returns 69 because
   something *it* wanted is unavailable. At release 4.21, Exim was changed to
@@ -1051,8 +908,8 @@ if ((rc = child_close(pid, timeout)) != 0)
       {
       addr->transport_return = DEFER;
       addr->special_action = SPECIAL_FREEZE;
-      addr->message = string_sprintf("pipe process failed to exec \"%s\"%s",
-        cmd, tmsg);
+      addr->message = string_sprintf("pipe process failed to exec \"%s\"",
+        cmd);
       }
 
     /* Otherwise take action only if not ignoring status */
@@ -1071,14 +928,16 @@ if ((rc = child_close(pid, timeout)) != 0)
 
       else
         {
-        const uschar *s = ob->temp_errors;
+        uschar *s = ob->temp_errors;
         uschar *p;
         uschar buffer[64];
         int sep = 0;
 
         addr->transport_return = FAIL;
-        while ((p = string_nextinlist(&s,&sep,buffer,sizeof(buffer))))
+        while ((p = string_nextinlist(&s,&sep,buffer,sizeof(buffer))) != NULL)
+          {
           if (rc == Uatoi(p)) { addr->transport_return = DEFER; break; }
+          }
         }
 
       /* Ensure the message contains the expanded command and arguments. This
@@ -1124,13 +983,6 @@ if ((rc = child_close(pid, timeout)) != 0)
         if (quote)
           addr->message = string_cat(addr->message, &size, &ptr, US"\"", 1);
         }
-
-      /* Add previous filter timeout message, if present. */
-
-      if (*tmsg != 0)
-        addr->message = string_cat(addr->message, &size, &ptr, tmsg,
-          Ustrlen(tmsg));
-
       addr->message[ptr] = 0;  /* Ensure concatenated string terminated */
       }
     }
