@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2009 */
+/* Copyright (c) University of Cambridge 1995 - 2003 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -21,16 +21,11 @@ typedef struct {
   int   max_output;
   int   timeout;
   int   options;
-  BOOL  force_command;
   BOOL  freeze_exec_fail;
-  BOOL  freeze_signal;
   BOOL  ignore_status;
-  BOOL  permit_coredump;
   BOOL  restrict_to_path;
-  BOOL  timeout_defer;
   BOOL  use_shell;
   BOOL  use_bsmtp;
-  BOOL  use_classresources;
   BOOL  use_crlf;
 } pipe_transport_options_block;
 
